# $Id: Makefile.h,v 1.5 1993/12/30 23:52:17 gg502 Exp $

# Common definitions for all makefiles ... these can be overridden
# in each makefile by putting additional definitions below the
# include statement

# Architecture and site on which executable will be run
TARGET = ipsc
#TARGET = sun
#TARGET = delta
TARGET_SITE = pnl.gov

# Architecture and site on which compilation is done
COMP_ARCH = sun4
COMP_SITE = pnl.gov

SEQUENTIAL = FALSE

##########################################################
#                                                        #
# Should NOT need to modify below here unless porting to #
# a new machine or changing compiler options             #
#                                                        #
##########################################################

# Some default names for directories.  TOPDIR should point to the top
# of this program's directory tree.  It can be a fully qualified path
# or relative. 

BINDIR =	${TOPDIR}/${TARGET}
LIBDIR =	${TOPDIR}/${TARGET}
INCDIR =	${TOPDIR}/include

# Name of the program we are building, the object file containing the
# program statement (should also include block data, if any), and the
# executable (including path).  Also construct the full name for this
# library.

PROGRAM =	vle2
MAIN_OBJ = 	${PROGRAM}.o
EXECUTABLE =	${BINDIR}/${PROGRAM}
LIBRARY =	${LIBDIR}/lib${LIBNAME}.a

ifeq ($(TARGET),sun4)
#
# Sun running SunOS
#
         FC = f77
         CC = gcc
         AR = ar
     RANLIB = ranlib
      SHELL = /bin/sh

       FOPT = -g
       COPT = -g
   INCLUDES = 	-I. -I${INCDIR} -I${TOPDIR}/src/ints -I${TOPDIR}/src/global \
		-I${TOPDIR}/src/tcgmsg -I/msrc/apps/hpctools/include

   WARNINGS = -pedantic -Wall -Wshadow -Wpointer-arith -Wcast-qual \
		-Wwrite-strings
    DEFINES = -DASCII10_IN -DTCGMSG
     FFLAGS = $(FOPT) $(INCLUDES) $(DEFINES)
     CFLAGS = $(COPT) $(INCLUDES) $(DEFINES) $(WARNINGS)
    ARFLAGS = rcv

       LIBS =	-L${LIBDIR} -L/msrc/proj/mss/lib/${ATSYS} \
		-L/msrc/apps/lib -L/msrc/apps/hpctools/lib \
		-lmain -ltoy -lpar -lintegral_api -lints -ltape10 \
		-ldenmat_gen -ldenmat_mdc -lglobal -ltcgmsg \
		-lma -llapack -lblas -ldb -lblasx
  EXPLICITF = FALSE
endif

ifeq ($(TARGET),IPSC)
#
# DELTA/IPSC running NX
#
        FC = if77
        CC = icc
       CPP = cpp860
        AR = ar860

    RANLIB = echo
     SHELL = /bin/sh
   INSTALL = echo $@ is built
      FOPT = -g -Knoieee -node
      COPT = -g -Knoieee -node
   INCLUDES = 	-I. -I${INCDIR} -I${TOPDIR}/src/ints -I${TOPDIR}/src/global \
		-I${TOPDIR}/src/tcgmsg -I/msrc/proj/mss/include
   DEFINES = -DASCII10_IN
    FFLAGS = $(FOPT) $(INCLUDES) $(DEFINES)
    CFLAGS = $(COPT) $(INCLUDES) $(DEFINES)
   ARFLAGS = rcv
#
# The PGI linker seems to be somewhat braindamaged.  -L adds a
# directory to the beginning of the search path, so that if you
# specify -La -Lb -Lc the search path will be c:b:a:defaults!
#
       LIBS =	-L/msrc/proj/mss/lib/ipsc_pgi40 -L${LIBDIR} \
		-lmain -ltoy -lpar -lintegral_api -lints -ltape10 \
		-ldenmat_gen -ldenmat_mdc -lglobal -ltcgmsg \
		-lma -llapack -lkmath -ldb -lblasx

 EXPLICITF = FALSE
endif

ifeq ($(TARGET),ipsc.msrc)
#
# DELTA/IPSC running NX
#
        FC = if77
        CC = icc
       CPP = cpp860
        AR = ar860

    RANLIB = echo
     SHELL = /bin/sh
   INSTALL = echo $@ is built

# The global array library MUST be compiled with -Mreentrant
# The ints library seems to have a bug in it and must NOT be compiled
# with -Mreentrant.  The rest don't seem to care much at the moment.

ifeq (${LIBNAME},global)
      FOPT = -g -Knoieee -node -Mreentrant
      COPT = -g -Knoieee -node -Mreentrant
else
      FOPT = -g -Knoieee -node 
      COPT = -g -Knoieee -node 
endif

   INCLUDES = 	-I. -I${INCDIR} -I${TOPDIR}/src/ints -I${TOPDIR}/src/global \
		-I/msrc/proj/mss/include

# The TCGMSG flag means something different to the global array
# library than it does the rest of the code.

ifeq (${LIBNAME},global)
    DEFINES = -DIPSC -DNX -DNO_BCOPY
else
   DEFINES = -DIPSC -DASCII10_IN -DTCGMSG
endif

    FFLAGS = $(FOPT) $(INCLUDES) $(DEFINES)
    CFLAGS = $(COPT) $(INCLUDES) $(DEFINES)
   ARFLAGS = rcv
#
ifeq (${SEQUENTIAL},TRUE)
   GLOBAL_LIB = global_sequential
   TCGMSG_LIB = tcgmsg_sequential
else
   GLOBAL_LIB = global
   TCGMSG_LIB = tcgmsg
endif
#
# The PGI linker seems to be somewhat braindamaged.  -L adds a
# directory to the beginning of the search path, so that if you
# specify -La -Lb -Lc the search path will be c:b:a:defaults!

       LIBS =	-L/msrc/proj/mss/lib/ipsc_pgi40 -L${LIBDIR} \
		-lmain -ltoy -lpar -lintegral_api -lints -ltape10 \
		-ldenmat_gen -ldenmat_mdc -l${GLOBAL_LIB} -l${TCGMSG_LIB} \
		-lma -llapack -lkmath -ldb -lblasx

#       LIBS =	-L/msrc/proj/mss/lib/ipsc_pgi40 -L${LIBDIR} \
#		-lmain -ltoy -lpar -lintegral_api -lints -ltape10 \
#		-ldenmat_gen -ldenmat_mdc -l${GLOBAL_LIB} -l${TCGMSG_LIB} \
#		-lma -llapack -lkmath -ldb /msrc/home/gg502/src/blasx/ipsc/libblasx.a
#
 EXPLICITF = FALSE
endif
ifeq ($(TARGET),ipsc)
#
# DELTA/IPSC running NX
#
        FC = if77
        CC = icc
       CPP = cpp860
        AR = ar860

    RANLIB = echo
     SHELL = /bin/sh
   INSTALL = echo $@ is built

# The global array library MUST be compiled with -Mreentrant
# The ints library seems to have a bug in it and must NOT be compiled
# with -Mreentrant.  The rest don't seem to care much at the moment.

ifeq (${LIBNAME},global)
      FOPT = -g -Knoieee -node -Mreentrant
      COPT = -g -Knoieee -node -Mreentrant
else
      FOPT = -g -Knoieee -node 
      COPT = -g -Knoieee -node 
endif

ifeq (${COMP_SITE},pnl.gov)
   INCLUDES = 	-I. -I${INCDIR} -I${TOPDIR}/src/ints -I${TOPDIR}/src/global \
		-I/msrc/proj/mss/include
endif
ifeq (${COMP_SITE},dl.ac.uk)
   INCLUDES = 	-I. -I${INCDIR} -I${TOPDIR}/src/ints -I${TOPDIR}/src/global \
		-I/usr01/deb/include
endif

# The TCGMSG flag means something different to the global array
# library than it does the rest of the code.

ifeq (${LIBNAME},global)
    DEFINES = -DIPSC -DNX -DNO_BCOPY
else
   DEFINES = -DIPSC -DASCII10_IN -DTCGMSG
endif

    FFLAGS = $(FOPT) $(INCLUDES) $(DEFINES)
    CFLAGS = $(COPT) $(INCLUDES) $(DEFINES)
   ARFLAGS = rcv
#
ifeq (${SEQUENTIAL},TRUE)
   GLOBAL_LIB = global_sequential
   TCGMSG_LIB = tcgmsg_sequential
else
   GLOBAL_LIB = global
   TCGMSG_LIB = tcgmsg
endif
#
# The PGI linker seems to be somewhat braindamaged.  -L adds a
# directory to the beginning of the search path, so that if you
# specify -La -Lb -Lc the search path will be c:b:a:defaults!
#
ifeq (${COMP_SITE},pnl.gov)
       LIBS =	-L/msrc/proj/mss/lib/ipsc_pgi40 -L${LIBDIR} \
		-lmain -ltoy -lpar -lintegral_api -lints -ltape10 \
		-ldenmat_gen -ldenmat_mdc -l${GLOBAL_LIB} -l${TCGMSG_LIB} \
		-lma -llapack -lkmath -ldb -lblasx
endif 
ifeq (${COMP_SITE},dl.ac.uk)
      LIBS =	-L/usr01/deb/lib/ipsc_pgi40 -L${LIBDIR} \
		-lmain -ltoy -lpar -lintegral_api -lints -ltape10 \
		-ldenmat_gen -ldenmat_mdc -l${GLOBAL_LIB} -l${TCGMSG_LIB} \
		-lma -llapack -lkmath -ldb -lblasx
endif
 EXPLICITF = FALSE
endif


ifeq ($(TARGET),rs)
#
# IBM RS/6000 AIX 
#
        FC = xlf
        CC = cc
        AR = ar
    RANLIB = ranlib
   INSTALL = echo
     SHELL = /bin/sh
      FOPT = -g
      COPT = -g
   INCLUDES = 	-I. -I${INCDIR} -I${TOPDIR}/src/ints -I${TOPDIR}/src/global \
		-I${TOPDIR}/src/tcgmsg -I/msrc/proj/mss/include
   DEFINES = 
    FFLAGS = $(FOPT)
    CFLAGS = $(COPT) $(INCLUDES) $(DEFINES)
   ARFLAGS = rcv
       LIBS =	-L${LIBDIR} -L/msrc/proj/mss/lib/${ATSYS} \
		-L/msrc/apps/lib \
		-lmain -ltoy -lpar -lintegral_api -lints -ltape10 \
		-ldenmat_gen -ldenmat_mdc -lglobal -ltcgmsg \
		-lma -llapack -lblas -ldb -lblasx \
		-brename:.ma_set_auto_verify,.ma_set_auto_verify_ \
		-brename:.ma_set_hard_fail,.ma_set_hard_fail_ \
		-brename:.ma_initialize,.ma_initialize_ \
		-brename:.ma_sizeof,.ma_sizeof_ \
		-brename:.ma_pop_stack,.ma_pop_stack_ \
		-brename:.ma_push_get,.ma_push_get_ \
		-brename:.ma_get_index,.ma_get_index_ \
		-brename:.ma_push_stack,.ma_push_stack_ \
		-brename:.ma_print_stats,.ma_print_stats_ \
		-brename:.ma_allocate_heap,.ma_allocate_heap_ \
		-brename:.ma_free_heap,.ma_free_heap_ \
		-brename:mbc_byte,mbc_byte_ \
		-brename:mbc_int,mbc_int_ \
		-brename:mbc_log,mbc_log_ \
		-brename:mbc_real,mbc_real_ \
		-brename:mbc_dbl,mbc_dbl_ \
		-brename:mbc_scpl,mbc_scpl_ \
		-brename:mbc_dcpl,mbc_dcpl_
     EXTRA = 
 EXPLICITF = TRUE
endif

ifeq ($(EXPLICITF),TRUE)
#
# Needed on machines where FC does not preprocess .F files
# with CPP to get .f files
#
#.SUFFIXES:	.o .s .F .f .c

.F.o:	
	$(MAKE) $*.f
	$(FC) -c $(FFLAGS) $*.f
	/bin/rm -f $*.f

.F.f:	
	$(CPP) $(INCLUDES) $(DEFINES) $*.F | sed 's/^#/CPP#/' > $*.f

endif
