


      subroutine ga_dgemm(transa, transb, m, n, k, alpha, g_a, g_b, 
     $                   beta, g_c )
      implicit none
      Character*1        transa, transb
      Integer            m, n, k
      Double precision   alpha, beta
      Integer            g_a, g_b, g_c
#include "mafdecls.h"
#include "globalp.h"
c
c  GA_DGEMM  performs one of the matrix-matrix operations:
c     C := alpha*op( A )*op( B ) + beta*C,
c  where  op( X ) is one of
c     op( X ) = X   or   op( X ) = X',
c
c  alpha and beta are scalars, and A, B and C are matrices, with op( A )
c  an m by k matrix,  op( B )  a  k by n matrix and  C an m by n matrix.
c
c  On entry, TRANSA specifies the form of op( A ) to be used in
c  the matrix multiplication as follows:
c              transa = 'N' or 'n',  op( A ) = A.
c              transa = 'T' or 't',  op( A ) = A'.
c
c  M      - On entry,  M  specifies  the number  of rows  of the  matrix
c           op( A )  and of the  matrix  C.  M  must  be at least  zero.
c  N      - On entry,  N  specifies the number  of columns of the matrix
c           op( B ) and the number of columns of the matrix C. N must be
c           at least zero.
c  K      - On entry,  K  specifies  the number of columns of the matrix
c           op( A ) and the number of rows of the matrix op( B ). K must
c           be at least  zero.
c
c     J. Nieplocha 08.22.93
c
c~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      Integer info, ik, Cbase, elemIW
      Logical lsame, notA, notB
      External lsame, daxpy
      Integer nrowA, ncolA, nrowB, ncolB, elemA, elemB, elemW, elemC
      Double Precision one, zero
      Logical status

      Integer hmA, adrmA        !mapA
      Integer hmB, adrmB        !mapB

      Integer hA, adrA          !A
      Integer hB, adrB          !B
      Integer hC, adrC          !C

      Integer hRW, adrRW        ! real workspace
      Integer hIW, adrIW        ! integer workspace

      Integer myproc, proc, iproc
c
c*** 
      call ga_check_handle(g_a, 'ga_dgemm: a')
      call ga_check_handle(g_b, 'ga_dgemm: b')
      call ga_check_handle(g_c, 'ga_dgemm: c')

      zero  = 0d0
      one   = 1d0
      notA  = lsame( transa, 'N' )
      notB  = lsame( transb, 'N' )
      if( notA )then
         nrowA = m
         ncolA = k
      else
         nrowA = k
         ncolA = m
      end if
      if( notB )then
         nrowB = k
         ncolB = n
      else
         nrowB = n
         ncolB = k
      end if
*	print *,'in dgemm m, n, k ', n, m, k
*        call flush(6)
c
c***  Test the input parameters (hacked dgemm).
      info = 0
      if(      ( .not.nota                 ).and.
     $         ( .not.lsame( transa, 'C' ) ).and.
     $         ( .not.lsame( transa, 'T' ) )      )then
         info = 1
      else if( ( .not.notb                 ).and.
     $         ( .not.lsame( transb, 'C' ) ).and.
     $         ( .not.lsame( transb, 'T' ) )      )then
         info = 2
      else if( m  .lt.0               )then
         info = 3
      else if( n  .lt.0               )then
         info = 4
      else if( k  .lt.0               )then
         info = 5
      end if
      if( info.ne.0 )then
         call errquit('ga_dgemm: error in array dimensions no. ',info)
         return
      end if
c
c***  Quick return if possible.
      if( ( m.eq.0 ).or.( n.eq.0 ).or.
     $    ( ( ( alpha.eq.zero ).or.( k.eq.0 ) ).and.( beta.eq.one )))
     $   return
c
      myproc = me
      proc   = nserver
c
*	print *,'in dgemm1'
*        call flush(6)
c
c***  allocate space for mapA, A, mapB, B, C, and some workspace
c***  there is no need for mapC -- C shares its map with B
c***                               (the same # of columns) 
      status = ma_push_get(MT_INT,k,'mapA',hmA,adrmA)
      status = ma_push_get(MT_INT,n,'mapB',hmB,adrmB)
      elemA  = m*(k/proc + 1) 	! space for locally stored part of A
      elemB  = k*(n/proc + 1) 	! space for locally stored part of B
      elemC  = m*(n/proc + 1) 	! space for locally stored part of C

      elemW  = (m+1)*(k/proc + 1) + 2*n*m/proc+ 2*n ! amount of workspace
      elemIW  = 4*k+ k/proc + 2 + n/proc! amount of int  workspace
      elemW= 2*elemW ! safer
*      elemW= 2*elemW ! safer

      status = ma_push_get(MT_DBL,elemA,'A',hA,adrA)
      status = ma_push_get(MT_DBL,elemB,'B',hB,adrB)
      status = ma_push_get(MT_DBL,elemC,'C',hC,adrC)
      status = ma_push_get(MT_INT,elemIW,'IW',hIW,adrIW)
*      write(*,'(L)')status
      status = ma_push_get(MT_DBL,elemW,'RW',hRW,adrRW)
*       write(*,'(L)')status
c
*	print *,me,' in dgemm2 mp, pr, n,ncola,nrowa '
* 	write(*,'(4(i5,1h,))') myproc, proc,ncola,nrowa 
*      call flush(6)
c
c***  load g_a and g_b into A and B depending on transA & transB
c
      call ga_sync()
c
      call GA__LOAD(g_a, myproc, proc, notA, int_mb(adrmA),
     &                            dbl_mb(adrA), nrowA,ncolA)
      call GA__LOAD(g_b, myproc, proc, notB, int_mb(adrmB),
     &                            dbl_mb(adrB), nrowB,ncolB)
*	print *,'in dgemm3-multiplying: elemA, elemB, elemW'
* 	print *, elemA, elemB, elemW
c
c***  matrix multiply: B := A*B;                    A[n,m] &   B[n,n]
c     call printmat(m, dbl_mb(adrA), int_mb(adrmA),
c    &          k, dbl_mb(adrB), int_mb(adrmB))
c     call flush(6)
c     call printsingle(m,m,dbl_mb(adrA))
c     call printsingle(m,n,dbl_mb(adrB))
*	print *,me,' mult: cols_of_A, cols_of_B ',k,n
*      call flush(6)
      call mxm35( m, k, dbl_mb(adrA), int_mb(adrmA),
     &            n, dbl_mb(adrB), int_mb(adrmB),
     &            dbl_mb(adrC),
     &            int_mb(adrIW),dbl_mb(adrRW))

c     call mxm3(k, dbl_mb(adrA), int_mb(adrmA),
c    &          n, dbl_mb(adrB), int_mb(adrmB),
c    &             int_mb(adrIW),dbl_mb(adrRW))
c
c***  now implement C := alpha * A*B + beta * C and store it in g_c 
      do ik = 1, elemC
	 dbl_mb(adrC + ik -1) = dbl_mb(adrC + ik -1) * alpha 
      enddo
*	print *,me,' in dgemm5',m,n
*        call flush(6)
c      call printsingle(m,n,dbl_mb(adrC))
      Cbase = 0
      if(beta .eq. 0d0) then 		! we just overwrite ga_c
        do ik = 1, n
 	   iproc = int_mb(ik-1+adrmB)
	   if(iproc .eq. myproc) then
	      call GA_PUT(g_c,1, m, ik,ik,dbl_mb(adrC+Cbase),1) 
	      Cbase = Cbase + m
	   endif
        enddo
      else            			! ga_c += beta*B
        do ik = 1, n
 	   iproc = int_mb(ik-1+adrmB)
	   if(iproc .eq. myproc) then
	      call GA_GET(g_c, 1, m, ik,ik, dbl_mb(adrA),1) ! A is used as temp
	      call DAXPY(m,beta,dbl_mb(adrA),1,dbl_mb(adrC+Cbase),1)
	      call GA_PUT(g_c, 1, m, ik,ik,dbl_mb(adrC+Cbase),1) 
	      Cbase = Cbase + m
	   endif
        enddo
      endif
c
      call ga_sync()
c
      status = ma_pop_stack(hRW) 
      status = ma_pop_stack(hIW) 
      status = ma_pop_stack(hC) 
      status = ma_pop_stack(hB) 
      status = ma_pop_stack(hA) 
      status = ma_pop_stack(hmB) 
      status = ma_pop_stack(hmA) 
      END
 
 
