      subroutine ga_iacc(g_a, ilo, ihi, jlo, jhi, buf, ld, alpha)
      implicit none
#include "mafdecls.h"
#include "globalp.h"
c
      integer g_a, ilo, ihi, jlo, jhi, ld
      integer buf(1:ld, 1:*), alpha
c
      integer ip_lo, ip_hi, jp_lo, jp_hi, ip, jp, iproc
      integer ilo_acc, ihi_acc, jlo_acc, jhi_acc
      integer ilo_ugh, ihi_ugh, jlo_ugh, jhi_ugh, ilimit, jlimit
c
#ifdef GA_TRACE
      call trace_stime
#endif
c
      call ga_check_handle(g_a, 'ga_iacc')
      if (ttyp(g_a) .ne. MT_INT) then
         call errquit(' ga_iacc: only supported for integers', g_a)
      else if (ilo.le.0 .or. ihi.gt.dims(1,g_a) .or.
     $         jlo.le.0 .or. jhi.gt.dims(2,g_a)) then
         call errquit(' ga_iacc: indices out of range ', g_a)
      endif
c
c     Map index ranges on to virtual processor array
c
      ip_lo = (ilo-1)/chnk(1,g_a)
      ip_hi = (ihi-1)/chnk(1,g_a)
      jp_lo = (jlo-1)/chnk(2,g_a)
      jp_hi = (jhi-1)/chnk(2,g_a)
c
c     Loop thru virtual processes that hold data
c
      do jp = jp_lo, jp_hi
         do ip = ip_lo, ip_hi
c
c     Currently the map from virtual processor array to real
c     processors is very simple and ignores topology
c
            iproc = ip + jp*pdim(1,g_a)
c
c     Figure out the range of data required from this processor
c
            ilo_acc = max(ilo, mapc(1,iproc,g_a))
            ihi_acc = min(ihi, mapc(2,iproc,g_a))
            jlo_acc = max(jlo, mapc(3,iproc,g_a))
            jhi_acc = min(jhi, mapc(4,iproc,g_a))
c
            if (iproc .eq. me .and. oserver) then
               call ga_iacc_local(g_a, ilo_acc, ihi_acc, jlo_acc,
     $              jhi_acc, buf(ilo_acc-ilo+1,jlo_acc-jlo+1), ld,
     $              alpha)
            else
c
c     Process data in chunks defined by the available buffer space
c     for incoming/outgoing messages. Ugh, loops within loops within ...
c
               ilimit = min(itmpsize-1, ihi_acc-ilo_acc+1)
               jlimit = min((itmpsize-1)/ilimit, jhi_acc-jlo_acc+1)
               do jlo_ugh = jlo_acc, jhi_acc, jlimit
                  jhi_ugh = min(jhi_acc, jlo_ugh+jlimit-1)
                  do ilo_ugh = ilo_acc, ihi_acc, ilimit
                     ihi_ugh = min(ihi_acc, ilo_ugh+ilimit-1)
                     call ga_iacc_remote(g_a, ilo_ugh, ihi_ugh, jlo_ugh,
     $                    jhi_ugh, buf(ilo_ugh-ilo+1,jlo_ugh-jlo+1), ld,
     $                    alpha, iproc)
                  enddo
               enddo
            endif
            Call FFFlush(0)
         enddo
      enddo
c
#ifdef GA_TRACE
      call trace_etime
      call trace_genrec(g_a, ilo, ihi, jlo, jhi, GA_OP_IACC)
#endif
c
      end
      subroutine ga_iacc_local(g_a, ilo, ihi, jlo, jhi, buf, ld, alpha)
      implicit none
#include "mafdecls.h"
#include "global.h"
#include "globalp.h"
c
      integer g_a, ilo, ihi, jlo, jhi, ld
      integer buf(1:ld, 1:*), alpha
c
      integer i, j, gbase, iproc, oldmask
c
c     First do usual paranoia check on arguments and data structure
c
#ifdef DATA_SERVER
      iproc = me - nserver
#else
      iproc = me
#endif
c
      call ga_check_handle(g_a, 'ga_iacc_local')
      if (ilo.lt.mapc(1,iproc,g_a) .or.
     $         ihi.gt.mapc(2,iproc,g_a) .or.
     $         jlo.lt.mapc(3,iproc,g_a) .or.
     $         jhi.gt.mapc(4,iproc,g_a)) then
         call errquit(' ga_iacc_local: local indices out of range ',
     $      g_a)
      else if (ldim(1,g_a)*ldim(2,g_a).ne.0 .and. indx(g_a).eq.0) then
         call errquit(' ga_iacc_local: null pointer? ', g_a)
      else
c
c     All seems OK ... inside a critical section acc the data
c
         oldmask = ga_mask(1)
         gbase = indx(g_a) - mapc(1,iproc,g_a) +
     $        (jlo-mapc(3,iproc,g_a))*ldim(1,g_a) 
         do j = jlo, jhi
            do i = ilo, ihi
               Int_mb(gbase + i) = Int_mb(gbase + i) +
     $              alpha*buf(i-ilo+1,j-jlo+1)
            enddo
            gbase = gbase + ldim(1,g_a)
         enddo
         oldmask = ga_mask(oldmask)
      endif
c
      end
      subroutine ga_iacc_remote(g_a, ilo, ihi, jlo, jhi, buf, ld, alpha,
     $     iproc)
      implicit none
#include "mafdecls.h"
#include "globalp.h"
#include "global.h"
#include "tcgmsg.h"
c
      integer g_a, ilo, ihi, jlo, jhi, ld, iproc
      Integer buf(1:ld, 1:*), alpha
c
      integer i, j, ldtmp, len, to, oldmask, firstmask
c
c     First do usual paranoia check on arguments and data structure
c
      call ga_check_handle(g_a, 'ga_iacc_remote')
      if (iproc.lt.0 .or. iproc.ge.nserver) then
         call errquit(' ga_iacc_remote: iproc out of range ', iproc)
      else if (ilo.lt.mapc(1,iproc,g_a) .or.
     $         ihi.gt.mapc(2,iproc,g_a) .or.
     $         jlo.lt.mapc(3,iproc,g_a) .or.
     $         jhi.gt.mapc(4,iproc,g_a)) then
         call errquit(' ga_iacc_remote: invalid index range ', g_a)
      else if ((ihi-ilo+1)*(jhi-jlo+1)+1.gt.itmpsize) then
         call errquit(' ga_iacc_remote: internal buffer too small',
     $        (ihi-ilo+1)*(jhi-jlo+1))
      else
c
c     All seems OK ... do a remote request (breaking cycles)
c
         if (me .lt. iproc) firstmask = ga_mask(1)
         call ga_snd_req(g_a, ilo, ihi, jlo, jhi, me, ttyp(g_a),
     $        GA_OP_IACC, iproc)
c
c     Copy data to temporary buffer (don't forget alpha)
c     
         oldmask = ga_mask(1)
         ldtmp = ihi-ilo+1
         do j = jlo, jhi
            do i = ilo, ihi
               itmp(i-ilo+1 + (j-jlo)*ldtmp) = buf(i-ilo+1,j-jlo+1) 
            enddo
         enddo
         len = (ihi-ilo+1)*(jhi-jlo+1) + 1
         itmp(len) = alpha
c
         to = iproc
#ifdef DATA_SERVER
         to = to + nserver
#endif
         call ga_snd_msg(GA_TYPE_IACC, itmp, mitob(len), to)
c
         oldmask = ga_mask(oldmask)
         if (me .lt. iproc) firstmask = ga_mask(firstmask)
c
      endif
c     
      end
