      subroutine ga_diag(g_a, g_s, g_v, eval)
      implicit none
#include "mafdecls.h"
#include "globalp.h"
      integer g_a               ! Matrix to diagonalize
      integer g_s               ! Metric
      integer g_v               ! Global matrix to return evecs
      double precision eval(*)  ! Local array to return evals
c
c
c     Solve the generalized eigen-value problem returning
c     all eigen-vectors and values in ascending order
c
c     The input matrices are not destroyed
c
c     Proc - # of processors
c     PANs - panel size in columns
c     n    - problem size
c
c     J. Nieplocha 08.22.93
c
c~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

      Integer iproc, PAN, istart, iend, k, n, nPAN, Proc, myproc
      Integer mypanel, myelem, mycol, elemZ, ipan, ik, info
      Logical status

      Integer hmA, adrmA        !mapA
      Integer hmB, adrmB        !mapB
      Integer hmZ, adrmZ        !mapZ

      Integer hA, adrA          !A
      Integer hB, adrB          !B
      Integer hZ, adrZ          !Z

      Integer isize, rsize, ptr_size
      Integer hIS, adrIS
      Integer hSC, adrSC
      Integer hIP, adrIP

      Double Precision DRAND
      External MEMREQ, PDSYGV, DRAND
      Intrinsic MOD

      Integer ELEM
      ELEM(istart,iend,n)=((iend-istart+1)*(2*n-istart-iend+2))/2
*      write(6,*)me,  'in diag'
c
c***  
      call ga_check_handle(g_a, 'ga_diag: a')
      call ga_check_handle(g_s, 'ga_diag: s')
      call ga_check_handle(g_v, 'ga_diag: v')
      
      if (dims(1,g_a) .ne. dims(2,g_a)) then
         call errquit('ga_diag: can only diag square matrix', 0)
      endif
      if (dims(1,g_s) .ne. dims(2,g_s)) then
         call errquit('ga_diag: can only diag square matrix', 0)
      endif
      if (dims(1,g_v) .ne. dims(2,g_v)) then
         call errquit('ga_diag: can only diag square matrix', 0)
      endif
c
      n = dims(1,g_a)
      myproc = me
      proc   = nserver
      PAN = 1  !panel size

      nPAN = n/PAN
*      write(*,*)'proc,myproc,pan,npan,n'
*      write(*,*)proc,myproc,pan,npan,n
c
c***  allocate map arrays
      status = ma_push_get(MT_INT,n,'mapA',hmA,adrmA)
*      write(6,*)status
      status = ma_push_get(MT_INT,n,'mapB',hmB,adrmB)
*      write(6,*)status
      status = ma_push_get(MT_INT,n,'mapZ',hmZ,adrmZ)
*      write(6,*)status
*      call flush(6)

c
c...  determine distribution of A,B, and Z  .................
c
      mypanel = 0 	! number of panels at given processor
      mycol   = 0 	! number of columns at given processor
      myelem  = 0 	! number of elements at given processor
c
c***  allocate first half of the panels - forwards
      do ipan = 1, nPAN/2 
	  iproc = MOD(ipan -1, Proc)
	  istart = (ipan - 1)*PAN + 1
	  iend   = istart + PAN - 1
	  do k = istart, iend
	     int_mb(-1+adrmA+k) = iproc
	     int_mb(-1+adrmB+k) = iproc
	     int_mb(-1+adrmZ+k) = iproc
          enddo
	  if(iproc .eq. myproc)then
	 	myelem = myelem + ELEM(istart,iend,n)
		mypanel = mypanel+1	
	  endif
      enddo	   
c     write(*,*)'1, mypanel, myelem'
c      write(*,*) mypanel, myelem
c      call flush(6)
c
c***  allocate second half of the panels - backwards
      ik = 1 	!ik is used to forward number processors
      do ipan =   nPAN-1, nPAN/2 +1 , -1
          ik = ik+1
	  iproc = MOD(ik -1, Proc)
	  istart = (ipan - 1)*PAN + 1
	  iend   = istart + PAN - 1
	  do k = istart, iend
	     int_mb(-1+adrmA+k) = iproc
	     int_mb(-1+adrmB+k) = iproc
	     int_mb(-1+adrmZ+k) = iproc
          enddo
	  if(iproc .eq. myproc)then
	 	myelem = myelem + ELEM(istart,iend,n)
		mypanel = mypanel+1	
	  endif
      enddo	   
c     write(*,*)'2, mypanel, myelem'
c     write(*,*) mypanel, myelem
c     call flush(6)
c
c***  actually, there is one more panel left for the mismatch 
      iproc  = 0 	!processor 0 gets the mismatch panel
      istart = (nPAN - 1)*PAN + 1
      iend   = n
      do k = istart, iend
          int_mb(-1+adrmA+k) = iproc
          int_mb(-1+adrmB+k) = iproc
          int_mb(-1+adrmZ+k) = iproc
      enddo
      mycol = mypanel * PAN
      if(iproc .eq. myproc)then
 	  myelem = myelem + ELEM(istart,iend,n)
	  mycol  = mycol + iend - istart + 1	
      endif
      elemZ = mycol * n
c     write(*,*)'3, mypanel, myelem'
c     write(*,*) mypanel, myelem
c     call flush(6)
c............................................................ 
c
c***  allocate A,B, and Z arrays
      status = ma_push_get(MT_DBL,myelem,'A',hA,adrA)
      status = ma_push_get(MT_DBL,myelem,'B',hB,adrB)
      status = ma_push_get(MT_DBL,elemZ,'Z',hZ,adrZ)

c     write(6,*) me, 'myelem, elemZ'
c     write(6,'(i8, 2x, i8)')   myelem, elemZ
c     call flush(6)
c
c***  load the Fock (A) and overlap (B) matrices 
      call ga_sync()
      istart = 0
      do k = 1, n
         if( int_mb(-1+adrmA+k) .eq. myproc) then
		myelem = n-k+1
		call GA_GET(g_a,k,n,k,k,dbl_mb(adrA+istart),1)
		call GA_GET(g_s,k,n,k,k,dbl_mb(adrB+istart),1)
		istart = istart + myelem
	 endif
      enddo
c     write(6,*)'fock matrix'
c     write(6,*)(dbl_mb(adrA+k-1),k=1,istart)
c     write(6,*)'overlap matrix'
c     write(6,*)(dbl_mb(adrB+k-1),k=1,istart)
c     write(6,*)  'calling memreq'
*       call	flush(6)
c
c***  allocate some memory for eigensolver 
      status = ma_push_get(MT_INT,n,'iscratch',hIS,adrIS)
      call MEMREQ(0,n,int_mb(adrmA),int_mb(adrmB),int_mb(adrmZ),
     &                  isize, rsize, ptr_size,int_mb(adrIS))
*      write(6,*) 'me, n, isize, ptr_size, rsize'
*      write(6,*) me, n, isize, ptr_size, rsize
*      call	flush(6)
      status = ma_pop_stack(hIS)
      if(.not. ma_push_get(MT_INT,isize,'iscratch',hIS,adrIS))
     &                 call errquit('eig allocator',1)
      if(.not. ma_push_get(MT_DBL,rsize,'scratch',hSC,adrSC))
     &                 call errquit('eig allocator',2)
      if(.not. ma_push_get(MT_DBL,ptr_size,'iptr',hIP,adrIP))
     &                 call errquit('eig allocator',3)
c
c***  now, call eigensolver
*      write(6,*) me, ' calling PDSYGV'
*      call	flush(6)
      call ga_sync()
      call PDSPGV( 1, n, dbl_mb(adrA), int_mb(adrmA),
     &                dbl_mb(adrB), int_mb(adrmB),  
     &                dbl_mb(adrZ), int_mb(adrmZ),  
     &                eval,  
     &                int_mb(adrIS), isize,  
     &                dbl_mb(adrIP), ptr_size,  
     &                dbl_mb(adrSC), rsize, info)  
c
*      print *,me, 'after PDSYGV'
c     write(6,*)(eval(k),k=1,n)
c     call	flush(6)
      call ga_sync()
c***  store the eigenvector matrix in global array g_v
      istart = 0
      do k = 1, n
        if( int_mb(-1+adrmZ+k) .eq. myproc) then
c   	        write(*,'(10e8.2)')(dbl_mb(adrZ+istart+ik),ik=0,n-1)
		call GA_PUT(g_v,1,n,k,k,dbl_mb(adrZ+istart),1)
		istart = istart + n
	 endif
      enddo
      call ga_sync()
c
c***  deallocate work arrays 
      status = ma_pop_stack(hIP) 
      status = ma_pop_stack(hSC) 
      status = ma_pop_stack(hIS) 
      status = ma_pop_stack(hZ) 
      status = ma_pop_stack(hB) 
      status = ma_pop_stack(hA) 
      status = ma_pop_stack(hmZ) 
      status = ma_pop_stack(hmB) 
      status = ma_pop_stack(hmA) 
      END

      subroutine ga__load(g_a, myproc,proc, ntrans, mapA, A,nrow,ncol)
      Implicit none
#include "globalp.h"
      Logical ntrans
      Integer g_a, proc, myproc, nrow, ncol
      Integer mapA(*)
      Double Precision A(*), b(1000)
c
c     This routine copies a part of the global distributed array g_a
c     into the column distributed array A.
c     Regular column wraparound distribution is used. mapA is determined.
c
c     J. Nieplocha 08.22.93
c
      Integer k, iproc, base,i,len 
      Integer Mod
      Intrinsic Mod
c
      if(ntrans)then
         len = ncol
      else
         len = nrow
      endif
      base = 0
      do k = 1, len 
	iproc   = Mod(k-1,proc)
        mapA(k) = iproc 
        if(iproc .eq. myproc) then
	      if(ntrans) then 
c       write(*,'( 1hN,i2,L,i2,i2)') k,ntrans,nrow,ncol
                 call GA_GET(g_a, 1, nrow, k,k, A(1+base*nrow),1) 
	      else
c       write(*,'( 1hT,i2,L,i2,i2)') k,ntrans,nrow,ncol
		 call GA_GET(g_a, k, k, 1, ncol, A(1+base*ncol),1) ! transpose
	      endif
	      base = base +  1
	endif
      enddo
c      write(*,'( L,i6,i6)') ntrans,nrow,ncol
*        call flush(6)
       if(ntrans) then 
c          call printsingle(nrow,base,A)
       else
c          call printsingle(ncol,base,A)
       endif
      END


      subroutine ga_symmetrize(g_a) 
c
c     Symmetrizes matrix A:  A := .5 * (A+A')
c     diag(A) remains unchanged
c 
c     J. Nieplocha 08.22.93
c
      implicit none
#include "mafdecls.h" 
#include "globalp.h"
      integer g_a               ! Matrix to symmetrize 
c
      Integer i 
      Double Precision alpha   
      Integer myproc, proc 
      Integer ilo, ihi, jlo, jhi, myelem, nrow,ncol
      Logical status, have_data

      Integer hA, adrA          !A
      Integer hB, adrB          !B

c
c***  check environment
c
      myproc = me 
      proc   = nserver 
      alpha = 0.5
c  
      call ga_check_handle(g_a, 'ga_symm: a')
      if (dims(1,g_a) .ne. dims(2,g_a)) then
         call errquit('ga_sym: can only sym square matrix', 0)
      endif
c
c***  Find the local distribution
      call ga_distribution(g_a, me, ilo, ihi, jlo, jhi, i)
      have_data = ihi.ge.0 .and. jhi.ge.0

      call ga_sync()
      if (have_data) then
         nrow = ihi-ilo+1
         ncol = jhi-jlo+1
         myelem = nrow * ncol
         status = ma_push_get(MT_DBL,myelem,'A',hA,adrA)
         status = ma_push_get(MT_DBL,myelem,'B',hB,adrB)
         
         call GA_GET(g_a, ilo, ihi, jlo, jhi, dbl_mb(adrA),nrow) 
         call GA_GET(g_a, jlo, jhi, ilo, ihi, dbl_mb(adrB),ncol) 
         call ga__add(nrow, ncol, dbl_mb(adrA), dbl_mb(adrB),alpha)
      endif

      call ga_sync()
      
      if(have_data) then
         call GA_PUT(g_a, ilo, ihi, jlo, jhi, dbl_mb(adrA),nrow) 
         status = ma_pop_stack(hB) 
         status = ma_pop_stack(hA) 
      endif

      call ga_sync()

      end

      subroutine ga__add(nrow, ncol, A, B, alpha)
c
c     A := alpha * (A + B)
c
c     J. Nieplocha 08.22.93
c
      implicit none
      Integer nrow, ncol
      Double Precision A(nrow,ncol), B(ncol,nrow), alpha
      Integer i,j
c
      do j = 1, ncol
         do i = 1, nrow
		a(i,j) = alpha*(a(i,j)+b(j,i))
         enddo
      enddo
      END
c
c
c----------------------------
      subroutine printsingle(n,m, A)
      implicit none
	integer n,m
	double precision a(n,*)
	integer i,j
      do i =1,n 
	write(6,'(10e8.2)')(a(i,j),j=1,m)
      enddo
      call flush(6)
      end


      subroutine printmat(n, A, mA,
     &          m, B, mB)
      implicit none
	integer n,m,mA(*),mB(*)
	double precision a(n,*), b(n,*)
c
	integer i,j
	print *,'n=', n,' m=',m
	print *,'maps of A & B'
	write(*,'(10i4)')(ma(i),i=1,n)
	write(*,'(10i4)')(mb(i),i=1,m)
	print *,'A'
        do i=1,n
	   write(6,'(10e8.2)')(a(i,j),j=1,n)
        enddo
	print *,'B'
        do i=1,n
	   write(6,'(10e8.2)')(b(i,j),j=1,m)
        enddo
        call flush(6)
	end
