      logical function ga_create(type, dim1, dim2, array_name,
     $     distribution, chunksize, g_a)
      implicit none
#include "mafdecls.h"
#include "globalp.h"
c
      character*(*) array_name, distribution
      integer type, dim1, dim2, chunksize, g_a, handle
c
c      array_name   - a unique character string [input]
c      type         - MA type [input]
c      dim1/2       - array(dim1,dim2) as in FORTRAN [input]
c      distribution - currently only 'BLOCK' [input]
c      chunksize    - min. size of blocks [input]
c      g_a       - integer handle for future references
c
      if (type .ne. MT_DBL) then
         write(6,*) ' ga_create: type not yet supported ', array_name
         ga_create = .false.
         g_a = -1
      else if (n_global_array .eq. max_global_array) then
         write(6,*) ' ga_create: too many arrays ', array_name
         ga_create = .false.
         g_a = -1
      else if (dim1 .le. 0 .or. dim2 .le. 0) then
         write(6,*) ' ga_create: array dimension is invalid ',array_name
         ga_create = .false.
         g_a = -1
      else if (.not. ma_allocate_heap(MT_DBL, dim1*dim2, array_name, 
     $        handle)) then
c
c     allocation can also be on the stack as create/destroy is sychronous
c
         write(6,*) ' failed to allocate memory for ', array_name
         ga_create = .false.
         g_a = -1
      else
         n_global_array = n_global_array + 1
         name(n_global_array)   = array_name
         dims(1,n_global_array) = dim1
         dims(2,n_global_array) = dim2
         dist(n_global_array)   = distribution
         ttyp(n_global_array)   = type
         actv(n_global_array)   = .true.
         chnk(n_global_array)   = chunksize
         hand(n_global_array)   = handle
         write(6,1) array_name, dim1, dim2, n_global_array
 1       format(' ga_create: "',a20,'"(',i5,',',i5,') has handle ',i2)
         ga_create = .true.
         g_a = n_global_array
      endif
c
      end
      logical function ga_destroy(g_a)
      implicit none
#include "mafdecls.h"
#include "globalp.h"
c
      integer g_a
c
      if (g_a .le.0 .or. g_a.gt.n_global_array) then
         write(6,*) ' ga_destroy: array handle is invalid ', g_a
         ga_destroy = .false.
      else if (.not. actv(g_a)) then
         write(6,*) ' ga_destroy: array has been destroyed already',
     $        g_a
         ga_destroy = .false.
      else
         if (.not. ma_free_heap(hand(g_a))) then
            write(6,*) ' ga_destroy: failed to free memory?', g_a
            ga_destroy = .false.
         else
            ga_destroy = .true.
            actv(g_a) = .false.
         endif
      endif
c
      end
      subroutine ga_get(g_a, ilo, ihi, jlo, jhi, buf, ld)
      implicit none
#include "mafdecls.h"
#include "globalp.h"
c
      integer g_a, ilo, ihi, jlo, jhi, ld
      double precision buf(1:ld, 1:*)
c
      integer i, j, ibase
c
      if (g_a.le.0 .or. g_a.gt.n_global_array) then
         call errquit(' ga_get: invalid array handle ', g_a)
      else if (.not. ma_get_index(hand(g_a), ibase)) then
         call errquit(' ga_get: unable to map array handle ?', g_a)
      else if (ilo.le.0 .or. ihi.gt.dims(1,g_a) .or.
     $         jlo.le.0 .or. jhi.gt.dims(2,g_a)) then
         call errquit(' ga_get: indices out of range ', g_a)
      else
         do j = jlo, jhi
            do i = ilo, ihi
               buf(i-ilo+1,j-jlo+1) =
     $              dbl_mb(ibase + (i-1) + (j-1)*dims(1,g_a))
*               write(6,*) ' get ', i, j, i + (j-1)*dims(1,g_a),
*     $              buf(i-ilo+1,j-jlo+1)
            enddo
         enddo
      endif
c
      end
      subroutine ga_put(g_a, ilo, ihi, jlo, jhi, buf, ld)
      implicit none
#include "mafdecls.h"
#include "globalp.h"
c
      integer g_a, ilo, ihi, jlo, jhi, ld
      double precision buf(1:ld, 1:*)
c
      integer i, j, ibase
c
      if (g_a.le.0 .or. g_a.gt.n_global_array) then
         call errquit(' ga_put: invalid array handle ', g_a)
      else if (.not. ma_get_index(hand(g_a), ibase)) then
         call errquit(' ga_put: unable to map array handle ?', g_a)
      else if (ilo.le.0 .or. ihi.gt.dims(1,g_a) .or.
     $         jlo.le.0 .or. jhi.gt.dims(2,g_a)) then
         call errquit(' ga_put: indices out of range', g_a)
      else
         do j = jlo, jhi
            do i = ilo, ihi
*               write(6,*) ' put ', i, j, i + (j-1)*dims(1,g_a),
*     $              buf(i-ilo+1,j-jlo+1)
               dbl_mb(ibase+i-1 + (j-1)*dims(1,g_a)) = 
     $              buf(i-ilo+1,j-jlo+1)
            enddo
         enddo
      endif
c
      end
      subroutine ga_update(g_a, ilo, ihi, jlo, jhi, buf, ld,
     $     alpha)
      implicit none
#include "mafdecls.h"
#include "globalp.h"
c
      integer g_a, ilo, ihi, jlo, jhi, ld
      double precision buf(1:ld, 1:*), alpha
c
      integer i, j, ij, ibase
c
      if (g_a.le.0 .or. g_a.gt.n_global_array) then
         call errquit(' ga_update: invalid array handle ', g_a)
      else if (.not. ma_get_index(hand(g_a), ibase)) then
         call errquit(' ga_update: unable to map array handle ?', g_a)
      else if (ilo.le.0 .or. ihi.gt.dims(1,g_a) .or.
     $         jlo.le.0 .or. jhi.gt.dims(2,g_a)) then
         call errquit(' ga_upd: indices out of range ', g_a)
      else
         do j = jlo, jhi
            do i = ilo, ihi
*               write(6,*) ' upd ', i, j, i + (j-1)*dims(1,g_a),
*     $              buf(i-ilo+1,j-jlo+1)
               ij = ibase + (i-1) + (j-1)*dims(1,g_a)
               dbl_mb(ij) = dbl_mb(ij) + alpha * buf(i-ilo+1,j-jlo+1)
            enddo
         enddo
      endif
c
      end
      subroutine ga_inquire(g_a, array_name, type, dim1, dim2)
      implicit none
#include "mafdecls.h"
#include "globalp.h"
c
      integer g_a, dim1, dim2, type
      character*(*) array_name
c      
c      g_a     - array handle [input]
c      array_name - name given to create [output]
c      type       - type given to create [output]
c      dim1/2     - array dimensions [output]
c
      if (g_a.le.0 .or. g_a.gt.n_global_array) then
         call errquit(' ga_inquire: invalid array handle ', g_a)
      else
         array_name = name(g_a)
         type       = ttyp(g_a)
         dim1       = dims(1,g_a)
         dim2       = dims(2,g_a)
      endif
c
      end
      subroutine ga_distribution(g_a, iproc, ilo, ihi, jlo, jhi,
     $     ma_index)   
      implicit none
#include "mafdecls.h"
#include "globalp.h"
c
      integer g_a, iproc, ilo, ihi, jlo, jhi, ma_index
c
c      g_a             - array handle [input]
c      iproc           - process number [input]
c      ilo/ihi/jlo/jhi - range held by process iproc [output]
c      ma_index        - if (iproc == this process) returns
c                        MA index in appropriately typed base array 
c                        of the data
c
      if (g_a.le.0 .or. g_a.gt.n_global_array) then
         call errquit(' ga_distribution: invalid array handle ', g_a)
      else if (iproc .ne. 0) then
         call errquit(' ga_distribution: iproc is not zero ', iproc)
      else if (.not. ma_get_index(g_a, ma_index)) then
         call errquit(' ga_distribution: unable to map array handle ?',
     $        g_a)
      else
         ilo = 1
         jlo = 1
         ihi = dims(1,g_a)
         jhi = dims(2,g_a)
      endif
c
      end
      double precision function ga_trace(g_a, g_b)
      implicit none
#include "mafdecls.h"
#include "globalp.h"
c
      integer g_a, g_b
      integer abase, bbase
      double precision ddot
      external ddot
c
      ga_trace = 0.0d0
      if (g_a.le.0 .or. g_a.gt.n_global_array) then
         call errquit('ga_trace: invalid array handle ', g_a)
      else if (g_b.le.0 .or. g_b.gt.n_global_array) then
         call errquit('ga_trace: invalid array handle ', g_b)
      else if (.not. ma_get_index(hand(g_a), abase)) then
         call errquit('ga_trace: unable to map array handle ?', g_a)
      else if (.not. ma_get_index(hand(g_b), bbase)) then
         call errquit('ga_trace: unable to map array handle ?', g_b)
      else if (dims(1,g_a).ne.dims(1,g_b) .or.
     $         dims(2,g_a).ne.dims(2,g_b)) then
         call errquit('ga_trace: dimensions do not match', 0)
      else
         ga_trace = ddot(dims(1,g_a)*dims(2,g_a),
     $        dbl_mb(abase), 1, dbl_mb(bbase), 1)
      endif
c
      end
      subroutine ga_print(g_a)
      implicit none
#include "mafdecls.h"
#include "globalp.h"
c
      integer g_a
      integer abase
c
      if (g_a.le.0 .or. g_a.gt.n_global_array) then
         call errquit('ga_trace: invalid array handle ', g_a)
      else if (.not. ma_get_index(hand(g_a), abase)) then
         call errquit('ga_trace: unable to map array handle ?', g_a)
      else
         write(6,*) name(g_a)
         call output(dbl_mb(abase), 1, dims(1,g_a), 1, dims(1,g_a),
     $        dims(1,g_a), dims(1,g_a), 1)
      endif
c
      end
      subroutine ga_diag(g_a, g_s, g_v, evals)
      implicit none
#include "mafdecls.h"
#include "globalp.h"
      integer g_a               ! Matrix to diagonalize
      integer g_s               ! Metric
      integer g_v               ! Global matrix to return evecs
      double precision evals(*) ! Local array to return evals
c
      integer abase, sbase, vbase, l_fv1, l_fv2, k_fv1, k_fv2, ierr
      logical status
c
c     Solve the generalized eigen-value problem returning
c     all eigen-vectors and values in ascending order
c
c     The input matrices may be destroyed
c
      if (g_a.le.0 .or. g_a.gt.n_global_array) then
         call errquit('ga_diag: invalid array handle for a', g_a)
      else if (.not. ma_get_index(hand(g_a), abase)) then
         call errquit('ga_diag: unable to map array handle a?', g_a)
      else if (dims(1,g_a) .ne. dims(2,g_a)) then
         call errquit('ga_diag: can only diag square matrix', 0)
      endif
      if (g_s.le.0 .or. g_s.gt.n_global_array) then
         call errquit('ga_diag: invalid array handle for s', g_s)
      else if (.not. ma_get_index(hand(g_s), sbase)) then
         call errquit('ga_diag: unable to map array handle s ?', g_s)
      else if (dims(1,g_s) .ne. dims(2,g_s)) then
         call errquit('ga_diag: can only diag square matrix', 0)
      endif
      if (g_v.le.0 .or. g_v.gt.n_global_array) then
         call errquit('ga_diag: invalid array handle for v', g_v)
      else if (.not. ma_get_index(hand(g_v), vbase)) then
         call errquit('ga_diag: unable to map array handle v ?', g_v)
      else if (dims(1,g_v) .ne. dims(2,g_v)) then
         call errquit('ga_diag: can only diag square matrix', 0)
      endif
c
*      call ga_print(g_a)
*      call ga_print(g_s)
c
c     allocate scratch space
c
      status = ma_push_get(MT_DBL, dims(1,g_a), 'fv1', l_fv1, k_fv1)
      status = ma_push_get(MT_DBL, dims(1,g_a), 'fv2', l_fv2, k_fv2)
c
      call rsg(dims(1,g_a), dims(1,g_a), dbl_mb(abase), dbl_mb(sbase),
     $     evals, 1, dbl_mb(vbase), dbl_mb(k_fv1), dbl_mb(k_fv2), ierr)
c
*      call ga_print(g_v)
*      write(6,*) ' evals '
*      call output(evals, 1, dims(1,g_a), 1, 1, dims(1,g_a), 1, 1)
c
      status = ma_pop_stack(l_fv2)
      status = ma_pop_stack(l_fv1)
c
      end
      subroutine ga_dgemm(transa, transb, m, n, k, alpha, g_a,
     $        g_b, beta, g_c)
      implicit none
#include "mafdecls.h"
#include "globalp.h"
      character*1 transa, transb
      integer m, n, k
      integer g_a, g_b, g_c
      double precision alpha, beta
c
c     c = alpha * a * b + beata c
c
      integer abase, bbase, cbase
c
c     Solve the generalized eigen-value problem returning
c     all eigen-vectors and values in ascending order
c
c     The input matrices may be destroyed
c
      if (g_a.le.0 .or. g_a.gt.n_global_array) then
         call errquit('ga_dgemm: invalid array handle for a', g_a)
      else if (.not. ma_get_index(hand(g_a), abase)) then
         call errquit('ga_dgemm: unable to map array handle a?', g_a)
      endif
      if (g_b.le.0 .or. g_b.gt.n_global_array) then
         call errquit('ga_dgemm: invalid array handle for a', g_b)
      else if (.not. ma_get_index(hand(g_b), bbase)) then
         call errquit('ga_dgemm: unable to map array handle a?', g_b)
      endif
      if (g_c.le.0 .or. g_c.gt.n_global_array) then
         call errquit('ga_dgemm: invalid array handle for a', g_c)
      else if (.not. ma_get_index(hand(g_c), cbase)) then
         call errquit('ga_dgemm: unable to map array handle a?', g_c)
      endif
c
      call dgemm(transa, transb, m, n, k, alpha, dbl_mb(abase),
     $     dims(1,g_a), dbl_mb(bbase), dims(1,g_b), beta, dbl_mb(cbase),
     $     dims(1,g_c))
c
      end
      subroutine ga_symmetrize(g_a)
      implicit none
#include "mafdecls.h"
#include "globalp.h"
      integer g_a               ! Matrix to symmetrize
c
      integer abase, i, j
      double precision tmp
c
      if (g_a.le.0 .or. g_a.gt.n_global_array) then
         call errquit('ga_sym: invalid array handle for a', g_a)
      else if (.not. ma_get_index(hand(g_a), abase)) then
         call errquit('ga_sym: unable to map array handle a?', g_a)
      else if (dims(1,g_a) .ne. dims(2,g_a)) then
         call errquit('ga_sym: can only sym square matrix', 0)
      endif
      do i = 1, dims(1,g_a)
         do j = 1, i-1
            tmp = (dbl_mb(abase-1+i+(j-1)*dims(1,g_a)) +
     $           dbl_mb(abase-1+j+(i-1)*dims(1,g_a))) * 0.5d0
            dbl_mb(abase-1+i+(j-1)*dims(1,g_a)) = tmp
            dbl_mb(abase-1+j+(i-1)*dims(1,g_a)) = tmp
         enddo
      enddo
c     
      end
      subroutine ga_zero(g_a)
      implicit none
#include "mafdecls.h"
#include "globalp.h"
      integer g_a               ! Matrix to zero
c
      integer abase
c
      if (g_a.le.0 .or. g_a.gt.n_global_array) then
         call errquit('ga_zero: invalid array handle for a', g_a)
      else if (.not. ma_get_index(hand(g_a), abase)) then
         call errquit('ga_zero: unable to map array handle a?', g_a)
      endif
c
      call dfill(dims(1,g_a)*dims(2,g_a), 0.0d0, dbl_mb(abase), 1)
c
      end
      subroutine ga_dscal(g_a, s)
      implicit none
#include "mafdecls.h"
#include "globalp.h"
      integer g_a               ! Matrix to scale
      double precision s
c
      integer abase
c
      if (g_a.le.0 .or. g_a.gt.n_global_array) then
         call errquit('ga_dscal: invalid array handle for a', g_a)
      else if (.not. ma_get_index(hand(g_a), abase)) then
         call errquit('ga_dscal: unable to map array handle a?', g_a)
      endif
c
      call dscal(dims(1,g_a)*dims(2,g_a), s, dbl_mb(abase), 1)
c
      end
      subroutine ga_add(alpha, g_a, beta, g_b, g_c)
      implicit none
#include "mafdecls.h"
#include "globalp.h"
c
      integer g_a, g_b, g_c
      integer abase, bbase, cbase, i, j, ij
      double precision alpha, beta
c
      if (g_a.le.0 .or. g_a.gt.n_global_array) then
         call errquit('ga_add: invalid array handle a', g_a)
      else if (g_b.le.0 .or. g_b.gt.n_global_array) then
         call errquit('ga_add: invalid array handle c', g_b)
      else if (g_c.le.0 .or. g_c.gt.n_global_array) then
         call errquit('ga_add: invalid array handle c', g_c)
      else if (.not. ma_get_index(hand(g_a), abase)) then
         call errquit('ga_add: unable to map array handle ?', g_a)
      else if (.not. ma_get_index(hand(g_b), bbase)) then
         call errquit('ga_add: unable to map array handle ?', g_b)
      else if (.not. ma_get_index(hand(g_c), cbase)) then
         call errquit('ga_add: unable to map array handle ?', g_c)
      else if (dims(1,g_a).ne.dims(1,g_b) .or.
     $         dims(1,g_a).ne.dims(1,g_c) .or.
     $         dims(2,g_a).ne.dims(2,g_c) .or.
     $         dims(2,g_a).ne.dims(2,g_b)) then
         call errquit('ga_add: dimensions do not match', 0)
      else
         do j = 1, dims(2,g_a)
            do i = 1, dims(1,g_a)
               ij = i-1 + (j-1)*dims(1,g_a)
               dbl_mb(cbase+ij) = alpha * dbl_mb(abase+ij) +
     $              beta * dbl_mb(bbase+ij)
            enddo
         enddo
      endif
c
      end
      subroutine ga_dcopy(g_a, g_b)
      implicit none
#include "mafdecls.h"
#include "globalp.h"
c
      integer g_a, g_b          ! Copy A into B
      integer abase, bbase
c
      if (g_a.le.0 .or. g_a.gt.n_global_array) then
         call errquit('ga_dcopy: invalid array handle ', g_a)
      else if (g_b.le.0 .or. g_b.gt.n_global_array) then
         call errquit('ga_dcopy: invalid array handle ', g_b)
      else if (.not. ma_get_index(hand(g_a), abase)) then
         call errquit('ga_dcopy: unable to map array handle ?', g_a)
      else if (.not. ma_get_index(hand(g_b), bbase)) then
         call errquit('ga_dcopy: unable to map array handle ?', g_b)
      else if (dims(1,g_a).ne.dims(1,g_b) .or.
     $         dims(2,g_a).ne.dims(2,g_b)) then
         call errquit('ga_dcopy: dimensions do not match', 0)
      else
         call dcopy(dims(1,g_a)*dims(2,g_a),
     $        dbl_mb(abase), 1, dbl_mb(bbase), 1)
      endif
c
      end
