      integer function ga_read_inc(g_a, i, j, inc)
      implicit none
#include "mafdecls.h"
#include "tcgmsg.h"
#include "globalp.h"
#include "global.h"
c     
      integer g_a               ! GA handle
      integer i, j              ! Indices of array element
      ineger inc                ! Increment
      integer p                 ! Owning process
c
c     Atomically read and increment the integer array element g_a(i,j)
c
c     *BEGIN CRITICAL SECTION*
c     return value = a(i,j)
c     a(i,j) += inc
c     *END CRITICAL SECTION*
c
#ifdef GA_TRACE
      call trace_stime
#endif
c
c     Error checking
c
      call ga_check_handle(g_a, 'ga_read_inc')
      if (ttyp(g_a) .ne. MT_INT) 
     $     call errquit('ga_read_inc: must be integer', g_a)
c
c     Figure out who owns the element and act accordingly
c
      if (.not. ga_locate(g_a, i, j, p))
     $     call errquit('ga_read_inc: i/j out of bounds', i*10000+j)
      if (p .eq. me .and. oserver) then
         ga_read_inc = ga_local_read_inc(g_a, i, j, inc)
      else
         ga_read_inc = ga_remote_read_inc(g_a, i, j, inc, p)
      endif
c
#ifdef GA_TRACE
      call trace_etime
      call trace_genrec(g_a, 0, 0, 0, 0, GA_OP_RDI)
#endif
c
      end
      integer function ga_remote_read_inc(g_a, i, j, inc, p)
      implicit none
#include "mafdecls.h"
#include "tcgmsg.h"
#include "globalp.h"
#include "global.h"
c
      integer g_a
      integer i, j, inc, p
      integer node, oldmask, value, len, from
c
#ifdef DATA_SERVER
      node = p + nserver
#else
      node = p
#endif
c
      if (me .lt. p) oldmask = ga_mask(1)
c
      call ga_snd_req(g_a, i, j, inc, 0, 0, 0, GA_OP_RDI, p)
      call ga_rcv_msg(GA_TYPE_RDI, value, mitob(1), len, node, from)
      ga_remote_read_inc = value
c
      if (me .lt. p) oldmask = ga_mask(oldmask)
c
      end
      integer function ga_local_read_inc(g_a, i, j, inc)
      implicit none
#include "mafdecls.h"
#include "globalp.h"
#include "global.h"
c
      integer g_a
      integer i, j, inc, iproc, base, ilo, jlo, idim, ij
c
#ifdef DATA_SERVER
      iproc = me - nserver
#else
      iproc = me
#endif
      base = indx(g_a)
      ilo  = mapc(1,iproc,g_a)
      jlo  = mapc(3,iproc,g_a)
      idim = ldim(1,g_a)
      ij   = base + (i-ilo) + (j-jlo)*idim
c
      oldmask = ga_mask(1)
      ga_local_read_inc = mb_int(ij)
      mb_int(ij) = mb_int(ij) + inc
      oldmask = ga_mask(oldmask)
c
      end
