/*****************************************************************
 *
 *    C routine mxm3: Q.W where Q and W are both column wrapped
 *                    and where the set of processors performing
 *                    the calculations for Q and those for W match
 *
 *                    assume that both map
 *
 *
 *
 */

#include <stdio.h>
#define max(a,b) ((a) > (b) ? (a) : (b))

void mxm35 ( n1, n2, colQ, mapQ, m, colW, mapW, colZ, iwork, work)
     int *n1, *n2, *mapQ, *m, *mapW, *iwork;
     double **colQ, **colW, **colZ, *work;
     /*
       
       computes Z<- Q*W where Z has the same column distribution as W
       it is assumed that Z is an n1 by m matrix
       
       assume that Q is n1 x n2 column-distributed across all the participating processors
       in this version we assume the Q and W share the same processors ( perhaps different
       data distributions )
       
       assume that W n2 by m matrix column distributed across all the participating processors
       
       this is our implementation of merry-go-around multiplication
       
       on input:
       
       n1 x n2 = dim of the matrix Q
       n2 x m = number of columns in vecZ
       
       num_vec = number of eigenvectors in w(those owned by this processor).
       
       work = the scratch array holding the matrix W;
       
       
       output:
       
       W is overwritten by QW.
       
       */
{
  static int ONE = 1, MSGTYP = 51;
  int ll, jj, k, i, nvecsQ, nvecsW, l, l1, l2;
  int *mapvecQ, *mapvecW;
  int isize, indx, jndx, kndx, ndx, msglen, me, num_row, numprocs;
  int *proclist, nprocs, *mapvec_in, last_proc, next_proc;
  int *iscrat, me_indx;
  int maxsz, rsize, osize;
  int *ijunk;
  
  double t, *ptr, scl, *buf_ptr;
  double *buffer, *in_buffer, *out_buffer;
  
  extern int count_list();
  extern int fil_mapvec_ ();
  
  extern int mxwrit_ (), mxread_ ();
  extern int menode_ (), mxmynd_ ();
  
  extern int fil_mapvec_ ();
  
  extern int count_list();
  extern int mxwrit_ (), mxread_ ();
  extern int mxmynd_ ();
  extern void mxsync_ ();
  extern void gshellsort_ ();
  
  /*
    blas call
    */

  extern void daxpy_ ();

  mxsync_ ();
  
  l1 = *n1;
  l2 = *n2;
  
  if ( ll < 1 )
    return;  /* error */
  
  me = mxmynd_ ();
  
  iscrat = iwork;
  mapvecQ = iscrat;
  nvecsQ = fil_mapvec_ ( &me, &l2, mapQ, mapvecQ );
  
  iscrat += nvecsQ;
  mapvecW = iscrat;
  nvecsW = fil_mapvec_ ( &me, m, mapW, mapvecW );
  
  if ( nvecsW + nvecsQ == 0 )
    return;
  
  iscrat += nvecsW;
  proclist = iscrat;
  
  ijunk = (int *) work;
  
  for ( indx = 0; indx < l2; indx++ )
    ijunk[indx] = mapQ[indx];

  isize = l2;
  for ( indx = 0; indx <  *m; indx++ ) {
    ijunk[isize] = mapW[indx];
    isize++;
  }
  
  /*
    the list of processors for this computation
    */
  
  indx = l2 + *m ;
  nprocs = reduce_list( indx, &ijunk[0], proclist);
  gshellsort_ ( &nprocs, proclist);
  
  /*
    the number of distinct processors
    */
  
  iscrat += nprocs;
  mapvec_in = iscrat;
  indx = me_indx = indxL ( me, nprocs, proclist);
  last_proc = (indx + nprocs - 1) % nprocs;
  last_proc = proclist[last_proc];
  next_proc = (indx + 1) % nprocs;
  next_proc = proclist[next_proc];
  
  maxsz = 0;
  for ( indx = 0; indx < nprocs; indx++ ) {
    isize = count_list( proclist[indx], mapQ, n2);   /* the number Q vectors that I own */
    maxsz = max( isize, maxsz );
  }
  
  /*
    
    load what I own of W into a buffer;
    for multiplication purpose
    
    */
  
  buffer = (double *) work;
  
  indx = 0;
  for ( jndx = 0; jndx < nvecsW; jndx++ ){
    dcopy_ ( &l2, colW[jndx], &ONE, &buffer[indx], &ONE);
    indx += l2;
  }
  buffer += indx;
  
  /*
    zero out the matrix
    */
  
  for ( jndx = 0; jndx < nvecsW; jndx++ ){
    ptr = colZ[jndx];
    for( indx = 0; indx < l1; indx++ )
      ptr[indx] = 0.0e0;
  }
  
  /*
    copy Q into output 
    shift matrix Q around and multiply
    */
  
  indx = 0;
  for ( jndx = 0; jndx < nvecsQ; jndx++ ){
    dcopy_ ( &l1, colQ[jndx], &ONE, buffer + indx, &ONE);
    indx+= l1;
  }
  
  for ( indx = 0; indx < nvecsQ; indx++ )
    mapvec_in[indx] = mapvecQ[indx];
  
  /*
    just computing matrix multiplication within our
    own vectors
    */
  
  for ( indx = 0; indx < nvecsW; indx++ ){  
    ptr = colZ[indx];
    for (jndx = 0; jndx < nvecsQ; jndx++ ){
      jj   = mapvec_in[jndx];
      scl = work[indx*l2 + jj];
      buf_ptr = buffer + jndx*l1;
      /*
	daxpy_ ( &l1, &scl, buf_ptr, &ONE, ptr, &ONE);
	*/
      for ( l = 0; l < l1; l++ )
	ptr[l] += scl*buf_ptr[l];
    }
  }
  
  /*
    this is the block that gets send around; first multiply what I have
    */
  
  in_buffer = buffer;
  out_buffer = in_buffer + maxsz*l1;
  
  isize = nvecsQ*l1;
  osize = isize;
  
  for ( i = 0; i < nprocs-1 ; i++ ) {
    dcopy_ (&osize, in_buffer, &ONE, out_buffer, &ONE);
    if ( (me_indx % 2) ==  0 ) {
      rsize = osize*sizeof(double);
      if ( rsize != 0 )
	rsize = mxwrit_ ( out_buffer, &rsize, &next_proc, &i );
      indx = ( me_indx + nprocs - i -1) % nprocs;
      indx = proclist[indx]; /* the current processor id number */
      isize = nvecsQ = fil_mapvec_ ( &indx, &l2, mapQ, mapvec_in);
      isize *= l1;
      
      /*
	mapvec_in contains the mapvecQ for processor proclist[i])
	*/
      
      rsize = isize * sizeof(double);
      if ( rsize != 0 )
	rsize = mxread_( in_buffer, &rsize, &last_proc, &i );
    }
    else {
      indx = ( me_indx + nprocs - i -1 ) % nprocs;
      indx = proclist[indx]; /* the current processor id number */
      isize = nvecsQ = fil_mapvec_ ( &indx, &l2, mapQ, mapvec_in);
      isize *= l1;
      
      /*
	mapvec_in contains the mapvecQ for processor proclist[i])
	*/
      
      rsize = isize * sizeof(double);
      if ( rsize != 0 )
	rsize = mxread_( in_buffer, &rsize, &last_proc, &i );
      
      rsize = osize * sizeof(double);
      if ( rsize != 0 )
	rsize = mxwrit_ ( out_buffer, &rsize, &next_proc, &i );
    }
    
    osize = isize;
    
    for ( indx = 0; indx < nvecsW; indx++ ){  
      ptr = colZ[indx];
      for (jndx = 0; jndx < nvecsQ; jndx++ ){
	jj   = mapvec_in[jndx];
	scl = work[indx*l2 + jj];
	buf_ptr = in_buffer + jndx*l1;
	/*
	  should daxpy this
	  daxpy_ ( &l1, &scl, buf_ptr, &ONE, ptr, &ONE);
	  */
	for ( l = 0; l < l1; l++ )
	  ptr[l] += scl*buf_ptr[l];
      }
    }
  }
  return;
}


