      program test
      implicit none
#include "mafdecls.h"
#include "global.h"
#include "tcgmsg.h"
      logical status
      integer heap, stack, fudge, n
      parameter (n = 128)
*      parameter (heap=n*n*3, stack=n*n, fudge=10240)
      parameter (heap=500000, stack=500000, fudge=10240)
c     
c     Intitialize the TCGMSG package
c
      call pbeginf
*     call setdbg(1)            ! Turn TCGMSG debugging on/off
c
c     Initialize the MA package
c
      if(.not. ma_initialize(MT_DBL,
     $     stack/ga_nnodes(), heap/ga_nnodes() + fudge))
     $     call errquit('ma failed', (stack+heap)/ga_nnodes()+fudge)
      status = ma_set_auto_verify(.true.)
c
c     Intitialize the GA package
c
      call ga_initialize()
c
c     Check support for double precision arrays
c
#ifdef GA_TRACE
      call trace_init(2000)
      call ga_sync
      call trace_stime
      call trace_etime
      call trace_genrec(0, 0, 0, 0, 0, 0)
#endif

      call check_dbl()

#ifdef GA_TRACE
      call trace_end(ga_nodeid())
#endif
c
c     Check support for integer arrays (limited !)
c
      call check_int()
c
c     Measure the performance !
c
      call check_time()
c
c     Tidy up the global array package
c
      call ga_terminate()
c
#ifndef NX
      if (ga_nodeid() .eq. 0) then
         write(6,*)
         call stats()
      endif
#endif
c
c     Tidy up after TCGMSG
c
      call pend()
c
      end
      subroutine check_dbl()
      implicit none
#include "mafdecls.h"
#include "global.h"
#include "tcgmsg.h"
c     
      integer n
      parameter (n = 128)
      double precision a(n,n), b(n,n), c(n,n), d(n,n)
      logical status
      integer g_a, g_b, g_c, count
      integer iran, i, j, loop, nloop, ilo, ihi, jlo, jhi, itmp
      integer nproc, me, int, ij, inc, ii, jj, ibuf(n*n), jbuf(n*n)
      integer ita, itb
      character*1 trans(2)
      parameter (nloop = 100)
      double precision drand, crap, sum1, sum2, x
      double precision nwords, vbuf(n*n), t_dgemm, t_ga_dgemm
      external drand
      intrinsic int
      data trans / 'n', 't' /
      iran(i) = int(drand(0)*dfloat(n)) + 1
c     
      nproc = ga_nnodes()
      me    = ga_nodeid()
c     
c     a() is a local copy of what the global array should start as
c
      do j = 1, n
         do i = 1, n
            a(i,j) = i-1 + (j-1)*n
         enddo
      enddo
*      write(6,*) ' correct '
*      call output(a, 1, n, 1, n, n, n, 1)
*      call ffflush(6)
c
c     Create a global array
c
      status = ga_create(MT_DBL, n, n, 'a', 1, 1, g_a)
      if (.not. status) then
         write(6,*) ' ga_create failed'
         call ffflush(6)
         call parerr(2)
      endif
c     
c     Zero the array
c
      if (me .eq. 0) then 
         write(6,21) 
 21      format(/' Checking zero ... ')
         call ffflush(6)
      endif
      call ga_zero(g_a)
c
c     Check that it is indeed zero
c
      call ga_get(g_a, 1, n, 1, n, b, n)
      do i = 1, n
         do j = 1, n
            if (b(i,j) .ne. 0.0d0) then
               write(6,*) ' zero ', me, i, j, b(i,j)
               call ffflush(6)
               call parerr(1)
            endif
         enddo
      enddo
      if (me.eq.0) then
         write(6,*)
         write(6,*) '   ga_zero is OK'
         write(6,*)
      endif
      call ga_sync()
c
c     Each node fills in disjoint sections of the array
c
      if (me .eq. 0) then
         write(6,2) 
 2       format(/' Checking disjoint put ... ')
         call ffflush(6)
      endif
      call ga_sync()
c
      inc = (n-1)/10 + 1
      ij = 0
      do j = 1, n, inc
         do i = 1, n, inc
            if (mod(ij,nproc) .eq. me) then
               ilo = i
               ihi = min(i+inc, n)
               jlo = j
               jhi = min(j+inc, n)
*               write(6,4) me, ilo, ihi, jlo, jhi
* 4             format(' node ',i2,' checking put ',4i4)
*               call ffflush(6)
               call ga_put(g_a, ilo, ihi, jlo, jhi, a(ilo, jlo), n)
            endif
            ij = ij + 1
         enddo
      enddo
      call ga_sync()
c
c     All nodes check all of a
c
      call dfill(n*n, 0.0d0, b, 1)
      call ga_get(g_a, 1, n, 1, n, b, n)
*      write(6,*) ' after get'
*      call output(b, 1, n, 1, n, n, n, 1)
c
      do i = 1, n
         do j = 1, n
            if (b(i,j) .ne. a(i,j)) then
               write(6,*) ' put ', me, i, j, a(i,j),b(i,j)
               call ffflush(6)
               call parerr(1)
            endif
         enddo
      enddo
      if (me.eq.0) then
         write(6,*)
         write(6,*) '   ga_put is OK'
         write(6,*)
      endif
      call ga_sync()
c
c     Now check nloop random gets from each node
c
      if (me .eq. 0) then
         write(6,5) nloop
 5       format(/' Checking random get (',i5,' calls)...')
         call ffflush(6)
      endif
      call ga_sync()
c
      nwords = 0
c
      crap = drand(me*51 + 1) ! Different seed for each process
      do loop = 1, nloop
         ilo = iran(loop)
         ihi = iran(loop)
         if (ihi.lt. ilo) then
            itmp = ihi
            ihi = ilo
            ilo = itmp
         endif
         jlo = iran(loop)
         jhi = iran(loop)
         if (jhi.lt. jlo) then
            itmp = jhi
            jhi = jlo
            jlo = itmp
         endif
c
         nwords = nwords + (ihi-ilo+1)*(jhi-jlo+1)
c     
         call dfill(n*n, 0.0d0, b, 1)
         call ga_get(g_a, ilo, ihi, jlo, jhi, b(ilo, jlo), n)
         if (me .eq. 0 .and. mod(loop-1, max(1,nloop/20)).eq.0) then
            write(6,1) loop, me, ilo, ihi, jlo, jhi, nwords
 1          format(' call ',i5, ' node ',i2,' checking get ',4i4,
     $           ' total ',d9.2)
            call ffflush(6)
         endif
         sum1 = 0.0d0
         do j = jlo, jhi
            do i = ilo, ihi
               sum1 = sum1 + b(i,j)
               if (b(i,j) .ne. a(i,j)) then
                  write(6,*) i, j, b(i,j), a(i,j)
                  call ffflush(6)
                  call parerr(1)
               endif
            enddo
         enddo
c
c     This test for writing out of range is quite slow ...
c     commented out here only to permit greater contention
c
*         sum2 = 0.0d0
*         do j = 1, n
*            do i = 1, n
*               sum2 = sum2 + b(i,j)
*            enddo
*         enddo
*         if (sum1 .ne. sum2) then
*            write(6,*) ' sums dont match ', sum1, sum2
*            call ffflush(6)
*            call parerr(2)
*         endif
      enddo
      if (me.eq.0) then
         write(6,*)
         write(6,*) '   ga_get is OK'
         write(6,*)
      endif
      call ga_sync()
c
c     Each node accumulates into disjoint sections of the array
c
      if (me .eq. 0) then
         write(6,9) 
 9       format(/' Checking disjoint accumulate ... ')
         call ffflush(6)
      endif
      call ga_sync()
c     
      crap = drand(12345)       ! Same seed for each process
      do j = 1, n
         do i = 1, n
            b(i,j) = drand(0)
         enddo
      enddo
c
      inc = (n-1)/10 + 1
      ij = 0
      do j = 1, n, inc
         do i = 1, n, inc
            x = drand(0)
            ilo = i
            ihi = min(i+inc, n)
            jlo = j
            jhi = min(j+inc, n)
            if (mod(ij,nproc) .eq. me) then
*               write(6,11) me, ilo, ihi, jlo, jhi
* 11            format(' node ',i2,' checking update ',4i4)
*               call ffflush(6)
               call ga_acc(g_a, ilo, ihi, jlo, jhi, b(ilo, jlo), n, x)
            endif
            ij = ij + 1
c     
c     Each process applies all updates to its local copy
c
            do jj = jlo, jhi
               do ii = ilo, ihi
                  a(ii,jj) = a(ii,jj) + x * b(ii,jj)
               enddo
            enddo
         enddo
      enddo
      call ga_sync()
c
c     All nodes check all of a
c
      call ga_get(g_a, 1, n, 1, n, b, n)
      do j = 1, n
         do i = 1, n
            if (abs(b(i,j) - a(i,j)) .gt. 1.0d-6) then
               write(6,*) ' acc ', me, i, j, a(i,j), b(i,j)
               call ffflush(6)
               call parerr(1)
            endif
         enddo
      enddo
      if (me.eq.0) then
         write(6,*)
         write(6,*) '   ga_acc is OK'
         write(6,*)
      endif
      call ga_sync()
c
c     Check the ga_add function
c
      if (me .eq. 0) then
         write(6,91)
 91      format(/' Checking add ...')
         call ffflush(6)
      endif
      if (.not. ga_create(MT_DBL, n, n, 'b', 1, 1, g_b)) then
         write(6,*) ' ga_create failed for second array'
         call ffflush(6)
         call parerr(2)
      endif
      crap = drand(12345)       ! Everyone has same seed
      do j = 1, n
         do i = 1, n
            b(i,j) = drand(0)
            a(i,j) = 0.1d0*a(i,j) + 0.9d0*b(i,j)
         enddo
      enddo
      if (me.eq.0) call ga_put(g_b, 1, n, 1, n, b, n)
      call ga_dadd(0.1d0, g_a, 0.9d0, g_b, g_b)
      call ga_get(g_b, 1, n, 1, n, b, n)
      do j = 1, n
         do i = 1, n
            if (abs(b(i,j) - a(i,j)) .gt. 1.0d-6) then
               write(6,*) ' add ', me, i, j, a(i,j), b(i,j)
               call ffflush(6)
               call parerr(1)
            endif
         enddo
      enddo
c
      call ga_sync()
      if (me .eq. 0) then
         write(6,*) 
         write(6,*) '   ga_dadd is OK'
         write(6,*)
      endif
c
c     Check the scatter routine into disjoint elements
c     of the array
c
      if (me .eq. 0) then
         write(6,191) 
 191     format(/' Checking disjoint scatter ... ')
         call ffflush(6)
      endif
      call ga_sync()
c     
      crap = drand(7890)       ! Same seed for each process
      ij = 0
      count = 0
      do j = 1,n
         do i = 1, n
            b(i,j) = drand(0)
            if (mod(ij,nproc).eq.me) then
               count = count + 1
               vbuf(count) = b(i,j)
               ibuf(count) = i
               jbuf(count) = j
            endif
            ij = ij + 1
         enddo
      enddo
      call ga_dscatter(g_a, vbuf, ibuf, jbuf, count)
c
      call ga_sync
c
c     All nodes check all of a
c
      call ga_get(g_a, 1, n, 1, n, a, n)
      do j = 1, n
         do i = 1, n
            if (b(i,j) .ne. a(i,j)) then
               write(6,*) ' sct ', me, i, j, a(i,j), b(i,j)
               call ffflush(6)
               call parerr(1)
            endif
         enddo
      enddo
      if (me.eq.0) then
         write(6,*)
         write(6,*) '   ga_dscatter is OK'
         write(6,*)
      endif
      call ga_sync()
c
c     Check the ddot function
c
      if (me .eq. 0) then
         write(6,19) 
 19      format(/' Checking ddot ...')
         call ffflush(6)
      endif
      crap = drand(12345)       ! Everyone has same seed
      sum1 = 0.0d0
      do j = 1, n
         do i = 1, n
            b(i,j) = drand(0)
            sum1 = sum1 + a(i,j)*b(i,j)
         enddo
      enddo
      if (me.eq.0) then 
         call ga_put(g_b, 1, n, 1, n, b, n)
         call ga_put(g_a, 1, n, 1, n, a, n)
      endif
      sum2 = ga_ddot(g_a,g_b)
      if (dabs(sum2 - sum1)/dabs(sum1) .gt. 1.0d-12) then
         write(6,*) ' ddot wrong ', sum1, sum2
         call ffflush(6)
         call parerr(1)
      else if (me.eq.0) then
         write(6,*)
         write(6,*) '   ddot is OK '
         write(6,*)
      endif
c
c     Check the ga_dscal function
c
      if (me .eq. 0) then
         write(6,92)
 92      format(/' Checking dscal ...')
         call ffflush(6)
      endif
      call dscal(n*n, 0.123d0, a, 1)
      call ga_dscal(g_a, 0.123d0)
      call ga_get(g_a, 1, n, 1, n, b, n)
      do j = 1, n
         do i = 1, n
            if (b(i,j) .ne. a(i,j)) then
               write(6,*) ' dscal ', me, i, j, a(i,j), b(i,j)
               call ffflush(6)
               call parerr(1)
            endif
         enddo
      enddo
      if (me.eq.0) then
         write(6,*)
         write(6,*) '   dscal is OK '
         write(6,*)
      endif
c
c     Check the ga_copy function
c
      if (me .eq. 0) then
         write(6,94)
 94      format(/' Checking copy ...')
         call ffflush(6)
      endif
      call ga_copy(g_a, g_b)
      call ga_get(g_b, 1, n, 1, n, b, n)
      do j = 1, n
         do i = 1, n
            if (b(i,j) .ne. a(i,j)) then
               write(6,*) ' copy ', me, i, j, a(i,j), b(i,j)
               call ffflush(6)
               call parerr(1)
            endif
         enddo
      enddo
      if (me.eq.0) then
         write(6,*)
         write(6,*) '   dcopy is OK '
         write(6,*)
      endif
c
c     Check the dgemm
c
      if (.not. ga_create(MT_DBL, n, n, 'c', 1, 1, g_c)) then
         write(6,*) ' ga_create failed for third array'
         call ffflush(6)
         call parerr(2)
      endif
      if (me .eq. 0) then
         write(6,93)
 93      format(/' Checking dgemm ...')
         call ffflush(6)
      endif
      crap = drand(79811)
      do i = 1, n
         do j = 1, n
            a(i,j) = drand(0)
            b(i,j) = drand(0)
            c(i,j) = drand(0)
         enddo
      enddo
      if (me.eq.0) then 
         call ga_put(g_a, 1, n, 1, n, a, n)
         call ga_put(g_b, 1, n, 1, n, b, n)
         call ga_put(g_c, 1, n, 1, n, c, n)
      endif
      call ga_sync
      t_dgemm = 0.0d0
      t_ga_dgemm = 0.0d0
      do ita = 1, 2
         do itb = 1, 2
            t_dgemm = t_dgemm - tcgtime()
c$$$            call dgemm(trans(ita),trans(itb),n,n,n,
c$$$     $           -1.1d0, a, n, b, n, 2.17d0, c, n)
            t_dgemm = t_dgemm + tcgtime()
            call ga_sync
            t_ga_dgemm = t_ga_dgemm - tcgtime()
c$$$            call ga_dgemm(trans(ita), trans(itb),n,n,n,
c$$$     $           -1.1d0, g_a, g_b, 2.17d0, g_c)
            t_ga_dgemm = t_ga_dgemm + tcgtime()
            call ga_get(g_c, 1, n, 1, n, d, n)
            do j = 1, n
               do i = 1, n
                  if (abs(d(i,j)-c(i,j)) .gt. 1.0d-10) then
                     write(6,*) ' dgemm ', me, i, j,
     $                    trans(ita), trans(itb), d(i,j),
     $                    c(i,j)
                     call ffflush(6)
                     call parerr(1)
                  endif
               enddo
            enddo
         enddo
      enddo
      if (me.eq.0) then
         write(6,95) dfloat(8*n*n*n)/t_dgemm, dfloat(8*n*n*n)/t_ga_dgemm
 95      format(/' Sequential rate =',1pd9.2, ' Parallel rate =',1pd9.2)
         write(6,*)
         write(6,*) '  dgemm is OK '
         write(6,*)
      endif
c
c     Delete the global arrays
c
      status = ga_destroy(g_c)
      status = ga_destroy(g_b)
      status = ga_destroy(g_a)
c     
      end
      subroutine check_int()
      implicit none
#include "mafdecls.h"
#include "global.h"
#include "tcgmsg.h"
c     
      integer n
      integer x, ii, jj
      parameter (n = 128)
      integer a(n,n), b(n,n)
      logical status
      integer g_a
      integer iran, i, j, loop, nloop, ilo, ihi, jlo, jhi, itmp
      integer nproc, me, int, ij, inc
      double precision nwords
      parameter (nloop = 100)
      double precision drand, crap, sum1, sum2
      external drand
      intrinsic int
      iran(i) = int(drand(0)*dfloat(n)) + 1
c     
      nproc = ga_nnodes()
      me    = ga_nodeid()
c     
c     a() is a local copy of what the global array should start as
c
      do j = 1, n
         do i = 1, n
            a(i,j) = i-1 + (j-1)*n
         enddo
      enddo
c     
c     Create a global array
c
      if (.not. ga_create(MT_INT, n, n, 'a', 1, 1, g_a)) then
         write(6,*) ' ga_create failed'
         call ffflush(6)
         call parerr(2)
      endif
c     
c     Zero the array
c
      if (me .eq. 0) then
         write(6,21) 
 21      format(/' Checking zero ... ')
         call ffflush(6)
      endif
      call ga_zero(g_a)
c
c     Check that it is indeed zero
c
      call ga_get(g_a, 1, n, 1, n, b, n)
      do i = 1, n
         do j = 1, n
            if (b(i,j) .ne. 0) then
               write(6,*) ' zero ', me, i, j, b(i,j)
               call ffflush(6)
               call parerr(1)
            endif
         enddo
      enddo
      if (me.eq.0) then
         write(6,*)
         write(6,*) '   ga_zero is OK'
         write(6,*)
      endif
      call ga_sync()
c
c     Each node fills in disjoint sections of the array
c
      if (me .eq. 0) then
         write(6,2) 
 2       format(/' Checking disjoint put ... ')
         call ffflush(6)
      endif
      call ga_sync()
c
      inc = (n-1)/10 + 1
      ij = 0
      do j = 1, n, inc
         do i = 1, n, inc
            if (mod(ij,nproc) .eq. me) then
               ilo = i
               ihi = min(i+inc, n)
               jlo = j
               jhi = min(j+inc, n)
*               write(6,4) me, ilo, ihi, jlo, jhi
* 4             format(' node ',i2,' checking put ',4i4)
*               call ffflush(6)
               call ga_put(g_a, ilo, ihi, jlo, jhi, a(ilo, jlo), n)
            endif
            ij = ij + 1
         enddo
      enddo
      if (me.eq.0) then
         write(6,*)
         write(6,*) '   ga_put is OK'
         write(6,*)
      endif
      call ga_sync()
c
c     All nodes check all of a
c
      call ga_get(g_a, 1, n, 1, n, b, n)
      do i = 1, n
         do j = 1, n
            if (b(i,j) .ne. a(i,j)) then
               write(6,*) ' put ', me, i, j, a(i,j),b(i,j)
               call ffflush(6)
               call parerr(1)
            endif
         enddo
      enddo
      call ga_sync()
c
c     Now check nloop random gets from each node
c
      if (me .eq. 0) then
         write(6,5) nloop
 5       format(/' Checking random get (',i5,' calls)...')
         call ffflush(6)
      endif
      call ga_sync()
c
      nwords = 0
c
      crap = drand(me*51 + 1) ! Different seed for each process
      do loop = 1, nloop
         ilo = iran(loop)
         ihi = iran(loop)
         if (ihi.lt. ilo) then
            itmp = ihi
            ihi = ilo
            ilo = itmp
         endif
         jlo = iran(loop)
         jhi = iran(loop)
         if (jhi.lt. jlo) then
            itmp = jhi
            jhi = jlo
            jlo = itmp
         endif
c     
         nwords = nwords + (ihi-ilo+1)*(jhi-jlo+1)
c
         call ifill(n*n, 0, b, 1)
         call ga_get(g_a, ilo, ihi, jlo, jhi, b(ilo, jlo), n)
c
         if (me .eq. 0 .and. mod(loop-1, max(1,nloop/20)).eq.0) then
            write(6,1) loop, me, ilo, ihi, jlo, jhi, nwords
 1          format(' call ',i5, ' node ',i2,' checking get ',4i4,
     $           ' total ',d9.2)
            call ffflush(6)
         endif
c
         sum1 = 0.0d0
         do j = jlo, jhi
            do i = ilo, ihi
               sum1 = sum1 + b(i,j)
               if (b(i,j) .ne. a(i,j)) then
                  write(6,*) i, j, b(i,j), a(i,j)
                  call ffflush(6)
                  call parerr(1)
               endif
            enddo
         enddo
c
c     This test for writing out of range is quite slow ...
c     commented out here only to permit greater contention
c
*         sum2 = 0.0d0
*         do j = 1, n
*            do i = 1, n
*               sum2 = sum2 + b(i,j)
*            enddo
*         enddo
*         if (sum1 .ne. sum2) then
*            write(6,*) ' sums dont match ', sum1, sum2
*            call ffflush(6)
*            call parerr(2)
*         endif
      enddo
      if (me.eq.0) then
         write(6,*)
         write(6,*) '   ga_get is OK'
         write(6,*)
      endif
      call ga_sync()
c
c     Each node accumulates into disjoint sections of the array
c
      if (me .eq. 0) then
         write(6,9) 
 9       format(/' Checking disjoint accumulate ... ')
         call ffflush(6)
      endif
      call ga_sync()
c     
      crap = drand(12345)       ! Same seed for each process
      do j = 1, n
         do i = 1, n
            b(i,j) = 1000*drand(0)
         enddo
      enddo
c
      inc = (n-1)/10 + 1
      ij = 0
      do j = 1, n, inc
         do i = 1, n, inc
            x = 2*drand(0)+1
            ilo = i
            ihi = min(i+inc, n)
            jlo = j
            jhi = min(j+inc, n)
            if (mod(ij,nproc) .eq. me) then
*               write(6,11) me, ilo, ihi, jlo, jhi
* 11            format(' node ',i2,' checking update ',4i4)
*               call ffflush(6)
               call ga_iacc(g_a, ilo, ihi, jlo, jhi, b(ilo, jlo), n, x)
            endif
            ij = ij + 1
c     
c     Each process applies all updates to its local copy
c
            do jj = jlo, jhi
               do ii = ilo, ihi
                  a(ii,jj) = a(ii,jj) + x * b(ii,jj)
               enddo
            enddo
         enddo
      enddo
      call ga_sync()
c
c     All nodes check all of a
c
      call ga_get(g_a, 1, n, 1, n, b, n)
      do j = 1, n
         do i = 1, n
            if (abs(b(i,j) - a(i,j)) .ne. 0) then
               write(6,*) ' iacc ', me, i, j, a(i,j), b(i,j)
               call ffflush(6)
               call parerr(1)
            endif
         enddo
      enddo
      if (me.eq.0) then
         write(6,*)
         write(6,*) '   ga_iacc is OK'
         write(6,*)
      endif
      call ga_sync()
c     
c     Delete the global array
c
      status = ga_destroy(g_a)
c     
      end
      subroutine ga_dummy(g_a, ilo, ihi, jlo, jhi, buf, len, a)
      end
      subroutine check_time
      implicit none
#include "mafdecls.h"
#include "global.h"
#include "tcgmsg.h"
c     
      integer n
      parameter (n = 128)
c
      double precision buf(n, n), used, time_per_word, time_per_call,
     $     bandwidth, tpredict, t0, t1, drand, crap, time_overhead
      integer len, ncalls, icall, ilo, ihi, jlo, jhi, g_a, nproc
      integer iran, low, i
      external drand
      intrinsic int
      iran(i) = min(int(drand(0)*dfloat(n)) + 1, low)
c
      crap = drand(ga_nodeid()*101)
c
c     Time overhead
c
      low = 128
      len = 32
      ncalls = 10000
      time_overhead = - tcgtime()
      do icall = 1, ncalls
         ilo = iran(icall)
         ihi = ilo + len
         jlo = iran(icall)
         jhi = jlo + len
         call ga_dummy(g_a, ilo, ihi, jlo, jhi, buf, len, 1.0d0)
      enddo
      time_overhead = (time_overhead +  tcgtime()) / ncalls
c
      if (ga_nodeid() .eq. 0) write(6,33) time_overhead
 33   format(/' Overhead per call (2xran# + subr. call) = ',1pd9.2/)
c
      call dfill(n*n, 0.0d0, buf, 1)
      nproc = ga_nnodes()
c
      if (.not. ga_create(MT_DBL, n, n, 'a', 1, 1, g_a))
     $     call errquit('check_time: ga_create failed', 0)
c
      if (ga_nodeid() .eq. 0) write(6,2)
 2    format(
     $     ' nproc  len   used  ncall   t/calls   t/words   rate'/
     $     ' -----  ---  -----  -----   -------   -------   ----')
c
      len = 1
 10   continue
c
c     Adjust ncalls so that each run takes about 2 seconds
c
      if (ga_nnodes() .eq. 1) then
         t0 = 30.0d-6
         t1 = 0.522d-6
      else
         t0 = 400d-6
         t1 = 3.6d-6
      endif
      tpredict = (time_overhead + t0 + len*len*t1)
      ncalls = min(1.0d6,max(1.0d0, 5.0d0 / tpredict))
c
      low = n - len + 1
      call ga_sync()
      used = - tcgtime()
*      if (nodeid() .eq. 0) then
      do icall = 1, ncalls
         ilo = iran(icall)
         ihi = ilo + len - 1
         jlo = iran(icall)
         jhi = jlo + len - 1
         call ga_acc(g_a, ilo, ihi, jlo, jhi, buf, len, 1.0d0)
      enddo
*      endif
      call ga_sync()
      used = used + tcgtime()
c
      time_per_call = used / ncalls - time_overhead
      time_per_word = used / (ncalls*len*len)
      bandwidth = 1.0d0 / time_per_word
      if (ga_nodeid() .eq. 0) 
     $     write(6,1) nproc, len, used, ncalls, time_per_call, 
     $     time_per_word, bandwidth
 1    format(1x,i5,i5,f7.2,i7,1x,1pd9.2,1x,1pd9.2,1x,1pd9.2)
c
      len = len * 2
      if (len .le. n) goto 10
c
      if (.not. ga_destroy(g_a))
     $     call errquit('check_time: destroy failed', 0)
c
      end

