C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C NAME
C     alloc_oneint -- Allocate memory required for the one-electron 
C     integrals on the heap and hide the handles.
C
C SYNOPSIS
      Logical Function Alloc_OneInt()
      Implicit NONE
C
C DESCRIPTION
C     Allocate memory needed for evaluating one-electron integrals on
C     the heap.  
C
C     Pointers to this memory can be obtained from Get_Index_OneInt.
C
C     Before entry, the integral environment must have been prepared
C     with a call to Init_Integrals.
C
C     The amount of memory that is required for the arrays allocted
C     here is available from Mem_OneInt.
C
C REVISION
C     $Id: alloc_oneint.F,v 1.1 1993/08/30 21:24:57 gg502 Exp $
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#include "stdio.fh"
#include "mafdecls.h"
c$$$#include "mafuncs.fh"
#include "integral_memory.fh"
C
C     Make sure environment has been initialized
C
      If ( Int_Env .eq. 0) then
         Write (LuErr, *) 'Alloc_OneInt: Integral environment has not ',
     $        'been initialized.'
         STOP
      EndIf
C
C     See if the sizes have been initialized for this environment
C
      If ( Abs(Int_Max_1e) + Abs(Int_L_Scr1e) .eq. 0)
     $     Call IntMem( Int_Env, Int_L_Scr1e, Int_L_Scr2e,
     $     Int_Max_1e, Int_Max_2e)
C
C     Now actually allocate these things!
C
      Alloc_OneInt = MA_Allocate_Heap( MT_DBL, Int_Max_1e,
     $    'S Integral buffer', Int_S_Hdl)
      If ( .NOT. Alloc_OneInt ) Return
C
      Alloc_OneInt = MA_Allocate_Heap( MT_DBL, Int_Max_1e,
     $    'T Integral buffer', Int_T_Hdl)
      If ( .NOT. Alloc_OneInt ) Return
C
      Alloc_OneInt = MA_Allocate_Heap( MT_DBL, Int_Max_1e,
     $    'V Integral buffer', Int_V_Hdl)
      If ( .NOT. Alloc_OneInt ) Return
C
      Alloc_OneInt = MA_Allocate_Heap( MT_INT, Int_Max_1e,
     $    '1e-int index I', Int_I1e_Hdl)
      If ( .NOT. Alloc_OneInt ) Return
C
      Alloc_OneInt = MA_Allocate_Heap( MT_INT, Int_Max_1e,
     $    '1e-int index J', Int_J1e_Hdl)
      If ( .NOT. Alloc_OneInt ) Return
C
      Alloc_OneInt = MA_Allocate_Heap( MT_DBL, Int_L_Scr1e,
     $    '1e-int scratch space', Int_Scr1e_Hdl)
      If ( .NOT. Alloc_OneInt ) Return
C
      Return
      End
