C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C NAME
C     get_index_oneint -- Obtain pointers to memory needed to process
C     one-electron integrals
C
C SYNOPSIS
      Logical Function Get_Index_OneInt( Env, S_Off, T_Off,
     $     V_Off, I_Off, J_Off, MxNInt, Scr_Off, LScr)
      Implicit NONE
      Integer Env, S_Off, T_Off, V_Off, I_Off, J_Off, MxNInt
      Integer Scr_Off, LScr
C
C ARGUMENTS
C     Env     Pointer to integral environment (OUT)
C     S_Off   Pointer to memory for overlap integrals (OUT)
C     T_Off   Pointer to memory for kinetic energy integrals (OUT)
C     V_Off   Pointer to memory for potential integrals (OUT)
C     I_Off   Pointer to memory for lhs indices (OUT)
C     J_Off   Pointer to memory for rhs indices (OUT)
C     MxNInt  Maximum number of integrals possible in a batch (OUT)
C     Scr_Off Pointer to scratch space for computation (OUT)
C     LScr    Length allocated to scratch array (OUT)
C
C DESCRIPTION
C     Obtains pointers for user arrays for one-electron integral
C     evaluation.  Handles must already have been obtained with
C     Alloc_OneInt or Push_OneInt.
C
C REVISION
C     $Id: get_index_oneint.F,v 1.1 1993/08/30 21:25:04 gg502 Exp $
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C#include "mafuncs.fh"
#include "mafdecls.h"
#include "integral_memory.fh"
C
      Env = Int_Env
      MxNInt = Int_Max_1e
      LScr = Int_L_Scr1e
C
      Get_Index_OneInt = MA_Get_Index( Int_S_Hdl, S_Off)
      If ( .NOT. Get_Index_OneInt ) Return
      Get_Index_OneInt = MA_Get_Index( Int_T_Hdl, T_Off)
      If ( .NOT. Get_Index_OneInt ) Return
      Get_Index_OneInt = MA_Get_Index( Int_V_Hdl, V_Off)
      If ( .NOT. Get_Index_OneInt ) Return
      Get_Index_OneInt = MA_Get_Index( Int_I1e_Hdl, I_Off)
      If ( .NOT. Get_Index_OneInt ) Return
      Get_Index_OneInt = MA_Get_Index( Int_J1e_Hdl, J_Off)
      If ( .NOT. Get_Index_OneInt ) Return
      Get_Index_OneInt = MA_Get_Index( Int_Scr1e_Hdl, Scr_Off)
      If ( .NOT. Get_Index_OneInt ) Return
C
      Return
      End
