C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C NAME
C     get_index_twoint -- Obtain pointers to memory needed to process
C     two-electron integrals
C
C SYNOPSIS
      Logical Function Get_Index_TwoInt( Env, Val_Off,
     $     I_Off, J_Off, K_Off, L_Off, MxNInt, Scr_Off, LScr)
      Implicit NONE
      Integer Env, Val_Off, I_Off, J_Off, K_Off, L_Off, MxNInt
      Integer Scr_Off, LScr
C
C ARGUMENTS
C     Env     Pointer to integral environment (OUT)
C     Val_Off Pointer to memory for integral values (OUT)
C     I_Off   Pointer to memory for lhs index 1 (OUT)
C     J_Off   Pointer to memory for lhs index 2 (OUT)
C     K_Off   Pointer to memory for rhs index 1 (OUT)
C     L_Off   Pointer to memory for rhs index 2 (OUT)
C     MxNInt  Maximum number of integrals possible in a batch (OUT)
C     Scr_Off Pointer to scratch space for computation (OUT)
C     LScr    Length allocated to scratch array (OUT)
C
C DESCRIPTION
C     Obtains pointers for user arrays for two-electron integral
C     evaluation.  Handles must already have been obtained with
C     Alloc_TwoInt or Push_TwoInt.
C
C REVISION
C     $Id: get_index_twoint.F,v 1.1 1993/08/30 21:25:06 gg502 Exp $
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C#include "mafuncs.fh"
#include "mafdecls.h"
#include "integral_memory.fh"
C
      Env = Int_Env
      MxNInt = Int_Max_2e
      LScr = Int_L_Scr2e
C
      Get_Index_TwoInt = MA_Get_Index( Int_Val_Hdl, Val_Off)
      If ( .NOT. Get_Index_TwoInt ) Return
      Get_Index_TwoInt = MA_Get_Index( Int_I2e_Hdl, I_Off)
      If ( .NOT. Get_Index_TwoInt ) Return
      Get_Index_TwoInt = MA_Get_Index( Int_J2e_Hdl, J_Off)
      If ( .NOT. Get_Index_TwoInt ) Return
      Get_Index_TwoInt = MA_Get_Index( Int_K2e_Hdl, K_Off)
      If ( .NOT. Get_Index_TwoInt ) Return
      Get_Index_TwoInt = MA_Get_Index( Int_L2e_Hdl, L_Off)
      If ( .NOT. Get_Index_TwoInt ) Return
      Get_Index_TwoInt = MA_Get_Index( Int_Scr2e_Hdl, Scr_Off)
      If ( .NOT. Get_Index_TwoInt ) Return
C
      Return
      End
