C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C NAME
C     mem_oneint -- Determine the amount of memory that is needed for
C     one-electron integral evaluation
C
C SYNOPSIS
      Integer Function Mem_OneInt(Units)
      Implicit NONE
      Integer Units
C
C ARGUMENTS
C     Units   Data type in which the memory requirements are to be
C             measured (IN)
C
C DESCRIPTION
C     Returns the amount of memory (in terms of UNITS) that would be 
C     allocated by Alloc_OneInt or Push_OneInt.
C
C     Before entry, the integral environment must have been prepared
C     with a call to Init_Integrals.
C
C REVISION
C     $Id: mem_oneint.F,v 1.1 1993/08/30 21:25:10 gg502 Exp $
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#include "stdio.fh"
#include "mafdecls.h"
c$$$#include "mafuncs.fh"
#include "integral_memory.fh"
C
C LOCAL VARIABLES
      Integer Values, Labels, Scratch
C
C     Make sure environment has been initialized
C
      If ( Int_Env .eq. 0) then
         Write (LuErr, *) 'Mem_OneInt: Integral environment has not ',
     $        'been initialized.'
         STOP
      EndIf
C
C     See if the sizes have been initialized for this environment
C
      If ( Abs(Int_Max_1e) + Abs(Int_L_Scr1e) .eq. 0)
     $     Call IntMem( Int_Env, Int_L_Scr1e, Int_L_Scr2e,
     $     Int_Max_1e, Int_Max_2e)
C
C     Space for S, T, V integrals
C
      Values = MA_SizeOf(MT_Dbl, 3 * Int_Max_1e, Units)
C
C     Space for indices I, J
C
      Labels = MA_SizeOf( MT_Int, 2 * Int_Max_1e, Units)
C
C     Scratch space
C
      Scratch = MA_SizeOf( MT_Dbl, Int_L_Scr1e, Units)
C
C     Add them up
C
      Mem_OneInt = Values + Labels + Scratch
C
      Return
      End
