C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C NAME
C     Mem_TwoInt_At -- Determine the amount of memory that is needed for
C     two-electron integral evaluation using atomic blocking.
C
C SYNOPSIS
      Integer Function Mem_TwoInt_At(Units)
      Implicit NONE
      Integer Units
C
C ARGUMENTS
C     Units   Data type in which the memory requirements are to be
C             measured (IN)
C
C DESCRIPTION
C     Returns the amount of memory (in terms of UNITS) that would be
C     allocated by Alloc_TwoInt or Push_TwoInt.
C
C     Before entry, the integral environment must have been prepared
C     with a call to Init_Integrals.
C
C REVISION
C     $Id: mem_twoint_at.F,v 1.1 1993/11/08 19:49:11 gg502 Exp $
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#include "stdio.fh"
#include "mafdecls.h"
c$$$#include "mafuncs.fh"
#include "integral_memory.fh"
C
C LOCAL VARIABLES
      Integer Values, Labels, Scratch
C
C     Make sure environment has been initialized
C
      If ( Int_Env .eq. 0) then
         Write (LuErr, *) 'Mem_TwoInt_At: Integral environment has ',
     $        'not been initialized.'
         STOP
      EndIf
C
C     See if the sizes have been initialized for this environment
C
      If ( Abs(Int_Max_2e) + Abs(Int_L_Scr2e) .eq. 0)
     $     Call IntMem( Int_Env, Int_L_Scr1e, Int_L_Scr2e,
     $     Int_Max_1e, Int_Max_2e)
C
C     Find out the number of integrals that can be expected in an
C     atomic block
C
      Call Max_Int_At(Int_Max_1e_At, Int_Max_2e_At)
C
C     Hack for Exact_Mem until we figure out what the deal is
C
      Int_L_Scr2e = Int( 1.5 * Int_L_Scr2e )
C
C     Space for integral values
C
      Values = MA_SizeOf( MT_Dbl, Int_Max_2e_At, Units)
C
C     Space for indices I, J, K, L
C
      Labels = MA_SizeOf( MT_Int, 4 * Int_Max_2e_At, Units)
C
C     Scratch space
C
      Scratch = MA_SizeOf(MT_Dbl, Int_L_Scr2e, Units)
C
C     Add them up
C
      Mem_TwoInt_At = Values + Labels + Scratch
C
      Return
      End
