C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C NAME
C     push_twoint -- Allocate memory required for the two-electron 
C     integrals on the stack and hide the handles.
C
C SYNOPSIS
      Logical Function Push_TwoInt()
      Implicit NONE
C
C DESCRIPTION
C     Allocate memory needed for evaluating two-electron integrals on
C     the stack.
C
C     Pointers to this memory can be obtained from Get_Index_TwoInt.
C
C     Before entry, the integral environment must have been prepared
C     with a call to Init_Integrals.
C
C     The amount of memory that is required for the arrays allocted
C     here is available from Mem_TwoInt.
C
C REVISION
C     $Id: push_twoint.F,v 1.1 1993/08/30 21:25:17 gg502 Exp $
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#include "stdio.fh"
#include "mafdecls.h"
c$$$#include "mafuncs.fh"
#include "integral_memory.fh"
C
C     Make sure environment has been initialized
C
      If ( Int_Env .eq. 0) then
         Write (LuErr, *) 'Push_TwoInt: Integral environment has not ',
     $        'been initialized.'
         STOP
      EndIf
C
C     See if the sizes have been initialized for this environment
C
      If ( Abs(Int_Max_2e) + Abs(Int_L_Scr2e) .eq. 0)
     $     Call IntMem( Int_Env, Int_L_Scr1e, Int_L_Scr2e,
     $     Int_Max_1e, Int_Max_2e)
C
C     Now actually allocate these things!
C
      Push_TwoInt = MA_Push_Stack( MT_DBL, Int_Max_2e,
     $    '2e Integral buffer', Int_Val_Hdl)
      If ( .NOT. Push_TwoInt ) Return
C
      Push_TwoInt = MA_Push_Stack( MT_INT, Int_Max_2e,
     $    '2e-int index I', Int_I2e_Hdl)
      If ( .NOT. Push_TwoInt ) Return
C
      Push_TwoInt = MA_Push_Stack( MT_INT, Int_Max_2e,
     $    '2e-int index J', Int_J2e_Hdl)
      If ( .NOT. Push_TwoInt ) Return
C
      Push_TwoInt = MA_Push_Stack( MT_INT, Int_Max_2e,
     $    '2e-int index K', Int_K2e_Hdl)
      If ( .NOT. Push_TwoInt ) Return
C
      Push_TwoInt = MA_Push_Stack( MT_INT, Int_Max_2e,
     $    '2e-int index L', Int_L2e_Hdl)
      If ( .NOT. Push_TwoInt ) Return
C
      Push_TwoInt = MA_Push_Stack( MT_DBL, Int_L_Scr2e,
     $    '2e-int scratch space', Int_Scr2e_Hdl)
      If ( .NOT. Push_TwoInt ) Return
C
      Return
      End
