C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C NAME
C     twoint_at_permsym -- Evaluate ERIs for a given atom quartet
C
C SYNOPSIS
      Subroutine TwoInt_At_PermSym(IntEnv, Basis, P_At, Q_At, R_At,
     $   S_At, ERI, Idx1, Idx2, Idx3, Idx4, NInts, Scr, LenScr,
     $   Canonical, Nonzero, PermSym, FPMO, FPAO)
      Implicit NONE
      Integer IntEnv, Basis, P_At, Q_At, R_At, S_At, NInts, LenScr
      Logical Canonical, Nonzero, PermSym
      Double precision FPMO, FPAO
      Integer Idx1(*), Idx2(*), Idx3(*), Idx4(*)
      Double precision ERI(*), Scr( LenScr)
C
C ARGUMENTS
C DESCRIPTION
C     An almost pointless layer over a direct call to the two-electron
C     integral package's TwoInt routine, but here we add the ability
C     to account for permutational symmetry by applying the appropriate
C     factors of two.  This tidies up the caller's routine nicely.
C
C REVISION
C     $Id: twoint_at_permsym.F,v 1.1 1993/11/08 19:49:17 gg502 Exp $
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#include "numerical_constants.fh"
C LOCAL VARIABLES
      Integer PSh, QSh, RSh, SSh
      Integer P_Sh_Lo, Q_Sh_Lo, R_Sh_Lo, S_Sh_Lo
      Integer P_Sh_Hi, Q_Sh_Hi, R_Sh_Hi, S_Sh_Hi
      Integer Q_Sh_Top, R_Sh_Top, S_Sh_Top
C
      Logical P_eq_Q, R_eq_S, PQ_eq_RS
C
      Integer N, P, Q, R, S, NIntSh
      Double precision Fact
C
C     Unfortunately, we have no idea what the operation count is for
C     the integrals at this point.  We can account for the operations
C     in applying the permutational symmetry factors, but what's the
C     point?
C
      FPMO = FP_Zero
      FPAO = FP_Zero
C
      NInts = 0
C
      Call GTO_At_to_Sh( Basis, P_At, P_Sh_Lo, P_Sh_Hi)
      Call GTO_At_to_Sh( Basis, Q_At, Q_Sh_Lo, Q_Sh_Hi)
      Call GTO_At_to_Sh( Basis, R_At, R_Sh_Lo, R_Sh_Hi)
      Call GTO_At_to_Sh( Basis, S_At, S_Sh_Lo, S_Sh_Hi)
C
C     Some tests used to determine the limits on 
C     the loops over shells.
C
      P_eq_Q = P_At .eq. Q_At
      R_eq_S = R_At .eq. S_At
      PQ_eq_RS = P_At .eq. R_At .AND. Q_At .eq. S_At
C
      Do PSh = P_Sh_Lo, P_Sh_Hi
C
         Q_Sh_Top = Q_Sh_Hi
         If ( P_eq_Q ) Q_Sh_Top = Psh
C
         Do QSh = Q_Sh_Lo, Q_Sh_Top
C
            R_Sh_Top = R_Sh_Hi
            If (PQ_eq_RS) R_Sh_Top = PSh
C
            Do RSh = R_Sh_Lo, R_Sh_Top
C
               S_Sh_Top = S_Sh_Hi
               If ( R_eq_S ) S_Sh_Top = RSh
               If ( PQ_eq_RS .AND. RSh .eq. PSh) S_Sh_Top = QSh
C
               Do SSh = S_Sh_Lo, S_Sh_Top
c$$$                     Write (6, '(A, 4I5)') 'Quartet', PSh, QSh,
c$$$     $                  RSh, SSh
c$$$                     Call FFFlush(6)
C
C                 Generate a batch of integrals
C
                  Call TwoInt(IntEnv, PSh, QSh, RSh, SSh,
     $               ERI(NInts+1),
     $               Idx1(NInts+1),
     $               Idx2(NInts+1),
     $               Idx3(NInts+1),
     $               Idx4(NInts+1),
     $               NIntSh, Scr, LenScr, Canonical, Nonzero)
C
                  NInts = NInts + NIntSh
               EndDo
            EndDo
         EndDo
      EndDo
C
C     Account for permutational symmetry if necessary, which means the
C     one or more of the shells are the same.
C
      If (PermSym .AND. ( P_At .eq. Q_At .OR. R_At .eq. S_At .OR.
     $   P_At .eq. R_At) ) then
C
         Do N = 1, NInts
            P = Idx1( N )
            Q = Idx2( N )
            R = Idx3( N )
            S = Idx4( N )
C
            Fact = FP_One
            If ( P .eq. Q ) Fact = Fact * 2
            If ( R .eq. S ) Fact = Fact * 2
            If ( P .eq. R .AND. Q .eq. S)  Fact = Fact * 2
C
            ERI( N) = ERI(N) / Fact
         EndDo
      EndIf
C
      Return
      End
