C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C NAME
C     twoint_permsym -- Evaluate ERIs for a given shell quartet
C
C SYNOPSIS
      Subroutine TwoInt_PermSym(IntEnv, Basis, Sh1, Sh2, Sh3, Sh4,
     $   ERI, Idx1, Idx2, Idx3, Idx4, NInts, Scr, LenScr,
     $   Canonical, Nonzero, PermSym, FPMO, FPAO)
      Implicit NONE
      Integer IntEnv, Basis, Sh1, Sh2, Sh3, Sh4, NInts, LenScr
      Logical Canonical, Nonzero, PermSym
      Double precision FPMO, FPAO
      Integer Idx1(*), Idx2(*), Idx3(*), Idx4(*)
      Double precision ERI(*), Scr( LenScr)
C
C ARGUMENTS
C DESCRIPTION
C     An almost pointless layer over a direct call to the two-electron
C     integral package's TwoInt routine, but here we add the ability
C     to account for permutational symmetry by applying the appropriate
C     factors of two.  This tidies up the caller's routine nicely.
C
C REVISION
C     $Id: twoint_permsym.F,v 1.1 1993/11/08 19:49:19 gg502 Exp $
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#include "numerical_constants.fh"
C LOCAL VARIABLES
      Integer N, P, Q, R, S
      Double precision Fact
C
C     Unfortunately, we have no idea what the operation count is for
C     the integrals at this point.  We can account for the operations
C     in applying the permutational symmetry factors, but what's the
C     point?
C
      FPMO = FP_Zero
      FPAO = FP_Zero
C     
C     Compute this batch of two-electron integrals
C
      Call TwoInt( IntEnv, Sh1, Sh2, Sh3, Sh4, ERI, Idx1, Idx2, Idx3,
     $   Idx4, NInts, Scr, LenScr, Canonical, NonZero)
C
C     Account for permutational symmetry if necessary, which means the
C     one or more of the shells are the same.
C
      If (PermSym .AND. ( Sh1 .eq. Sh2 .OR. Sh3 .eq. Sh4 .OR.
     $   Sh1 .eq. Sh3) ) then
C
         Do N = 1, NInts
            P = Idx1( N )
            Q = Idx2( N )
            R = Idx3( N )
            S = Idx4( N )
C
            Fact = FP_One
            If ( P .eq. Q ) Fact = Fact * 2
            If ( R .eq. S ) Fact = Fact * 2
            If ( P .eq. R .AND. Q .eq. S)  Fact = Fact * 2
C
            ERI( N) = ERI(N) / Fact
         EndDo
      EndIf
C
      Return
      End
