      Subroutine hf1(xyz,zan,ncenters,
     &               Axyz,Aprims,Acoef,NPA,La,
     &               Bxyz,Bprims,Bcoef,NPB,Lb,
     &               W0,maxW0,dryrun)
c
c$Id: hf1.f,v 1.1 1993/08/31 01:26:39 gg502 Exp $
c

      Implicit real*8 (a-h,o-z)
      Implicit integer (i-n)

      Parameter (Li=1)

      Dimension xyz(3,ncenters),zan(ncenters)

      Dimension Axyz(3),Aprims(NPA),Acoef(NPA)
      Dimension Bxyz(3),Bprims(NPB),Bcoef(NPB)

      Dimension W0(maxW0)

      Dimension Nxyz(3,2)

      logical dryrun
c******************************************************************************

c Define the number of shell components on each center.

      La2 = ((La+1)*(La+2))/2
      Lb2 = ((Lb+1)*(Lb+2))/2

c Define the angular momentum of the charge distribution.

      Lp = La + Lb

c Define the number of angular momentum functions <= Lp.

      Lp3 = ((Lp+1)*(Lp+2)*(Lp+3))/6

c Assign pointers to scratch space.

      iw1  = 1
      iw2  = iw1  + La2*Lb2
      iw3  = iw2  + La2*Lb2
      iw4  = iw3  + La2*Lb2
      iwl  = iw4  + (NPA*NPB)*3 - 1
      iwlmax = iwl

      if( iwl.gt.maxW0 )then

       write(*,*) 'HF1:  Insufficient scratch space.'
       write(*,*) '      needed    ',iwl
       write(*,*) '      allocated ',maxW0

       print *,'Sab (C):  iw1  ',iw1
       print *,'Tab (C):  iw2  ',iw2
       print *,'Vab (C):  iw3  ',iw3
       print *,'alpha  :  iw4  ',iw4 
       print *,'last   :  iwl  ',iwl,maxW0

       stop

      end if

c Define the prefactor of the charge distribution.

      if (dryrun) then
        NAB = NPA*NPB
      else
        call hf1set(Axyz,Aprims,Acoef,NPA,
     &         Bxyz,Bprims,Bcoef,NPB,
     &         W0(iw4),NAB)
      endif

c Assign pointers to scratch space.

      iw5  = iw4  + NAB*3
      iw6  = iw5  + NAB*3*((La+Li)+(Lb+Li)+1)*((La+Li)+1)*((Lb+Li)+1)
      iw7  = iw6  + NAB*Lp3
      iw8  = iw7  + (Lp+1)**3
      iw9  = iw8  + NAB*3
      iw10 = iw9  + NAB*3
      iwl  = iw10 + NAB*3 - 1
      iwlmax = max(iwl,iwlmax)
      
      if( iwl.gt.maxW0 )then

       write(*,*) 'HF1:  Insufficient scratch space.'
       write(*,*) '      needed    ',iwl
       write(*,*) '      allocated ',maxW0

       print *,'Sab (C):  iw1  ',iw1
       print *,'Tab (C):  iw2  ',iw2
       print *,'Vab (C):  iw3  ',iw3
       print *,'alpha  :  iw4  ',iw4 
       print *,'E      :  iw5  ',iw5
       print *,'R0     :  iw6  ',iw6
       print *,'NLM    :  iw7  ',iw7
       print *,'P      :  iw8  ',iw8
       print *,'PT     :  iw9  ',iw9
       print *,'AB2I   :  iw10 ',iw10
       print *,'last   :  iwl  ',iwl,maxW0

       stop

      end if

c Define the center of the charge distribution.

      if (.not.dryrun)
     &       call hfctr2(Axyz,Bxyz,W0(iw4),W0(iw8),NAB)

c Define the Hermite linear expansion coefficients.

      if (.not.dryrun)
     &       call hf1mke(Axyz,Bxyz,W0(iw4),W0(iw8),W0(iw9),W0(iw10),
     &       W0(iw5),NAB,La,Lb)

c Reuse scratch space after iw8.

      iw9  = iw8  + NAB*3
      iw10 = iw9  + NAB
      iw11 = iw10 + NAB*3
      iw12 = iw11 + NAB*2
      iwl  = iw12 + NAB*(Lp+1)*Lp3 - 1
      iwlmax = max(iwl,iwlmax)

      if( iwl.gt.maxW0 )then

       write(*,*) 'HF1:  Insufficient scratch space.'
       write(*,*) '      needed    ',iwl
       write(*,*) '      allocated ',maxW0

       print *,'Sab (C):  iw1  ',iw1
       print *,'Tab (C):  iw2  ',iw2
       print *,'Vab (C):  iw3  ',iw3
       print *,'alpha  :  iw4  ',iw4 
       print *,'E      :  iw5  ',iw5
       print *,'R0     :  iw6  ',iw6
       print *,'NLM    :  iw7  ',iw7
       print *,'P      :  iw8  ',iw8
       print *,'RS     :  iw9  ',iw9
       print *,'PC     :  iw10 ',iw10
       print *,'ff     :  iw11 ',iw11
       print *,'Rj     :  iw12 ',iw12
       print *,'last   :  iwl  ',iwl,maxW0

       stop

      end if

c Define the auxiliary function integrals.

      if (.not.dryrun)
     &       call hf1mkr(xyz,zan,ncenters,
     &       W0(iw4),W0(iw8),W0(iw9),W0(iw10),W0(iw11),W0(iw12),
     &       W0(iw6),W0(iw7),NAB,Lp,Lp3)

c Reuse scratch space after iw7.

      iw8  = iw7  + (Lp+1)**3
      iwl  = iw8  + NAB - 1
      iwlmax = max(iwl,iwlmax)
      
      if( iwl.gt.maxW0 )then

       write(*,*) 'HF1:  Insufficient scratch space.'
       write(*,*) '      needed    ',iwl
       write(*,*) '      allocated ',maxW0

       print *,'Sab (C):  iw1  ',iw1
       print *,'Tab (C):  iw2  ',iw2
       print *,'Vab (C):  iw3  ',iw3
       print *,'alpha  :  iw4  ',iw4 
       print *,'E      :  iw5  ',iw5
       print *,'R0     :  iw6  ',iw6
       print *,'NLM    :  iw7  ',iw7
       print *,'Ti     :  iw8  ',iw8
       print *,'last   :  iwl  ',iwl,maxW0

       stop

      end if
c
c if dryrun return memory in maxw0 value
c
      if (dryrun) then
        maxw0 = iwlmax
        return
      endif

c Loop over shell components.

      mm = -1
      do 30 ica = 1,La2
       do 20 icb = 1,Lb2
        mm = mm + 1

c Define the angular momentum indices.

        call defNxyz(La,ica,Nxyz(1,1))
        call defNxyz(Lb,icb,Nxyz(1,2))

c Compute <a|S|b>.

        call hfasb(W0(iw5),Nxyz,W0(iw1+mm),NAB,La,Lb)

c Compute <a|T|b>.

        call hfatb(W0(iw4),W0(iw5),Nxyz,W0(iw2+mm),W0(iw8),NAB,La,Lb)

c Compute <a|V|b>.

        call hfavb(W0(iw5),W0(iw6),W0(iw7),Nxyz,W0(iw3+mm),NAB,La,Lb)

   20  continue
   30 continue

      end
