      Subroutine hf1mke(Axyz,Bxyz,alpha,P,PT,AB2I,E,NAB,La,Lb)
c
c$Id: hf1mke.f,v 1.1 1993/08/31 01:26:40 gg502 Exp $
c

      Implicit real*8 (a-h,o-z)
      Implicit integer (i-n)

      Parameter (Li=1)

      Dimension Axyz(3),Bxyz(3),alpha(3,NAB),P(3,NAB)

      Dimension PT(NAB,3),AB2I(3*NAB)

      Dimension E(NAB,3,0:((La+Li)+(Lb+Li)),0:(La+Li),0:(Lb+Li))
c
c Define the Hermite linear expansion coefficients for the product of 
c two monomials. These coefficients are scaled by a factor appropriate
c for 2-ctr OIs, defined as
c
c         /   PI  \ 3/2     /   a b   __2 \
c   ES = | ------- |    EXP| - -----  AB   |
c         \ a + b /         \  a + b      /
c
c This scaling factor is returned as the 3rd index of the exponents array
c (i.e., "alpha(3,m)").
c
c     Recursion Formulae:
c
c       0,0
c     Ex    = ES
c       0
c
c       0,0
c     Ey    = 1.0
c       0
c
c       0,0
c     Ez    = 1.0
c       0
c
c       Ia+1,Ib          Ia,Ib         Ia,Ib            Ia,Ib
c     Ex        = AB2I Ex      + PAx Ex      + (Ip+1) Ex
c       Ip               Ip-1          Ip               Ip+1
c
c       Ia,Ib+1          Ia,Ib         Ia,Ib            Ia,Ib
c     Ex        = AB2I Ex      + PBx Ex      + (Ip+1) Ex
c       Ip               Ip-1          Ip               Ip+1
c
c     Indices for E(m,k,Ip,Ia,Ib):
c
c     1 [  m  ] NAB
c     1 [  k  ] 3     {X,Y,Z}
c     0 [  Ip ] Ia+Ib
c     0 [  Ia ] La+Li
c     0 [  Ib ] Lb+Li
c
c     N.B.:  Ia & Ib must be incremented 1 angular momentum unit beyond 
c            La & Lb to satisfy requirements of the kinetic integrals.
c            To facilitate this requirement, the parameter "Li" is set 
c            equal to 1 for the 1 e- integrals.
c
c******************************************************************************

c Initialize the Hermite expansion coefficients.

      do 10 m = 1,3*NAB*((La+Li)+(Lb+Li)+1)*((La+Li)+1)*((Lb+Li)+1)
       E(m,1,0,0,0) = 0.D0
   10 continue

c Define E(Ip,Ia,Ib) for Ip=0, Ia=0, Ib=0.

      do 100 m = 1,NAB

       E(m,1,0,0,0) = alpha(3,m)
       E(m,2,0,0,0) = 1.D0
       E(m,3,0,0,0) = 1.D0

       AB2I(m      ) = 0.5D0/(alpha(1,m) + alpha(2,m))
       AB2I(m+1*NAB) = AB2I(m)
       AB2I(m+2*NAB) = AB2I(m)

       PT(m,1) = P(1,m) - Axyz(1)
       PT(m,2) = P(2,m) - Axyz(2)
       PT(m,3) = P(3,m) - Axyz(3)

  100 continue

c Define E(Ip,Ia,Ib) for Ip=0,Ia+Ib, Ia=1,(La+Li), Ib=0.

      do 250 Ia = 1,(La+Li)

c                          ===>   Ip = 0   <===

       do 210 m = 1,3*NAB
        E(m,1,0,Ia,0) =   PT(m,1)*E(m,1,0,Ia-1,0)
     &                  +         E(m,1,1,Ia-1,0)
  210  continue

c                       ===>   Ip = 1,Ia-1   <===

        do 230 Ip = 1,Ia-1

         do 220 m = 1,3*NAB
          E(m,1,Ip,Ia,0) =   AB2I(m)*E(m,1,Ip-1,Ia-1,0)
     &                     + PT(m,1)*E(m,1,Ip  ,Ia-1,0)
     &                     +  (Ip+1)*E(m,1,Ip+1,Ia-1,0)
  220   continue

  230  continue

c                         ===>   Ip = Ia   <===

       Ip = Ia

       do 240 m = 1,3*NAB
        E(m,1,Ip,Ia,0) =   AB2I(m)*E(m,1,Ip-1,Ia-1,0)
     &                   + PT(m,1)*E(m,1,Ip  ,Ia-1,0)
  240  continue

  250 continue

c Define E(Ip,Ia,Ib) for Ip=0,Ia+Ib, Ia=0,(La+Li), Ib=1,(Lb+Li).

      do 300 m = 1,NAB
       PT(m,1) = P(1,m) - Bxyz(1)
       PT(m,2) = P(2,m) - Bxyz(2)
       PT(m,3) = P(3,m) - Bxyz(3)
  300 continue

      do 360 Ib = 1,(Lb+Li)

       do 350 Ia = 0,(La+Li)

c                          ===>   Ip = 0   <===

        do 310 m = 1,3*NAB
         E(m,1,0,Ia,Ib) =   PT(m,1)*E(m,1,0,Ia,Ib-1)
     &                    +         E(m,1,1,Ia,Ib-1)
  310   continue

c                       ===>   Ip = 1,Ia+Ib-1   <===

        do 330 Ip = 1,Ia+Ib-1

         do 320 m = 1,3*NAB
          E(m,1,Ip,Ia,Ib) =   AB2I(m)*E(m,1,Ip-1,Ia,Ib-1)
     &                      + PT(m,1)*E(m,1,Ip  ,Ia,Ib-1)
     &                      +  (Ip+1)*E(m,1,Ip+1,Ia,Ib-1)
  320    continue

  330   continue

c                        ===>   Ip = Ia+Ib   <===

        Ip = Ia + Ib

        do 340 m = 1,3*NAB
         E(m,1,Ip,Ia,Ib) =   AB2I(m)*E(m,1,Ip-1,Ia,Ib-1)
     &                     + PT(m,1)*E(m,1,Ip  ,Ia,Ib-1)
  340   continue

  350  continue

  360 continue

      end
