      Subroutine hf2(Axyz,Aprims,Acoef,NPA,La,
     &               Bxyz,Bprims,Bcoef,NPB,Lb,
     &               Cxyz,Cprims,Ccoef,NPC,Lc,
     &               Dxyz,Dprims,Dcoef,NPD,Ld,
     &               W0,maxW0,dryrun)
c
c$Id: hf2.f,v 1.1 1993/08/31 01:26:47 gg502 Exp $
c

      Implicit real*8 (a-h,o-z)
      Implicit integer (i-n)

      Dimension Axyz(3),Aprims(NPA),Acoef(NPA)
      Dimension Bxyz(3),Bprims(NPB),Bcoef(NPB)
      Dimension Cxyz(3),Cprims(NPC),Ccoef(NPC)
      Dimension Dxyz(3),Dprims(NPD),Dcoef(NPD)

      Dimension W0(maxW0)
      
      Logical dryrun
c******************************************************************************

c Define the number of shell components on each center.

      La2 = ((La+1)*(La+2))/2
      Lb2 = ((Lb+1)*(Lb+2))/2
      Lc2 = ((Lc+1)*(Lc+2))/2
      Ld2 = ((Ld+1)*(Ld+2))/2

c Define the angular momentum of the charge distribution.

      Lp = La + Lb
      Lq = Lc + Ld
      Lr = Lp + Lq

c Define the number of angular momentum functions <= Lr.

      Lr3 = ((Lr+1)*(Lr+2)*(Lr+3))/6

c Assign pointers to scratch space.

      iw1  = 1
      iw2  = iw1  + La2*Lb2*Lc2*Ld2
      iwl  = iw2  + (NPA*NPB)*3 - 1
      iwlmax = iwl

      if( iwl.gt.maxW0 )then

       write(*,*) 'HF2:  Insufficient scratch space.'
       write(*,*) '      needed    ',iwl
       write(*,*) '      allocated ',maxW0

       print *,'ERI (C):  iw1  ',iw1
       print *,'alpha_p:  iw2  ',iw2 
       print *,'last   :  iwl  ',iwl,maxW0

       stop

      end if

c Define the prefactor of the charge distribution (P).

      if (dryrun) then
        NAB = NPA*NPB
      else
        call hf2set(Axyz,Aprims,Acoef,NPA,
     &         Bxyz,Bprims,Bcoef,NPB,
     &         W0(iw2),NAB)
      endif

c Assign pointers to scratch space.

      iw3  = iw2  + NAB*3
      iwl  = iw3  + (NPC*NPD)*3 - 1
      iwlmax = max(iwl,iwlmax)

      if( iwl.gt.maxW0 )then

       write(*,*) 'HF2:  Insufficient scratch space.'
       write(*,*) '      needed    ',iwl
       write(*,*) '      allocated ',maxW0

       print *,'ERI (C):  iw1  ',iw1
       print *,'alpha_p:  iw2  ',iw2 
       print *,'alpha_q:  iw3  ',iw3 
       print *,'last   :  iwl  ',iwl,maxW0

       stop

      end if

c Define the prefactor of the charge distribution (Q).

      if (dryrun) then
        NCD = NPC*NPD
      else
        call hf2set(Cxyz,Cprims,Ccoef,NPC,
     &         Dxyz,Dprims,Dcoef,NPD,
     &         W0(iw3),NCD)
      endif

c Assign pointers to scratch space.

      NABCD = NAB*NCD

      iw4  = iw3  + NCD*3
      iw5  = iw4  + NAB*3*(La+Lb+1)*(La+1)*(Lb+1)
      iw6  = iw5  + NCD*3*(Lc+Ld+1)*(Lc+1)*(Ld+1)
      iw7  = iw6  + NABCD*Lr3
      iw8  = iw7  + (Lr+1)**3
      iw9  = iw8  + NAB*3
      iw10 = iw9  + NCD*3
      iw11 = iw10 + max(NAB,NCD)*3
      iwl  = iw11 + max(NAB,NCD)*3 - 1
      iwlmax = max(iwl,iwlmax)

      if( iwl.gt.maxW0 )then

       write(*,*) 'HF2:  Insufficient scratch space.'
       write(*,*) '      needed    ',iwl
       write(*,*) '      allocated ',maxW0

       print *,'ERI (C):  iw1  ',iw1
       print *,'alpha_p:  iw2  ',iw2 
       print *,'alpha_q:  iw3  ',iw3 
       print *,'Ep     :  iw4  ',iw4
       print *,'Eq     :  iw5  ',iw5
       print *,'R0     :  iw6  ',iw6
       print *,'NLM    :  iw7  ',iw7
       print *,'P      :  iw8  ',iw8
       print *,'Q      :  iw9  ',iw9
       print *,'PT     :  iw10 ',iw10
       print *,'AB2I   :  iw11 ',iw11
       print *,'last   :  iwl  ',iwl,maxW0

       stop

      end if

c Define the center of the charge distributions.

      if (.not.dryrun) then

        call hfctr2(Axyz,Bxyz,W0(iw2),W0(iw8),NAB)

        call hfctr2(Cxyz,Dxyz,W0(iw3),W0(iw9),NCD)

c Define the Hermite linear expansion coefficients.

        call hf2mke(Axyz,Bxyz,W0(iw2),W0(iw8),W0(iw10),W0(iw11),
     &            W0(iw4),NAB,La,Lb)

        call hf2mke(Cxyz,Dxyz,W0(iw3),W0(iw9),W0(iw10),W0(iw11),
     &            W0(iw5),NCD,Lc,Ld)

      endif

c Assign pointers to scratch space.

      iw10 = iw9  + NCD*3
      iw11 = iw10 + NABCD*3
      iw12 = iw11 + NABCD*2
      iwl  = iw12 + NABCD*(Lr+1)*Lr3 - 1
      iwlmax = max(iwl,iwlmax)

      if( iwl.gt.maxW0 )then

       write(*,*) 'HF2:  Insufficient scratch space.'
       write(*,*) '      needed    ',iwl
       write(*,*) '      allocated ',maxW0

       print *,'ERI (C):  iw1  ',iw1
       print *,'alpha_p:  iw2  ',iw2 
       print *,'alpha_q:  iw3  ',iw3 
       print *,'Ep     :  iw4  ',iw4
       print *,'Eq     :  iw5  ',iw5
       print *,'R0     :  iw6  ',iw6
       print *,'NLM    :  iw7  ',iw7
       print *,'P      :  iw8  ',iw8
       print *,'Q      :  iw9  ',iw9
       print *,'PQ     :  iw10 ',iw10
       print *,'ff     :  iw11 ',iw11
       print *,'Rj     :  iw12 ',iw12
       print *,'last   :  iwl  ',iwl,maxW0

       stop

      end if

c Define the auxiliary function integrals.

      if (.not.dryrun)
     &       call hf2mkr
     &       (W0(iw2),W0(iw3),W0(iw8),W0(iw9),W0(iw10),W0(iw11),
     &        W0(iw12),W0(iw6),W0(iw7),NAB,NCD,Lr,Lr3)

c Assign pointers to scratch space.

      Lp3 = ((Lp+1)*(Lp+2)*(Lp+3))/6
      Lq3 = ((Lq+1)*(Lq+2)*(Lq+3))/6

      iw8  = iw7  + (Lr+1)**3
      iw9  = iw8  + NAB*Lp3
      iw10 = iw9  + NCD*Lq3
      iwl  = iw10 + NABCD - 1
      iwlmax = max(iwl,iwlmax)

      if( iwl.gt.maxW0 )then

       write(*,*) 'HF2:  Insufficient scratch space.'
       write(*,*) '      needed    ',iwl
       write(*,*) '      allocated ',maxW0

       print *,'ERI (C):  iw1  ',iw1
       print *,'alpha_p:  iw2  ',iw2 
       print *,'alpha_q:  iw3  ',iw3 
       print *,'Ep     :  iw4  ',iw4
       print *,'Eq     :  iw5  ',iw5
       print *,'R0     :  iw6  ',iw6
       print *,'NLM    :  iw7  ',iw7
       print *,'EEp    :  iw8  ',iw8
       print *,'EEq    :  iw9  ',iw9
       print *,'sum    :  iw10 ',iw10
       print *,'last   :  iwl  ',iwl,maxW0

       stop

      end if
      if (dryrun) then
        maxW0 =  iwlmax
        return
      endif

c Compute the ERIs.

      call hferi(W0(iw4),W0(iw5),W0(iw6),W0(iw7),W0(iw8),W0(iw9),
     &           W0(iw10),W0(iw1),NAB,NCD,La,Lb,Lc,Ld)

      end
