      Subroutine hfmke(Axyz,Bxyz,alpha,P,PA,PB,AB2I,E,NAB,La,Lb,Li)
c
c$Id: hfmke.f,v 1.1 1993/08/31 01:27:05 gg502 Exp $
c

      Implicit real*8 (a-h,o-z)
      Implicit integer (i-n)

      Dimension Axyz(3),Bxyz(3),alpha(3,NAB),P(3,NAB)

      Dimension PA(NAB,3),PB(NAB,3),AB2I(3*NAB)

      Dimension E(NAB,3,0:((La+Li)+(Lb+Li)),0:(La+Li),0:(Lb+Li))
c
c     Recursion Formulae:
c
c       0,0
c     Ex    = ES
c       0
c
c       0,0
c     Ey    = 1.0
c       0
c
c       0,0
c     Ez    = 1.0
c       0
c
c       Ia+1,Ib          Ia,Ib         Ia,Ib           Ia,Ib
c     Ex        = AB2I Ex      + PAx Ex      + (N+1) Ex
c       N                N-1           N               N+1
c
c       Ia,Ib+1          Ia,Ib         Ia,Ib           Ia,Ib
c     Ex        = AB2I Ex      + PBx Ex      + (N+1) Ex
c       N                N-1           N               N+1
c
c     Indices for E(m,k,N,Ia,Ib):
c
c     1 [  m  ] NAB
c     1 [  k  ] 3     {X,Y,Z}
c     0 [  N  ] Ia+Ib
c     0 [  Ia ] La+Li
c     0 [  Ib ] Lb+Li
c
c     N.B.:  ES is a scaling factor.
c
c            For 1 e- integrals, ES should be 
c
c                      /   PI  \ 3/2      /   a b   __2 \
c                     | ------- |     EXP| - -----  AB   |
c                      \ a + b /          \  a + b      /
c
c            For 2 e- integrals, ES should be 
c
c                        1/2  5/4
c                       2   PI        /   a b   __2 \
c                      --------   EXP| - -----  AB   |
c                        a + b        \  a + b      /
c        
c     N.B.:  Ia & Ib must be incremented 1 angular momentum unit beyond 
c            La & Lb to satisfy requirements of the kinetic integrals.
c            To facilitate this requirement, the variable "Li" should be
c            set to 1 for the 1 e- integrals and 0 for the 2 e- integrals.
c
c******************************************************************************

c Initialize the Hermite expansion coefficients.

      do 10 m = 1,3*NAB*((La+Li)+(Lb+Li)+1)*((La+Li)+1)*((Lb+Li)+1)
       E(m,1,0,0,0) = 0.D0
   10 continue

c Define E(N,Ia,Ib) for N=0, Ia=0, Ib=0.

      do 100 m = 1,NAB

       E(m,1,0,0,0) = alpha(3,m)
       E(m,2,0,0,0) = 1.D0
       E(m,3,0,0,0) = 1.D0

       AB2I(m      ) = 0.5D0/(alpha(1,m) + alpha(2,m))
       AB2I(m+1*NAB) = AB2I(m)
       AB2I(m+2*NAB) = AB2I(m)

       PA(m,1) = P(1,m) - Axyz(1)
       PA(m,2) = P(2,m) - Axyz(2)
       PA(m,3) = P(3,m) - Axyz(3)

       PB(m,1) = P(1,m) - Bxyz(1)
       PB(m,2) = P(2,m) - Bxyz(2)
       PB(m,3) = P(3,m) - Bxyz(3)

  100 continue

c Define E(N,Ia,Ib) for N=0, Ia+Ib, Ia=1,(La+Li), Ib=0.

      do 240 Ia = 1,(La+Li)

c                          ===>   N = 0   <===

       do 200 m = 1,3*NAB
        E(m,1,0,Ia,0) = PA(m,1)*E(m,1,0,Ia-1,0) + E(m,1,1,Ia-1,0)
  200  continue

c                       ===>   N = 1,Ia-1   <===

        do 220 N = 1,Ia-1

         do 210 m = 1,3*NAB
          E(m,1,N,Ia,0) =   AB2I(m)*E(m,1,N-1,Ia-1,0)
     &                    + PA(m,1)*E(m,1,N  ,Ia-1,0)
     &                    +   (N+1)*E(m,1,N+1,Ia-1,0)
  210   continue

  220  continue

c                         ===>   N = Ia   <===

       N = Ia

       do 230 m = 1,3*NAB
        E(m,1,N,Ia,0) =   AB2I(m)*E(m,1,N-1,Ia-1,0)
     &                  + PA(m,1)*E(m,1,N  ,Ia-1,0)
  230  continue

  240 continue

c Define E(N,Ia,Ib) for N=0,Ia+Ib, Ia=0,(La+Li), Ib=1,(Lb+Li).

      do 350 Ib = 1,(Lb+Li)

       do 340 Ia = 0,(La+Li)

c                          ===>   N = 0   <===

        do 300 m = 1,3*NAB
         E(m,1,0,Ia,Ib) = PB(m,1)*E(m,1,0,Ia,Ib-1) + E(m,1,1,Ia,Ib-1)
  300   continue

c                       ===>   N = 1,Ia+Ib-1   <===

        do 320 N = 1,Ia+Ib-1

         do 310 m = 1,3*NAB
          E(m,1,N,Ia,Ib) =   AB2I(m)*E(m,1,N-1,Ia,Ib-1)
     &                     + PB(m,1)*E(m,1,N  ,Ia,Ib-1)
     &                     +   (N+1)*E(m,1,N+1,Ia,Ib-1)
  310    continue

  320   continue

c                        ===>   N = Ia+Ib   <===

        N = Ia + Ib

        do 330 m = 1,3*NAB
         E(m,1,N,Ia,Ib) =   AB2I(m)*E(m,1,N-1,Ia,Ib-1)
     &                    + PB(m,1)*E(m,1,N  ,Ia,Ib-1)
  330   continue

  340  continue

  350 continue

      end
