      subroutine twoint(ienv,ish, jsh, ksh, lsh,ERI,
     &       indi,indj,indk,indl,nint,
     &       scratch,lscratch,ocanon,ozero)
c
c$Id: twoint.f,v 1.1 1993/08/31 01:27:15 gg502 Exp $
c
c
c:description
c twoint returns all integrals and basis function indecies 
c with canonical indexes that are generated over shells 
c ish,jsh,ksh,lsh.  zero integrals are removed.  
c 
c Input:
c       ish  shell index.  Note:
c       jsh  shell index.  All shell indecies must be canonical
c       ksh  shell index.  e.g., ish >= jsh, ksh >= lsh, ijsh >= klsh
c       lsh  shell index.
c       scratch(*) scratch array used in integral computation
c       lscratch  length of scratch array. If lscrach=0 then a call 
c                 is made to icmem2 to caluclate the required length.
c       NOTE: The requisite memory maximums for ERI(*),indx(*), and 
c             scratch are returned from intmem.  
c       ocanon   determines whether to return canonical indecies
c       ozero    determines whether or not to remove zero integrals
c Output:
c        ERI(*)  array that holds non-zero (ERI's) integrals. 
c        indi(*) array that holds i basis function index for ERI's
c        indj(*)  "     "     "   j   "      "       "    "   "
c        indk(*)  "     "     "   k   "      "       "    "   "
c        indl(*)  "     "     "   l   "      "       "    "   "
c
c Denmat Arrays used:
c                    ktype(*)
c                    kng(*)
c                    katom(*)
c                    kstart(*)
c                    kloc(*)
c
c Statement functions:
c                     isym2(i,j)  returns canonical ij index
c
      
      implicit none
      include 'int.h'
c: passed
      integer ienv,ish,jsh,ksh,lsh,lscratch,nint
      integer indi(*),indj(*),indk(*),indl(*)
      real*8 ERI(*),scratch(*)
      logical ocanon,ozero
c:local
      real*8 tol2e
      parameter (tol2e=1.0d-10)
      integer i,j,La,npa,Lb,npb,Lc,npc,Ld,npd
      integer La2,Lb2,Lc2,Ld2,ioff,joff,koff,loff,icount
      integer idum,jdum,ijdum,kldum,kdum,ldum,l,k
      logical allishl
c
      integer min,max,isym2
      intrinsic min,max
      integer icmem2
      external icmem2
      external hf2
      external canonical
c
      isym2(i,j)=(max(i,j)*(max(i,j)-1))/2 + min(i,j)
c
c check that environment is okay
c
      if (ienv.ne.565) then
        write(6,*)' environment not set call intinit first '
        stop ' env error '
      endif
c
      
      La  = ktype(ish)-1
      npa = kng(ish)
      Lb  = ktype(jsh)-1
      npb = kng(jsh)
      Lc  = ktype(ksh)-1
      npc = kng(ksh)
      Ld  = ktype(lsh)-1
      npd = kng(lsh)
      
c figure out length of scratch??
      if (lscratch.eq.0) then
        call hf2(
     &         c(1,katom(ish)),ex(kstart(ish)),
     &         cspd(kstart(ish),ktype(ish)),NPA,La,
     &         c(1,katom(jsh)),ex(kstart(jsh)),
     &         cspd(kstart(jsh),ktype(jsh)),NPB,Lb,
     &         c(1,katom(ksh)),ex(kstart(ksh)),
     &         cspd(kstart(ksh),ktype(ksh)),NPC,Lc,
     &         c(1,katom(lsh)),ex(kstart(lsh)),
     &         cspd(kstart(lsh),ktype(lsh)),NPD,Ld,
     &         scratch,lscratch,.true.)
        
      endif
      
      call hf2(
     &       c(1,katom(ish)),ex(kstart(ish)),
     &       cspd(kstart(ish),ktype(ish)),NPA,La,
     &       c(1,katom(jsh)),ex(kstart(jsh)),
     &       cspd(kstart(jsh),ktype(jsh)),NPB,Lb,
     &       c(1,katom(ksh)),ex(kstart(ksh)),
     &       cspd(kstart(ksh),ktype(ksh)),NPC,Lc,
     &       c(1,katom(lsh)),ex(kstart(lsh)),
     &       cspd(kstart(lsh),ktype(lsh)),NPD,Ld,
     &       scratch,lscratch,.false.)
      
      nint = 0 
      icount = 0
      
      La2 = (La+1)*(La+2)/2
      Lb2 = (Lb+1)*(Lb+2)/2
      Lc2 = (Lc+1)*(Lc+2)/2
      Ld2 = (Ld+1)*(Ld+2)/2
      
      ioff = kloc(ish)-1
      joff = kloc(jsh)-1
      koff = kloc(ksh)-1
      loff = kloc(lsh)-1
      
      allishl = isym2(ish,jsh).eq.isym2(ksh,lsh)
      
      do 00100 i=1,La2
        idum = ioff+i
        
        do 00200 j=1,Lb2
          jdum = joff + j
          ijdum = isym2(idum,jdum)
          
          do 00300 k=1,Lc2
            kdum = koff + k
            
            do 00400 l=1,Ld2
              ldum = loff + l
              kldum = isym2(kdum,ldum)
              
              icount = icount + 1
c. . . . . . . . . . . . . . . . . . . . . . ! check threshold if required
              if (ozero.and.(abs(scratch(icount)).gt.tol2e)) then
c
c since shells are canonical if all shells are equal in their pair index
c basis function ij must be .ge. kl.  if shells are not equal in their 
c pair index then the restricion over basis function ij.ge.kl is ignored
c This gives all integrals once !!!!!
c
                if (   idum.ge.jdum.and.kdum.ge.ldum.and.
     &                 ((.not.allishl).or.ijdum.ge.kldum)) then
                  nint = nint + 1
                  ERI(nint) = scratch(icount)
                  if (ocanon) then
c. . . . . . . . . . . . . . . . . . . . . . . . . . . ! do canonicalize
                    call canonical( idum, jdum, kdum, ldum,
     &                     indi(nint), indj(nint),
     &                     indk(nint), indl(nint)   )
                  else
c. . . . . . . . . . . . . . . . . . . . . . . . . . . ! do not canonicalize
                    indi(nint) = idum
                    indj(nint) = jdum
                    indk(nint) = kdum
                    indl(nint) = ldum
                  endif
                endif
              else
c.................................! do not check for zero's 
c
c since shells are canonical if all shells are equal in their pair index
c basis function ij must be .ge. kl.  if shells are not equal in their 
c pair index then the restricion over basis function ij.ge.kl is ignored
c This gives all integrals once !!!!!
c
                if (   idum.ge.jdum.and.kdum.ge.ldum.and.
     &                 ((.not.allishl).or.ijdum.ge.kldum)) then
                  nint = nint + 1
                  ERI(nint) = scratch(icount)
                  if (ocanon) then
c. . . . . . . . . . . . . . . . . . . . . . . . . . . ! do canonicalize
                    call canonical( idum, jdum, kdum, ldum,
     &                     indi(nint), indj(nint),
     &                     indk(nint), indl(nint)   )
                  else
c. . . . . . . . . . . . . . . . . . . . . . . . . . . ! do not canonicalize
                    indi(nint) = idum
                    indj(nint) = jdum
                    indk(nint) = kdum
                    indl(nint) = ldum
                  endif
                endif
              endif
00400       continue
00300     continue
00200   continue
00100 continue
      end
