#ifdef SNGLPR
      Subroutine SGeMA (TranA, M, N, Alpha, A, LDA,
     $   Gamma, C, LDC)
#else
      Subroutine DGeMA (TranA, M, N, Alpha, A, LDA,
     $   Gamma, C, LDC)
#endif
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C $Id: dgema.F,v 1.1 1993/08/30 21:25:27 gg502 Exp $
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C NAME
C     dgema, sgema -- general matrix addition
C
C SYNOPSIS
      Character*1 TranA
      Integer M, N, LDA, LDC
#ifdef SNGLPR
      Real Alpha, Gamma
      Real A(LDA,*), C(LDC,*)
#else
      Double precision Alpha, Gamma
      Double precision A(LDA,*), C(LDC,*)
#endif
C
C ARGUMENTS
C     TranA   Use transpose of A (input)
C     M       Computational row dimension (input)
C     N       Computational column dimension (input)
C     Alpha   Coefficient for A (input)
C     A       Matrix (input)
C     LDA     Leading dimension of A (input)
C     Gamma   Coefficient for C (input)
C     C       Result matrix (modified)
C     LDC     Leading dimension of C (input)
C
C DESCRIPTION
C     Performs the operations C <-- aA + cC for general matrices.
C
C ROUTINES REQUIRED
C     From BLAS -- Support routines
      Logical LSame
      External LSame, XErBLA
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C LOCAL VARIABLES
C     Info    For argument sanity checks
C
      Integer Info
#ifdef SNGLPR
      Real RZero, ROne
      Parameter (RZero = 0.0, ROne = 1.0)
#else
      Double Precision RZero, ROne
      Parameter (RZero = 0.0d0, ROne = 1.0d0)
#endif
C
      Integer i, j
C
C     Test input parameters
C
      Info = 0
      If ( .not. (LSame(TranA, 'N') .OR. LSame(TranA, 'T')) ) then
         Info = 1
      ElseIf (M .lt. 0) then
         Info = 2
      ElseIf (N .lt. 0) then
         Info = 3
      ElseIf (LDA .lt. Max(1, M)) then
         Info = 6
      ElseIf (LDC .lt. Max(1, M)) then
         Info = 9
      EndIf
      If (Info .ne. 0) then
#ifdef SNGLPR
         Call XErBLA('SGEMA', Info)
#else
         Call XErBLA('DGEMA', Info)
#endif
         Return
      EndIf
C
C     See if we can get away easily
C
      If (M .eq. 0 .OR. N .eq. 0) Return
C
C     Zero or scale the result as necessary
C
      If (Gamma .eq. RZero) then
         Do 100 j = 1, N
            Do 110 i = 1, M
               C(i,j) = RZero
 110        Continue
 100     Continue
      ElseIf (Gamma .ne. ROne) then
         Do 120 j = 1, N
            Do 130 i = 1, j
               C(i,j) = Gamma * C(i,j)
 130        Continue
 120     Continue
      EndIf
C
C     See if we can get out now
C
      If (Alpha .eq. RZero) Return
C
C     Does A really get scaled?
C
      If ( Alpha .eq. ROne) then
         If (LSame(TranA, 'N')) then
C
C           C <-- C + aA
C
            Do 1000 j = 1, N
               Do 1010 i = 1, M
                  C(i,j) = C(i,j) + A(i,j)
 1010          Continue
 1000       Continue
C
C           C <-- C + aA'
C
         Else
            Do 1040 j = 1, N
               Do 1050 i = 1, M
                  C(i,j) = C(i,j) + A(j,i)
 1050          Continue
 1040       Continue
         EndIf
C
C        We have to do the whole thing, including scaling of A
C
      Else
         If (LSame(TranA, 'N')) then
C
C           C <-- C + aA
C
            Do 2000 j = 1, N
               Do 2010 i = 1, M
                  C(i,j) = C(i,j) + Alpha * A(i,j)
 2010          Continue
 2000       Continue
C
C           C <-- C + aA'
C
         Else
            Do 2040 j = 1, N
               Do 2050 i = 1, M
                  C(i,j) = C(i,j) + Alpha * A(j,i)
 2050          Continue
 2040       Continue
         EndIf
      EndIf
C
      Return
      End
