C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C NAME
C     get_ao_fock_matrix -- Extract the AO basis Fock matrix from the
C     run-time database.
C
C SYNOPSIS
      Logical Function Get_AO_Fock_Matrix(N, F, LDF)
      Implicit NONE
      Integer N, LDF
      Double Precision F(LDF, N)
C
C ARGUMENTS
C     N       Dimension of Fock matrix (IN)
C     F       Fock matrix (OUT)
C     LDF     Leading dimension of Fock matrix (IN)
C
C DESCRIPTION
C     Reads AO Fock matrix from the run-time database.  At this time,
C     this amounts to a GETARRAY call to the pdft10 file.
C
C MEMORY USAGE
C     Temporarily uses N(N+1)/2 MT_Dbls on the stack to read symmetric 
C     packed format Fock matrix from pdft10.
C
C REVISION
C $Id: get_ao_fock_matrix.F,v 1.2 1993/10/03 10:53:47 gg502 Exp $
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#include "stdio.fh"
#include "denmat_iostuff.fh"
#include "denmat_funcs.fh"
#include "mafdecls.h"
C
C LOCAL VARIABLES
      Integer F_SP_Hdl, F_SP_Ind, F_SP_Size
      Logical LResult
C
C     This is the size of the symmetric packed matrix
C
      F_SP_Size = N * (N+1) / 2
C
C     Allocate space to read the symmetric packed Fock matrix
C
      LResult = MA_Push_Get( MT_Dbl, F_SP_Size,
     $   'Temp. for Sym. packed Fock matrix', F_SP_Hdl, F_SP_Ind)
C
      If ( .NOT. LResult ) then
         Write (LuErr, *) ' Get_AO_Fock_Matrix: Unable to allocate ',
     $      ' scratch space on the stack.'
         Get_AO_Fock_Matrix = .FALSE.
         Return
      EndIf
C
      Call DGetArray( ICk, Dbl_MB( F_SP_Ind), F_SP_Size, 360)
C
C     Convert this beast to full general storage
C
      Call DSP2GE( 'U', N, DBl_MB( F_SP_Ind ), F, LDF)
C
C     Free the scratch space
C
      LResult = MA_Pop_Stack( F_SP_Hdl)
C
      If ( .NOT. LResult ) then
         Write (LuErr, *) ' Get_AO_Fock_Matrix: Unable to free ',
     $      ' scratch space on the stack.'
         Get_AO_Fock_Matrix = .FALSE.
         Return
      EndIf
C
C     All work completed
C
      Get_AO_Fock_Matrix = .TRUE.
      Return
      End
