C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C NAME
C     Get_AO_Metric -- Extract the AO basis metrix (overlap matrix)
C     from the run-time database.
C
C SYNOPSIS
      Logical Function Get_AO_Metric(N, S, LDS)
      Implicit NONE
      Integer N, LDS
      Double Precision S(LDS, N)
C
C ARGUMENTS
C     N       Dimension of metric (IN)
C     S       Metric (OUT)
C     LDS     Leading dimension of metric (IN)
C
C DESCRIPTION
C     Reads AO metric from the run-time database.  At this time,
C     this amounts to a GETARRAY call to the pdft10 file.
C
C MEMORY USAGE
C     Temporarily uses N(N+1)/2 MT_Dbls on the stack to read symmetric 
C     packed format metric from pdft10.
C
C REVISION
C $Id: get_ao_metric.F,v 1.2 1993/10/03 10:53:48 gg502 Exp $
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#include "stdio.fh"
#include "denmat_iostuff.fh"
#include "denmat_funcs.fh"
#include "mafdecls.h"
C
C LOCAL VARIABLES
      Integer S_SP_Hdl, S_SP_Ind, S_SP_Size
      Logical LResult
C
C     This is the size of the symmetric packed matrix
C
      S_SP_Size = N * (N+1) / 2
C
C     Allocate space to read the symmetric packed metric
C
      LResult = MA_Push_Get( MT_Dbl, S_SP_Size,
     $   'Temp. for Sym. packed metric', S_SP_Hdl, S_SP_Ind)
C
      If ( .NOT. LResult ) then
         Write (LuErr, *) ' Get_AO_Metric: Unable to allocate ',
     $      ' scratch space on the stack.'
         Get_AO_Metric = .FALSE.
         Return
      EndIf
C
      Call DGetArray( ICk, Dbl_MB( S_SP_Ind), S_SP_Size, 350)
C
C     Convert this beast to full general storage
C
      Call DSP2GE( 'U', N, DBl_MB( S_SP_Ind ), S, LDS)
C
C     Free the scratch space
C
      LResult = MA_Pop_Stack( S_SP_Hdl)
C
      If ( .NOT. LResult ) then
         Write (LuErr, *) ' Get_AO_Metric: Unable to free ',
     $      ' scratch space on the stack.'
         Get_AO_Metric = .FALSE.
         Return
      EndIf
C
C     All work completed
C
      Get_AO_Metric = .TRUE.
      Return
      End
