C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C NAME
C     Get_MO_Coefficients -- Extract the MO coefficient matrix from the
C     run-time database.
C
C SYNOPSIS
      Logical Function Get_MO_Coefficients(NAO, NMO, C, LDC)
      Implicit NONE
      Integer NAO, NMO, LDC
      Double Precision C(LDC, NMO)
C
C ARGUMENTS
C     NAO     Row dimension of MO coefficients (IN)
C     NMO     Column dimension of MO coefficients (IN)
C     C       MO coefficient matrix (OUT)
C     LDC     Leading dimension of C (IN)
C
C DESCRIPTION
C     Reads MO coefficients from the run-time database.  At this time,
C     this amounts to a GETARRAY call to the pdft10 file.
C
C MEMORY USAGE
C     If NAO != LDC, uses NAO * NMO MT_DBLs on the stack.
C
C REVISION
C $Id: get_mo_coefficients.F,v 1.3 1993/10/03 10:53:49 gg502 Exp $
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#include "stdio.fh"
#include "denmat_iostuff.fh"
#include "denmat_funcs.fh"
#include "mafdecls.h"
C
C LOCAL VARIABLES
      Logical LResult
      Integer Scr_Hdl
      Integer Scr_Ind
C
C     If the target matrix C is the same size as the object in the
C     database, there is no need to use scratch space to get it.
C
      If ( NAO .eq. LDC) then
         Call DGetArray( ICk, C, NAO*NMO, 359)
      Else
C
C        Allocate space to read the matrix
C
         LResult = MA_Push_Get( MT_Dbl, NAO*NMO,
     $      'Temp. for MO coefficients', Scr_Hdl, Scr_Ind)
C
         If ( .NOT. LResult ) then
            Write (LuErr, *) ' Get_MO_Coefficients: Unable to ',
     $         ' allocate scratch space on the stack.'
            Get_MO_Coefficients = .FALSE.
            Return
         EndIf
C
         Call DGetArray( ICk, Dbl_MB( Scr_Ind), NAO*NMO, 359)
C        
C        Copy this matrix into the target
C
         Call DGECOP( 'N', NAO, NMO, Dbl_MB( Scr_Ind), NAO, C, LDC)
C
C        Free the scratch space
C
         LResult = MA_Pop_Stack( Scr_Hdl)
C
         If ( .NOT. LResult ) then
            Write (LuErr, *) ' Get_MO_Coefficients: Unable to free ',
     $         ' scratch space on the stack.'
            Get_MO_Coefficients = .FALSE.
            Return
         EndIf
      EndIf
C
C     All work completed
C
      Get_MO_Coefficients = .TRUE.
      Return
      End
