C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C NAME
C     setup -- Prepare for calculation, including actions that require
C     dynamic memory.
C
C SYNOPSIS
      Subroutine Setup(NAOBF, NMOBF, NOccMO, C_Hdl,
     $     F_MO_Hdl, F_AO_Hdl, S_Hdl, QProj_Hdl, SInv_Hdl, FQ_Hdl)
      Implicit NONE
      Integer NAOBF, NMOBF, NOccMO, C_Hdl,
     $     F_MO_Hdl, F_AO_Hdl, S_Hdl, QProj_Hdl, SInv_Hdl, FQ_Hdl
C
C ARGUMENTS
C     NAOBF     Number of AO basis functions (IN)
C     NMOBF     Number of MO basis functions (IN)
C     NOccMO    Number of occupied MOs (IN)
C     C_Hdl     Memhandle for SCF MO coefficients (OUT)
C     F_MO_Hdl  Memhandle for MO basis Fock matrix (OUT)
C     F_AO_Hdl  Memhandle for AO basis Fock matrix (OUT)
C     S_Hdl     Memhandle for AO overlap matrix (OUT)
C     QProj_Hdl Memhandle for projection operator which eliminates
C               contributions due to occupied orbitals (OUT)
C     SInv_Hdl  Memhandle for S^{-1} (OUT)
C     FQ_Hdl    Memhandle for AO basis Fock matrix projected with Q (OUT)
C
C DESCRIPTION
C     Prepare for E(2) energy calculation.  In this case, obtain or
C     construct the various O(NAOBF^2) quantities which will be used
C     as inputs (remaining unchanged) throughout the calculation.
C
C DYNAMIC MEMORY
C     7 NAOBF*NAOBF MT_DBL objects are allocated for the live of the
C     calculation (freed in WRAPUP).  
C
C     Up to 2 NAOBF*NAOBF + 2 NAOBF MT_DBLs are used as workspace
C     local to the routine.  Children may also use dynamic memory.
C
C REVISION
C     $Id: setup.F,v 1.2 1993/11/08 19:49:30 gg502 Exp $
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#include "denmat_iostuff.fh"
C These guys are included in integral_api.fh at the moment
C #include "mafdecls.h"
C #include "mafuncs.fh"
#include "stdio.fh"
#include "numerical_constants.fh"
#include "integral_api.fh"
#include "denmat_funcs.fh"
#include "precision.h"
C
C FUNCTIONS & SUBROUTINES
      Logical Get_AO_Fock_Matrix, Get_MO_Coefficients, Get_AO_Metric,
     $  Get_AO_Density_Matrix
      External Get_AO_Fock_Matrix, Get_MO_Coefficients, Get_AO_Metric,
     $  Get_AO_Density_Matrix
C
C LOCAL VARIABLES
      Integer ISeed, Info
      Integer        Z_Hdl,
     $     Metric_Hdl, Wrk1_Hdl, Wrk2_Hdl, D_Hdl
      Integer C_Ind, Z_Ind,
     $     Metric_Ind, Wrk1_Ind, Wrk2_Ind, D_Ind
      Integer F_MO_Ind, F_AO_Ind, S_Ind, QProj_Ind, SInv_Ind, FQ_Ind
      Logical LResult
C
C PARAMETERS
      Logical Canonical_Ref
      Parameter (Canonical_Ref = .TRUE.)
C
C     ***********************************************
C     * Allocate and read in SCF coefficient matrix *
C     ***********************************************
C
      LResult = MA_Push_Get( MT_Dbl, NAOBF*NMOBF, 'SCF MO Coefficients',
     $   C_Hdl, C_Ind)
C
      LResult = Get_MO_Coefficients( NAOBF, NMOBF, Dbl_MB( C_Ind),
     $   NAOBF)
C
      If ( .NOT. LResult) then
         Write (LuErr, *) ' Setup: Unable to get MO coefficients.'
         Stop
      Endif
C
C     ************************************************
C     * Decanonicalize SCF coefficients if necessary *
C     ************************************************
C     Make a random unitary transformation to decanonicalize the MOs.
C     This requires that the occupied-virtual blocks of the transformation
C     be zero.
C
C     Since the resulting orbitals have no physical significance, they
C     are likely to be relatively hard to converge.  But that makes 
C     them a good, general test, no?
C
      If ( .NOT. Canonical_Ref ) then
         LResult = MA_Push_Get( MT_DBL, NAOBF**2, 'decanonicalizer',
     $      D_Hdl, D_Ind)
C
C        Make random occupied-occupied and virtual-virtual blocks
C        separately to maintain Brilliuon condition.
C
         Call DGEZER( NAOBF, NAOBF, DBL_MB(D_Ind), NAOBF)
         ISeed = 0
         Call DGERAN( NOccMO, NOccMO, DBL_MB( D_Ind), NAOBF, ISeed)
         Call DGERAN( NAOBF-NOccMO, NAOBF-NOccMO,
     $      DBL_MB( D_Ind + NOccMO * (NAOBF+1) ), NAOBF, ISeed)
C
C        Now insure it is unitary.  Metric is the unit matrix.
C
         LResult = MA_Push_Get( MT_DBL, NAOBF, 'Work1',
     $      Wrk1_Hdl, Wrk1_Ind)
         LResult = MA_Push_Get( MT_DBL, NAOBF, 'Work2',
     $      Wrk2_Hdl, Wrk2_Ind)
         LResult = MA_Push_Get( MT_DBL, NAOBF * NAOBF, 'Metric',
     $      Metric_Hdl, Metric_Ind)
C
         Call DGEONE( NAOBF, NAOBF, DBL_MB(Metric_Ind), NAOBF)
         Call OrthV2( NAOBF, NAOBF, DBL_MB(D_Ind), DBL_MB(Metric_Ind),
     $      DBL_MB( wrk1_Ind ), DBL_MB( wrk2_Ind ))
C
         LResult = MA_Pop_Stack( Metric_Hdl)
         LResult = MA_Pop_Stack( Wrk2_Hdl)
         LResult = MA_Pop_Stack( Wrk1_Hdl)
C
         LResult = MA_Push_Get( MT_DBL, NAOBF*NAOBF, 'Work1',
     $      Wrk1_Hdl, Wrk1_Ind)
C
         Call DGEMM('N', 'N', NAOBF, NAOBF, NAOBF,
     $      FP_One, Dbl_MB( C_Ind), NAOBF, Dbl_MB( D_Ind), NAOBF,
     $      FP_Zero, Dbl_MB( Wrk1_Ind), NAOBF)
         Call DGECOP('N', NAOBF, NAOBF, Dbl_MB(Wrk1_Ind), NAOBF,
     $      Dbl_MB(C_Ind), NAOBF)
C
         LResult = MA_Pop_Stack( Wrk1_Hdl)
         LResult = MA_Pop_Stack( D_Hdl)
C
      EndIf
C
C     ************************************************
C     * Allocate and read in AO basis overlap matrix *
C     ************************************************
C
      LResult = MA_Push_Get( MT_Dbl, NAOBF*NAOBF, 'AO overlap matrix',
     $   S_Hdl, S_Ind)
C
      LResult = Get_AO_Metric( NAOBF, Dbl_MB( S_Ind), NAOBF)
C
      If ( .NOT. LResult) then
         Write (LuErr, *) ' Setup: Unable to get AO overlap.'
         Stop
      Endif
C
C     *************
C     * Form S^-1 *
C     *************
C
      LResult = MA_Push_Get(MT_Dbl, NAOBF*NAOBF, 'S^-1',
     $   SInv_Hdl, SInv_Ind)
      LResult = MA_Push_Get(MT_Int, NAOBF, 'S^-1 scratch 1',
     $   Wrk1_Hdl, Wrk1_Ind)
      LResult = MA_Push_Get(MT_Dbl, NAOBF*NAOBF, 'S^-1 scratch 2',
     $   Wrk2_Hdl, Wrk2_Ind)
C
      Call DGECOP( 'NNNNNNNNNNNNN', NAOBF, NAOBF, Dbl_MB( S_Ind), NAOBF,
     $   Dbl_MB(SInv_Ind), NAOBF)
      Call DGETRF( NAOBF, NAOBF, Dbl_MB(SInv_Ind), NAOBF,
     $   Int_MB( Wrk1_Ind), Info)
      Call DGETRI(NAOBF, Dbl_MB(SInv_Ind), NAOBF, Int_MB(Wrk1_Ind),
     $   Dbl_MB(Wrk2_Ind), NAOBF*NAOBF, Info)
C
      LResult = MA_Pop_Stack( Wrk2_Hdl)
      LResult = MA_Pop_Stack( Wrk1_Hdl)
C
C     ********************************
C     * Get the AO basis Fock matrix *
C     ********************************
C
      If ( .NOT. MA_Push_Get( MT_DBL, NAOBF * NAOBF, 'AO Fock matrix',
     $    F_AO_Hdl, F_AO_Ind) ) then
         Write (LuErr, *) 'Failed to allocate ', NAOBF*NAOBF,
     $        ' doubles for AO Fock matrix.'
         Stop
      EndIf
C
      LResult = Get_AO_Fock_Matrix( NAOBF, Dbl_MB( F_AO_Ind), NAOBF)
C
      If ( .NOT. LResult) then
         Write (LuErr, *) ' Setup: Unable to get AO Fock matrix.'
         Stop
      Endif
C
C     ***********************************************
C     * Form MO basis Fock matrix from AO basis one *
C     ***********************************************
C
      LResult = MA_Push_Get( MT_Dbl, NMOBF*NMOBF, 'MO Fock matrix',
     $   F_MO_Hdl, F_MO_Ind)
C
      If ( .NOT. MA_Push_Get( MT_DBL, NMOBF * NMOBF, 'Scratch matrix',
     $     Z_Hdl, Z_Ind) ) then
         Write (LuErr, *) 'Failed to allocate ', NMOBF*NMOBF,
     $        ' doubles for a scratch matrix.'
         Stop
      EndIf
C
      Call DGEMM('T', 'N', NMOBF, NAOBF, NAOBF, FP_One,
     $     Dbl_MB( C_Ind ), NAOBF, Dbl_MB( F_AO_Ind), NAOBF,
     $     FP_Zero, Dbl_MB( Z_Ind ), NMOBF)
      Call DGEMM('N', 'N', NMOBF, NMOBF, NAOBF, FP_One,
     $     Dbl_MB( Z_Ind ), NMOBF, Dbl_MB( C_Ind), NAOBF,
     $     FP_Zero, Dbl_MB( F_MO_Ind ), NMOBF)
      LResult = MA_Pop_Stack( Z_Hdl )
C
C     *********************************************
C     * Form the projector for the external space *
C     *********************************************
C     Form AO density from MO coefficients (occupation number = 2 for RHF)
C
      LResult = MA_Push_Get( MT_Dbl, NAOBF*NAOBF, 'External projector',
     $   QProj_Hdl, QProj_Ind)
C
      Call DSYRK('L', 'N', NAOBF, NOccMO, FP_Two, Dbl_MB( C_Ind), NAOBF,
     $   FP_Zero, Dbl_MB( QProj_Ind), NAOBF)
      Call DSY2GE('L', NAOBF, Dbl_MB(QProj_Ind), NAOBF)
C
C     Wrk1 <-- SD
C
      LResult = MA_Push_Get( MT_Dbl, NAOBF*NAOBF, 'scratch 1',
     $   Wrk1_Hdl, Wrk1_Ind)
      Call DGEMM('N', 'N', NAOBF, NAOBF, NAOBF,
     $   FP_One, Dbl_MB(S_Ind), NAOBF, Dbl_MB(QProj_Ind), NAOBF,
     $   FP_Zero, Dbl_MB(Wrk1_Ind), NAOBF)
C
C     QProj <-- 1 - 1/2 SD
C
      Call DGEONE(NAOBF, NAOBF, Dbl_MB(QProj_Ind), NAOBF)
      Call DGEMA('N', NAOBF, NAOBF,
     $   -FP_Half, Dbl_MB(Wrk1_Ind), NAOBF,
     $   FP_One, Dbl_MB(QProj_Ind), NAOBF)
C
      LResult = MA_Pop_Stack( Wrk1_Hdl)
C
C     ******************************************************
C     * Form projected Fock matrix needed for denominators *
C     ******************************************************
C
      LResult = MA_Push_Get( MT_Dbl, NAOBF*NAOBF,
     $   'Projected Fock matrix', FQ_Hdl, FQ_Ind)
      LResult = MA_Push_Get( MT_Dbl, NAOBF*NAOBF, 'scratch 1',
     $   Wrk1_Hdl, Wrk1_Ind)
C
      Call DGEMM( 'N', 'N', NAOBF, NAOBF, NAOBF,
     $   FP_One, Dbl_MB( QProj_Ind), NAOBF, Dbl_MB( F_AO_Ind), NAOBF,
     $   FP_Zero, Dbl_MB(Wrk1_Ind), NAOBF)
      Call DGEMM( 'N', 'T', NAOBF, NAOBF, NAOBF,
     $   FP_One, Dbl_MB( Wrk1_Ind), NAOBF, Dbl_MB( QProj_Ind), NAOBF,
     $   FP_Zero, Dbl_MB(FQ_Ind), NAOBF)
C
      LResult = MA_Pop_Stack( Wrk1_Hdl)
C
      Write (LuOut, *) 'Done with Setup'
C
      Return
      End

