C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C NAME
C     Par3_GenE2 -- compute E(2) energy
C
C SYNOPSIS
      Double Precision Function Par3_GenE2( KOp_Hdl, T2_Hdl,
     $   NExt, NInt, FPMO, FPAO)
      Implicit NONE
      Integer NExt, NInt, KOp_Hdl, T2_Hdl
      Double Precision FPMO, FPAO
C
C ARGUMENTS
C     KOp_Hdl GA handle for exchange operators (IN)
C     T2_Hdl  GA handle for double excitation amplitudes (IN)
C     NExt    Dimension of "external" space (IN)
C     NInt    Dimension of "internal" space (IN)
C     FPMO    Numer of floating point multiply operations (OUT)
C     FPAO    Numer of floating point add operations (OUT)
C 
C DESCRIPTION
C     Compute the contraction of the exchange operators with the
C     double excitation amplitudes.  If T2 are the converged MBPT(2)
C     amplitudes, this is the MBPT(2) energy.
C
C     This operation costs NExt*NExt*NInt*(NInt+1)/2 FP * and +
C     operations.
C
C REVISION
C     $Id: par3_gene2.F,v 1.1 1993/11/08 19:50:17 gg502 Exp $
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#include "numerical_constants.fh"
#include "mafdecls.h"
#include "stdio.fh"
#ifdef TCGMSG
#include "tcgmsg.h"
#include "msgids.fh"
#endif /* TCGMSG */
C FUNCTIONS
      Double precision DDOT
      External DDOT
      Logical Verify_Locality
      External Verify Locality
C
C LOCAL VARIABLES
      Double Precision EIJ
      Integer I, J, IJ
      Integer LclK_Hdl, LclK_Ind, LclT_Hdl, LclT_Ind
      Logical LResult
      Integer ExtLo, ExtHi, IJLo, IJHi, Junk
C
C     *******************************************
C     * Computational cost is simple to compute *
C     *******************************************
C
      FPMO = NExt*NExt*NInt*(NInt+1)/2
      FPAO = NExt*NExt*NInt*(NInt+1)/2
C
C     ********************************
C     * Local buffers for KOp and T2 *
C     ********************************
C
      LResult = MA_Push_Get( MT_Dbl, NExt*NExt, 'Local KOp buffer',
     $   LclK_Hdl, LclK_Ind)
      LResult = MA_Push_Get( MT_Dbl, NExt*NExt, 'Local T2 buffer',
     $   LclT_Hdl, LclT_Ind)
C
C     *************************
C     * Accumulate the energy *
C     *************************
C
      Par3_GenE2 = FP_Zero
C
C
C     ********************************************
C     * Work on the step elements that are local * 
C     ********************************************
C
      Call GA_Distribution( KOp_Hdl, NodeID(), ExtLo, ExtHi,
     $   IJLo, IJHi, Junk)
C
C     Check that T2 conforms
C
      If ( .NOT. Verify_Locality( T2_Hdl, ExtLo, ExtHi, IJLo, IJHi) )
     $   Write (LuErr, 9000) 'T2', 'KOp',  NodeID()
C
 9000 Format( 1X, 'Par3_Next_Amp: WARNING: Distribution of global ',
     $   'array ', A, ' does not match ', A, 'on process ', I5, '.')
C
      Do IJ = IJLo, IJHi
         Call Detriangularize(IJ, I, J)
C
C        Get local copy of KOp and T2
C
         Call GA_Get( KOp_Hdl, 1, NExt*NExt, IJ, IJ,
     $      Dbl_MB( LclK_Ind), NExt*NExt)
         Call GA_Get( T2_Hdl, 1, NExt*NExt, IJ, IJ,
     $      Dbl_MB( LclT_Ind), NExt*NExt)
C
C        Now compute energy
C
         EIJ = DDOT( NExt*NExt, Dbl_MB(LclT_Ind), 1,
     $      Dbl_MB(LclK_Ind), 1)
C
C        Factor of 2 because we're doing I>=J only
C
         Par3_GenE2 = Par3_GenE2 +  2 * EIJ
 1010    Continue
C
C        Over counted on the (I, I) contribution, so back it off
C
         If ( I .eq. J) Par3_GenE2 = Par3_GenE2 -  EIJ
C
      EndDo ! IJ
C
      LResult = MA_Pop_Stack( LclT_Hdl)
      LResult = MA_Pop_Stack( LclK_Hdl)
C
C     *****************************
C     * Collect everyone's answer *
C     *****************************
C
      Call DGOp(MSG_E2sum, Par3_GenE2, 1, '+')
C
      Return
      End
