C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C NAME
C     verify_locality -- Check that the local elements of a global array
C     match the desired pattern
C
C SYNOPSIS
      Logical Function Verify_Locality( Handle, Lo1, Hi1, Lo2, Hi2)
      Implicit NONE
      Integer Handle, Lo1, Hi1, Lo2, Hi2
C
C ARGUMENTS
C     Handle  GA handle for object to check (IN)
C     Lo1     Beginning of desired range of first index (IN)
C     Hi1     End of desired range of first index (IN)
C     Lo2     Beginning of desired range of second index (IN)
C     Hi2     End of desired range of second index (IN)
C
C DESCRIPTION
C     Returns .TRUE. if the array elements Handle(Lo1:Hi1, Lo2:Hi2)
C     are local to the node.  Returns .FALSE. otherwise.
C
C     Intended to help insure that data distributions match up
C     as desired.
C
C REVISION
C     $Id: verify_locality.F,v 1.3 1993/11/08 19:50:34 gg502 Exp $
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#ifdef TCGMSG
#include "global.h"
#include "tcgmsg.h"
#endif /* TCGMSG */
C LOCAL VARIABLES
      Integer MemHandle
      Integer MyLo1, MyHi1, MyLo2, MyHi2
C
      Call GA_Distribution( Handle, GA_NodeID(), MyLo1, MyHi1,
     $   MyLo2, MyHi2, MemHandle)
C
C     We are happy if the range we want, (Lo1:Hi1, Lo2:Hi2),
C     is inside the range on this node, (MyLo1:MyHi1, MyLo2:MyHi2).
C     Of course if they don't match exactly, _some_ process is
C     likely to be unhappy.
C
      If (    MyLo1 .gt. Lo1 .OR. MyHi1 .lt. Hi1
     $   .OR. MyLo2 .gt. Lo2 .OR. MyHi2 .lt. Hi2 ) then
         Verify_Locality = .FALSE.
      Else
         Verify_Locality = .TRUE.
      EndIf
C
      Return
      End
