C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C NAME
C     Toy1_MixedT2 -- compute second order T2 from mixed MO/AO quantities
C
C SYNOPSIS
      Subroutine Toy1_MixedT2( MOInt, T2, T2New, F, FAO, S, Scr, Z1, Z2,
     $   QProj, FExt, C, SInv, NOcc, NBF)
      Implicit NONE
      Integer NOcc, NBF
      Double Precision MOInt(NBF, NBF, NBF, NBF),
     $   T2( NBF, NBF, NBF, NBF), T2New(NBF, NBF, NBF, NBF),
     $   F(NBF, NBF), FAO(NBF, NBF), S(NBF, NBF), Scr(NBF, NBF),
     $   Z1(NBF, NBF), Z2(NBF, NBF), QProj(NBF, NBF), FExt(NBF, NBF),
     $   C(NBF, NBF), SInv(NBF, NBF)
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#include "numerical_constants.fh"
#include "stdio.fh"
#include "precision.h"
#include "mafdecls.h"
C FUNCTIONS
      Double Precision DDOT, DNRM2, Toy1_GenE2
      External DDOT, DNRM2, Toy1_GenE2
C
C LOCAL VARIABLES
      Double Precision Den, Diff, E2
      Integer I, J, A, B, M, IErr, NVir, Iter
C
      NVir = NBF - NOcc
      Iter = 1
C
      Call DGEWR( NBF, NBF, F, NBF, 0, 0, LuOut, 'o-o Fock matrix',
     $   80, 'E13.6', IErr)
C
      Call DGEWR( NBF, NBF, FAO, NBF, 0, 0, LuOut, 'v-v Fock matrix',
     $   80, 'E13.6', IErr)
C
      Call DGEWR( NBF, NBF, FExt, NBF, 0, 0, LuOut,
     $   'Projected Fock matrix',
     $   80, 'E13.6', IErr)
C
      Call DGEWR( NBF, NBF, QProj, NBF, 0, 0, LuOut,
     $   'Projector',
     $   80, 'E13.6', IErr)
C
c$$$      Call DGEWR( NBF, NBF, S, NBF, 0, 0, LuOut, 'Overlap matrix',
c$$$     $   80, 'E13.6', IErr)
c$$$C
      Call DVZER( NBF**4, T2, 1)
      Call DVZER( NBF**4, T2New, 1)
C
 8000 Diff = FP_Zero
C
      Call DCOPY( NBF**4, T2New, 1, T2, 1)
C
      Do 1000 I = 1, NOcc
         Do 1010 J = 1, NOcc
C
            Call DVZER(NBF*NBF, Scr, 1)
C
            Do 1020 A = 1, NBF
               Do 1030 B = 1, NBF
C
C                 Start with the integrals (AA+BB + AB)
C
                  Scr(A, B) = 
     $                 ( MOInt(I, A, J, B) - MOInt(I, B, J, A))
     $                 + MOInt(I, A, J, B)
 1030          Continue
 1020       Continue
c$$$            Write (6, *) 'Scr ', I, J, ' starts as ',
c$$$     $           DDOT(NBF**2, Scr, 1, Scr, 1)
c$$$            If ( DDOT(NBF**2, Scr, 1, Scr, 1) .gt. 0.10d0)
c$$$     $         Call DGEWR( NBF, NBF, Scr, NBF, 0, 0, LuOut,
c$$$     $         'Scratch matrix', 80, 'E13.6', IErr)
C
C           Now contract T with F and accumulate in Scr
C
C           Scr <-- + F(ae) T(ij,ef) S(fb)
C
            Call DGEMM( 'N', 'N', NBF, NBF, NBF, FP_One,
     $           FAO(1, 1), NBF, T2(1, 1, I, J), NBF, FP_Zero, Z1, NBF)
            Call DGEMM( 'N', 'N', NBF, NBF, NBF, FP_One,
     $           Z1, NBF, S, NBF, FP_One, Scr, NBF)
c$$$            Write (6, *) 'Scr ', I, J, ' step 1 is ',
c$$$     $           DDOT(NBF**2, Scr, 1, Scr, 1)
c$$$            If ( DDOT(NBF**2, Scr, 1, Scr, 1) .gt. 0.10d0)
c$$$     $         Call DGEWR( NBF, NBF, Scr, NBF, 0, 0, LuOut,
c$$$     $         'Scratch matrix', 80, 'E13.6', IErr)
C
C           Scr <-- + S(af) T(ij,fe) F(eb)
C
            Call DGEMM( 'N', 'N', NBF, NBF, NBF, FP_One,
     $           S, NBF, T2(1, 1, I, J), NBF, FP_Zero, Z1, NBF)
            Call DGEMM( 'N', 'N', NBF, NBF, NBF, FP_One,
     $         Z1, NBF, FAO, NBF, FP_One, Scr, NBF)
c$$$            Write (6, *) 'Scr ', I, J, ' step 2 is ',
c$$$     $           DDOT(NBF**2, Scr, 1, Scr, 1)
c$$$            If ( DDOT(NBF**2, Scr, 1, Scr, 1) .gt. 0.1d0)
c$$$     $         Call DGEWR( NBF, NBF, Scr, NBF, 0, 0, LuOut,
c$$$     $         'Scratch matrix', 80, 'E13.6', IErr)
C
C           For occupied contraction, we need another loop
C
            Call DGEZER(NBF, NBF, Z2, NBF)
C
            If ( I .ge. J) then
               Do 2000 M = 1, NOcc
c$$$              Write (6, *) 'A ', I, M, ' --> ', M, J
c$$$              Write (6, *) 'B ', M, J, ' --> ', I, M
C
C                 Z2 <-- F(im) T(mj,ab)
C
                  Call DGEMA('N', NBF, NBF,
     $               F(I, M), T2(1, 1, M, J), NBF,
     $               FP_One, Z2, NBF)
C
C                 Z2 <-- T(im,ab) F(mj)
C
                  Call DGEMA('N', NBF, NBF,
     $               F(M, J), T2(1, 1, I, M), NBF,
     $               FP_One, Z2, NBF)
C
 2000          Continue
            EndIf
            If (I .gt. J) then
               Do 2010 M = 1, NOcc
c$$$              Write (6, *) 'A ', I, M, ' --> ', M, J
c$$$              Write (6, *) 'B ', M, J, ' --> ', I, M
C
C                 Z2 <-- F(im) T(mj,ab)
C
                  Call DGEMA('N', NBF, NBF,
     $               F(J, M), T2(1, 1, M, I), NBF,
     $               FP_One, Z2, NBF)
C
C                 Z2 <-- T(im,ab) F(mj)
C
                  Call DGEMA('N', NBF, NBF,
     $               F(M, I), T2(1, 1, J, M), NBF,
     $               FP_One, Z2, NBF)
C
 2010          Continue
            EndIf
C
C           Scr <-- - S(ae) [ F(im) T(mj,ef) + T(im,ef) F(mj) ] S(fb)
C                             \------------- Z2 ------------/
C
            Call DGEMM( 'N', 'N', NBF, NBF, NBF,
     $         FP_One, S, NBF, Z2, NBF, FP_Zero, Z1, NBF)
            Call DGEMM( 'N', 'N', NBF, NBF, NBF,
     $         -FP_One, Z1, NBF, S, NBF, FP_One, Scr, NBF)
C
c$$$            Write (6, *) 'Scr ', I, J, ' step 3 is ',
c$$$     $           DDOT(NBF**2, Scr, 1, Scr, 1)
c$$$            If (Iter .lt. 2) Call DGEWR( NBF, NBF, Scr, NBF, 0, 0,
c$$$     $         LuOut, 'Scratch matrix', 80, 'E13.6', IErr)
C
C           Transform Scr to AO basis [ S r_{ij} S --> r_{ij} ]
C
            Call DGEMM( 'N', 'N', NBF, NBF, NBF,
     $         FP_One, SInv, NBF, Scr, NBF,
     $         FP_Zero, Z1, NBF)
            Call DGEMM( 'N', 'N', NBF, NBF, NBF,
     $         FP_One, Z1, NBF, SInv, NBF,
     $         FP_Zero, Scr, NBF)
c$$$            Write (6, *) 'Scr ', I, J, ' step 3.1 is ',
c$$$     $           DDOT(NBF**2, Scr, 1, Scr, 1)
C
C           Project into the external-only basis
C
            Call DGEMM( 'T', 'N', NBF, NBF, NBF,
     $         FP_One, QProj, NBF, Scr, NBF,
     $         FP_Zero, Z1, NBF)
            Call DGEMM( 'N', 'N', NBF, NBF, NBF,
     $         FP_One, Z1, NBF, QProj, NBF,
     $         FP_Zero, Scr, NBF)
c$$$C
c$$$C           Now transform to MO basis
c$$$C
c$$$            Call DGEMM( 'N', 'N', NBF, NBF, NBF,
c$$$     $         FP_One, S, NBF, Scr, NBF, FP_Zero, Z1, NBF)
c$$$            Call DGEMM( 'N', 'N', NBF, NBF, NBF,
c$$$     $         FP_One, Z1, NBF, S, NBF, FP_Zero, Scr, NBF)
c$$$C
c$$$            Call DGEMM( 'T', 'N', NBF, NBF, NBF,
c$$$     $         FP_One, C, NBF, Scr, NBF, FP_Zero, Z1, NBF)
c$$$            Call DGEMM( 'N', 'N', NBF, NBF, NBF,
c$$$     $         FP_One, Z1, NBF, C, NBF, FP_Zero, Scr, NBF)
C
c$$$            Write (6, *) 'Scr ', I, J, ' step 3.5 is ',
c$$$     $           DDOT(NBF**2, Scr, 1, Scr, 1)
c$$$            If ( Iter .lt. 2) Call DGEWR( NBF, NBF, Scr, NBF, 0, 0,
c$$$     $         LuOut, 'Scratch matrix', 80, 'E13.6', IErr)
c$$$            If ( DDOT(NBF**2, Scr, 1, Scr, 1) .gt. 0.10d0)
c$$$     $         Call DGEWR( NBF, NBF, Scr, NBF, 0, 0, LuOut,
c$$$     $         'Scratch matrix', 80, 'E13.6', IErr)
C
C           Apply denominators
C
            Do 3000 A = 1, NBF
               Do 3010 B = 1, NBF
C
c$$$                  Den = F(I, I) + F(J, J)
c$$$     $                 - F(A, A) - F(B, B)
c$$$                  Den = F(I, I) + F(J, J)
c$$$     $                 - FAO(A, A) - FAO(B, B)
                  Den = F(I, I) + F(J, J)
     $                 - FExt(A, A) - FExt(B, B)
C
c$$$                  Write (6, *) 'Den ', I, J, A, B, Den
                  If ( Scr(A, B) .ne. FP_Zero ) then
                     Scr(A, B) = Scr(A, B) / (2* Den)
                  EndIf
C
 3010          Continue
 3000       Continue
C
c$$$            Write (6, *) 'Scr ', I, J, ' step 3.6 is ',
c$$$     $           DDOT(NBF**2, Scr, 1, Scr, 1)
c$$$            If ( Iter .lt. 2) Call DGEWR( NBF, NBF, Scr, NBF, 0, 0,
c$$$     $         LuOut, 'Scratch matrix', 80, 'E13.6', IErr)
C
C           Add contribution to amplitudes
C
            Call DGEMA('N', NBF, NBF,
     $         FP_One, Scr, NBF,
     $         FP_Zero, T2New(1, 1, I, J), NBF)
            Call DGEMA('N', NBF, NBF,
     $         FP_One, T2(1, 1, I, J), NBF,
     $         FP_One, T2New(1, 1, I, J), NBF)
C
C           Project amplitudes to insure orthogonality to occupied space
C
            Call DGEMM( 'T', 'N', NBF, NBF, NBF,
     $         FP_One, QProj, NBF, T2New(1, 1, I, J), NBF,
     $         FP_Zero, Z1, NBF)
            Call DGEMM( 'N', 'N', NBF, NBF, NBF,
     $         FP_One, Z1, NBF, QProj, NBF,
     $         FP_Zero, T2New(1, 1, I, J), NBF)
C
C           
C
 1010    Continue
 1000 Continue
C
      Call DAXPY( NBF**4, -FP_One, T2New, 1, T2, 1)
      Diff = DNRM2( NBF**4, T2, 1)**2
C
C     Compute energy based on current amplitudes
C
      E2 = Toy1_GenE2( MOInt, T2New, 1, NBF, 1, NBF, 1, NOcc, 1, NOcc,
     $   NBF)
C
      Write (LuOut, *) ' Iteration ', Iter, ' Change is ', Sqrt( Diff ),
     $   ' Energy is ', E2
      Call Flush( LuOut)
      If ( Sqrt( Diff ) .gt. 1.0d-6) then
         Iter = Iter + 1
         Goto 8000
      EndIf
C
      Write (LuOut, *) 'T2 converged after ', Iter, ' iterations.'
      Call DCOPY( NBF**4, T2New, 1, T2, 1)
C
      Return
      End
