C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C NAME
C     Toy1_ncMOT2 -- compute second order T2 from noncanonical HF MOs
C
C SYNOPSIS
      Subroutine Toy1_ncMOT2( MOInt, T2, F, FDiag, Scr, NOcc, NBF)
      Implicit NONE
      Integer NOcc, NBF
      Double Precision MOInt(NBF, NBF, NBF, NBF),
     $     T2( NBF, NBF, NBF, NBF), F(NBF, NBF),
     $     FDiag(NBF), Scr(NOcc+1:NBF, NOcc+1:NBF)
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#include "numerical_constants.fh"
#include "stdio.fh"
#include "precision.h"
C FUNCTIONS
      Double Precision DDOT, DNRM2
      External DDOT, DNRM2
C
C LOCAL VARIABLES
      Double Precision Den, OldT, Diff
      Integer I, J, A, B, M, IErr, NVir, Iter
C
      NVir = NBF - NOcc
      Iter = 1
C
C     Get the diagonals from the Fock matrix
C
      Call DCOPY( NBF, F(1, 1), NBF+1, FDiag, 1)
C
C     And clear out the diagonal of the Fock matrix
C
      Call DVZER( NBF, F(1, 1), NBF+1)
C
c$$$C
c$$$      Call DGEWR( NBF, NBF, F, NBF, 0, 0, LuOut, 'Fock matrix', 80,
c$$$     $     'E13.6', IErr)
c$$$C
c$$$      Call DGEWR( 1, NBF, FDiag, 1, 0, 0, LuOut,
c$$$     $   'Fock matrix diagonals', 80, 'E13.6', IErr)
C
      Call DVZER( NBF**4, T2, 1)
C
 8000 Diff = FP_Zero
      Do 1000 I = 1, NOcc
         Do 1010 J = 1, NOcc
C
            Do 1020 A = NOcc + 1, NBF
               Do 1030 B = NOcc + 1, NBF
C
C                 Start with the integrals (AA+BB + AB)
C
                  Scr(A, B) = 
     $                 ( MOInt(I, A, J, B) - MOInt(I, B, J, A))
     $                 + MOInt(I, A, J, B)
 1030          Continue
 1020       Continue
c$$$            Write (6, *) 'Scr ', I, J, ' starts as ',
c$$$     $           DDOT(NVIR**2, Scr, 1, Scr, 1)
C
C           Now contract T with F and accumulate in Scr
C
C           Scr <-- + F(ae) T(ij,eb)
C
            Call DGEMM( 'N', 'N', NVir, NVir, NVir, FP_One,
     $           F(NOcc+1, NOcc+1), NBF,
     $           T2(NOcc+1, NOcc+1, I, J), NBF,
     $           FP_One, Scr, NVir)
c$$$            Write (6, *) 'Scr ', I, J, ' step 1 is ',
c$$$     $           DDOT(NVIR**2, Scr, 1, Scr, 1)
C
C           Scr <-- + T(ij,ae) F(eb)
C
            Call DGEMM( 'N', 'N', NVir, NVir, NVir, FP_One,
     $           T2(NOcc+1, NOcc+1, I, J), NBF,
     $           F(NOcc+1, NOcc+1), NBF,
     $           FP_One, Scr, NVir)
c$$$            Write (6, *) 'Scr ', I, J, ' step 2 is ',
c$$$     $           DDOT(NVIR**2, Scr, 1, Scr, 1)
C
C           For occupied contraction, we need another loop
C
            Do 2000 M = 1, NOcc
C
C              Scr <-- - F(im) T(mj,ab)
C
               Call DGEMA('N', NVir, NVir,
     $            -F(I, M), T2(NOcc+1, NOcc+1, M, J), NBF,
     $            FP_One, Scr, NVir)
C
C              Scr <-- - T(im,ab) F(mj)
C
               Call DGEMA('N', NVir, NVir,
     $            -F(M, J), T2(NOcc+1, NOcc+1, I, M), NBF,
     $            FP_One, Scr, NVir)
C
 2000       Continue
c$$$            Write (6, *) 'Scr ', I, J, ' step 3 is ',
c$$$     $           DDOT(NVIR**2, Scr, 1, Scr, 1)
C
C           Apply denominators and check convergence
C
            Do 3000 A = NOcc+1, NBF
               Do 3010 B = NOcc+1, NBF
C
                  OldT = T2(A, B, I, J)
C
                  Den = FDiag(I) + FDiag(J)
     $                 - FDiag(A) - FDiag(B)
C
                  T2(A, B, I, J) = Scr(A, B) / Den
                  Diff = Diff + (T2(A, B, I, J) - OldT)**2
C
 3010          Continue
 3000       Continue
C
C           
C
 1010    Continue
 1000 Continue
C
      Write (LuOut, *) ' Iteration ', Iter, ' Change is ', Sqrt( Diff )
      If ( Sqrt( Diff ) .gt. 1.0d-6) then
         Iter = Iter + 1
         Goto 8000
      EndIf
C
      Write (LuOut, *) 'T2 converged after ', Iter, ' iterations.'
C
C     Restore the diagonals to the Fock matrix
C
      Call DCOPY( NBF, FDiag, 1, F(1, 1), NBF+1)
C
      Return
      End
