C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C NAME
C     Toy2_Apply_Denom -- Apply denominators to residual
C
C SYNOPSIS
      Subroutine Toy2_Apply_Denom( NExt, NInt, R, FInt, LDFInt,
     $   FQ, LDFQ, Q, LDQ, Scratch, FPMO, FPAO)
      Implicit NONE
      Integer NInt, NExt, LDFInt, LDFQ, LDQ
      Double precision FPMO, FPAO
      Double Precision R(NExt, NExt, NInt*(NInt+1)/2),
     $   FInt(LDFInt, NInt), FQ(LDFQ, NExt), Q(LDQ, NExt), 
     $   Scratch(NExt, NExt)
C
C ARGUMENTS
C     NExt    Dimension of external space (IN)
C     NInt    Dimension of internal (occupied) space (IN)
C     R       Residual of t amplitude equation, in external basis (INOUT)
C     FInt    Fock matrix in internal basis (IN)
C     LDFInt  Leading dimension of FInt (IN)
C     FQ      Fock matrix in external basis PROJECTED WITH Q (IN)
C     LDFQ    Leading dimension of FQ (IN)
C     Q       Projection operator to remove contributions due to
C             internal orbtials form an object in the external basis (IN)
C     LDQ     Leading dimension of Q (IN)
C     Scratch Scratch matrix (Scratch)
C     FPMO    Number of FP multiply operations (OUT)
C     FPAO    Number of FP add operations (OUT)
C
C DESCRIPTION
C     Apply denominators to the residual of the amplitude equation
C     in order to form and update for the next iteration of the
C     equations.
C
C     The present scheme uses FQ, which is the external basis Fock
C     matrix with all of the internal orbital contributions projected
C     out, FQ = Q F Q^T.  This, in combination with the initial projection
C     of the residuals, Q^T r_{ij} Q, seems to work.  If both projections
C     are skipped, then the denominators can be small and cause problems,
C     especially if some of the internal orbitals are dominated by a
C     single AO.
C
C     The factor of 2 in the denominator is also empirical -- a damping
C     factor. Obviously more study will be needed on the best  way to
C     form these updates!
C
C     This routine requires O( 2 NExt^3 NInt(NInt+1)/2 ) FP operations
C     because of the projection.  The denominators themselves are
C     cheap, O( NExt^2 NInt(NInt-1)/2 ).  Every IJ is independent in
C     this routine.
C
C BUGS
C     FP divisions (in applying the denominators) are counted as
C     multiplications.
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#include "numerical_constants.fh"
#include "precision.h"
C
C LOCAL VARIABLES
      Double Precision Den, DenIJ
      Integer I, J, A, B, IJ
C
      FPMO = FP_Zero
      FPAO = FP_Zero
C
      Do I = 1, NInt
         Do J = 1, I
            IJ = I * (I-1)/2 + J
C
C           ****************************************************
C           * Project out contributions from internal orbitals *
C           ****************************************************
C
            FPMO = FPMO + 2 * NExt * NExt * NExt
            FPAO = FPAO + 2 * NExt * NExt * NExt
C
            Call DGEMM( 'T', 'N', NExt, NExt, NExt,
     $         FP_One, Q, LDQ, R(1, 1, IJ), NExt,
     $         FP_Zero, Scratch, NExt)
            Call DGEMM( 'N', 'N', NExt, NExt, NExt,
     $         FP_One, Scratch, NExt, Q, LDQ,
     $         FP_Zero, R(1, 1, IJ), NExt)
C
C           **********************
C           * Apply denominators * 
C           **********************
C
            FPMO = FPMO + NExt * NExt
            FPAO = FPAO + 1 + 2 * NExt * NExt
C
            DenIJ = FInt(I, I) + FInt(J, J)
C
            Do A = 1, NExt
               Do B = 1, NExt
C
                  Den = DenIJ - FQ(A, A) - FQ(B, B)
C
                  R(A, B, IJ) = R(A, B, IJ) / Den
C
               EndDo   ! A
            EndDo      ! B
         EndDo   ! J
      EndDo      ! I
C
      Return
      End
