C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C NAME
C     Toy2_GenE2 -- compute E(2) energy
C
C SYNOPSIS
      Double Precision Function Toy2_GenE2( KOp, T2,
     $   NExt, NInt, FPMO, FPAO)
      Implicit NONE
      Integer NExt, NInt
      Double Precision FPMO, FPAO
      Double Precision KOp(NExt, NExt, NInt*(NInt+1)/2),
     $     T2(NExt, NExt, NInt*(NInt+1)/2)
C
C ARGUMENTS
C     KOp     Exchange operators (IN)
C     T2      Double excitation amplitudes (IN)
C     NExt    Dimension of "external" space (IN)
C     NInt    Dimension of "internal" space (IN)
C     FPMO    Numer of floating point multiply operations (OUT)
C     FPAO    Numer of floating point add operations (OUT)
C 
C DESCRIPTION
C     Compute the contraction of the exchange operators with the
C     double excitation amplitudes.  If T2 are the converged MBPT(2)
C     amplitudes, this is the MBPT(2) energy.
C
C     This operation costs NExt*NExt*NInt*(NInt+1)/2 FP * and +
C     operations.
C
C REVISION
C     $Id: toy2_gene2.F,v 1.1 1993/08/30 21:25:59 gg502 Exp $
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#include "numerical_constants.fh"
C FUNCTIONS
      Double precision DDOT
      External DDOT
C
C LOCAL VARIABLES
      Double Precision EIJ
      Integer I, J, IJ
C
C     *******************************************
C     * Computational cost is simple to compute *
C     *******************************************
C
      FPMO = NExt*NExt*NInt*(NInt+1)/2
      FPAO = NExt*NExt*NInt*(NInt+1)/2
C
C     *************************
C     * Accumulate the energy *
C     *************************
C
      Toy2_GenE2 = FP_Zero
C
      Do 1000 I = 1, NInt
         Do 1010 J = 1, I
            IJ = I * (I-1)/2 + J
C
            EIJ = DDOT( NExt*NExt, T2(1, 1, IJ), 1,
     $         KOp(1, 1, IJ), 1)
C
C           Factor of 2 because we're doing I>=J only
C
            Toy2_GenE2 = Toy2_GenE2 +  2 * EIJ
 1010    Continue
C
C        Over counted on the (I, I) contribution, so back it off
C
         Toy2_GenE2 = Toy2_GenE2 -  EIJ
C
 1000 Continue
      Return
      End
